<?php

/*

Title:			Gallery Comment Cruncher
Purpose:		A general comment management system for Gallery.

Version: 		1.00
Release Date:	July 8, 2009
Author:			Russ Tanner

Home Page:		http://urlbam.com/ha/M002u
License:		http://urlbam.com/ha/M002r
Download:		http://urlbam.com/ha/M002t
Report Bugs:	http://urlbam.com/ha/M002m

Copyright:		All Rights Reserved by Russ Tanner

*/

// PHP Settings //

// Don't display sensitive information on the screen
ini_set('display_errors', 0);
ini_set('log_errors', 0);


// Constants //

// Status Constants

define('STATUS_OK', 0);								// No error
define('STATUS_InvalidListName', 1);				// The specifield list name is invalid
define('STATUS_ErrorReadingListFile', 2);			// An error occured reading a list file
define('STATUS_ErrorWritingListFile', 3);			// An error occured writing a list file
define('STATUS_InvalidDataSet', 4);					// The specified data set is invalid
define('STATUS_ErrorSortingList', 5);				// An error occured sorting a list
define('STATUS_DatabaseParameterMissing', 6);		// A necessary database parameter was not specified
define('STATUS_ErrorConnectingToDatabase', 7);		// We were unable to connect to the database
define('STATUS_ErrorSelectingDatabase', 8);			// An error occured selecting the database
define('STATUS_ErrorQueryingDatabase', 9);			// An error occured querying the database
define('STATUS_ErrorDetectingTablePrefix', 10);		// An error occured detecting the table prefix
define('STATUS_ErrorDetectingColumnPrefix', 11);	// An error occured detecting the column prefix
define('STATUS_ErrorReadingBatchFile', 12);			// An error occured reading a batch file
define('STATUS_ErrorWritingBatchFile', 13);			// An error occured writing a batch file
define('STATUS_ErrorReadingFile', 14);				// An error occured reading a file
define('STATUS_ErrorWritingFile', 15);				// An error occured writing a file
define('STATUS_ErrorReadingBatchControlFile', 16);	// An error occured reading the batch control file
define('STATUS_InvalidIndex', 17);					// The specified index is invalid.
define('STATUS_ListIsEmpty', 18);					// The specified list file contains no entries.
define('STATUS_InvalidParameter', 19);				// A specified parameter is invalid.
define('STATUS_ParameterMissing', 20);				// A required parameter is missing.

// Line Types
// These describe the types of lines in a file.
define('LINETYPE_Value', 0);						// The line is a value
define('LINETYPE_Comment', 1);						// The line is a comment
define('LINETYPE_Blank', 2);						// The line is blank or contains only whitespace



// Global Configuration Object //

class cfg {

	// Development Mode?
	// If true, templates are fetched from disk, otherwise, they are inline.
	public static $DevMode					= false;

	// Quote Style
	// When HTML quoting occurs, what PHP quoting style do we use?
	public static $QuoteStyle 				= ENT_QUOTES;

	// Character Set
	// What character set is this application displaying in?
	public static $CharSet					= 'UTF-8';

	// Date Format
	// How do we want to display dates?
	// This string is passed to the PHP date() function's "format" parameter.
	public static $DateFormat				= 'D, d M Y H:i:s';

	// Log Filename
	// This is the name of a file that is used for logging during debugging.
	public static $Log_FileName				= 'cc.log';

	// Buffer Filename
	// This is the name of a file used in readfile() operations.
	// This is used as a workaround to avoid PHP out-of-memory errors when displaying large pages.
	public static $Buffer_FileName			= 'cc-buffer.txt';

	// Temp PHP File
	// This is a temporary PHP file that is built dynamically and redirected to in some handlers.
	// This is normally done as a workaround to avoid PHP out-of-memory errors when displaying large pages.
	public static $PHPTemp_FileName 		= 'cc-tmp.php';

	// Default token values...

	// The HTML title of a page.
	public static $def_TITLE				= 'Gallery Comment Cruncher';
	// The displayed title of Bug Trap dialogs.
	public static $def_Title_BugTrap		= 'Bug Trap';
	// The displayed title of Error dialogs.
	public static $def_Title_Error			= 'Error';
	// The default SubmitDelay used in the Batch template (in milliseconds).
	// This is basically the amount of time you have to hit the "Submit" button before it disappears.
	public static $def_Batch_SubmitDelay	= '3000';

	// Form-Field Session Prefix
	// When form-field values are stored in the session, their keys are prefixed with this string.
	public static $SessionPrefix_FormFields	= 'ff_';

	// Global Tokens
	// This is a list of tokens that are available to all page templates.
	public static $GlobalTokens				= '';

	// Default Field Values
	// This is a list of default values for all form fields.
	public static $DefaultFieldValues		= '';

	// Persistent Fields
	// This is a list of form fields that are saved in the session, application-wide.
	public static $FieldsToPersist			= '';

	// Common Messages
	// This is a list of common messages.
	public static $Msgs						= '';

	// Maps a ListName to a MethodName.
	// This is used to determine which method is used to delete comments when a certain ListName is specified.
	public static $map_ListToMethod			= '';

	// This is used to determine which method is used when a column is specified.
	public static $map_ColumnToMethod		= '';

	// Maps a real Comment.column name to a friendly name.
	public static $map_ColumnToFriendly		= '';

	// Maps a column name to the correct template.
	// This is used in the FindDuplicates handler.
	public static $map_ColumnToTemplate		= '';

	// SQL Templates
	// This maps official "MethodNames" to SQL templates.
	public static $map_MethodToSQL			= '';

	// Maps common template names to their associated file names.
	public static $map_TemplateToFile		= '';

	// Maps common list names to their associated file names.
	public static $map_ListToFile			= '';

	// Maps status codes to standard error messages.
	public static $map_StatusToMsg			= '';
	
	// Maps status codes to a standard error message helpful-hints.
	public static $map_StatusToHint			= '';

	// All of our templates are stored here.
	public static $Templates				= '';

	public static function Init() {
		/*
		Purpose
			Used to initialize this class.
		*/
		
		// Default Field Values.
		self::$DefaultFieldValues = array(
			'DBServer'						=> 'localhost',
			'DBUsername'					=> '',
			'DBPassword'					=> '',
			'DBName'						=> '',
			'TablePrefix'					=> 'g2_',
			'ColumnPrefix'					=> 'g2_',
			'SubjectList'					=> "poker\r\ncasino\r\ncialis",
			'BodyList'						=> "jessica biel\r\ndate\r\nviagra",
			'AuthorList'					=> "Bweryang\r\nBob",
			'IPList'						=> "207.46.197.32\r\n198.175.96.33",
			'FindComments_Value'			=> '%',
			'FindComments_Max'				=> 100,
			'FindDuplicates_Value'			=> '%',
			'FindDuplicates_Count'			=> 2,
			'FindDuplicates_Max'			=> 100
		);

		// Initialize map_ListToMethod.
		self::$map_ListToMethod = array(
			'SubjectList'					=> 'Subject',
			'BodyList'						=> 'Body',
			'AuthorList'					=> 'Author',
			'IPList'						=> 'IP'
		);

		// Initialize map_ColumnToFriendlyName
		self::$map_ColumnToMethod = array(
			'subject'						=> 'Subject',
			'comment'						=> 'Body',
			'author'						=> 'Author',
			'host'							=> 'IP'
		);

		// Initialize map_ColumnToFriendlyName
		self::$map_ColumnToFriendly	= array(
			'subject'						=> 'Subject',
			'comment'						=> 'Body',
			'author'						=> 'Author',
			'host'							=> 'IP Address',
			'id'							=> 'Comment ID'
		);

		// Initialize map_ColumnToTemplate
		self::$map_ColumnToTemplate = array(
			'subject'						=> 'DuplRecSubject',
			'comment'						=> 'DuplRecBody',
			'author'						=> 'DuplRecAuthor',
			'host'							=> 'DuplRecIP'
		);

		// Initialize map_MethodToSQL.
		self::$map_MethodToSQL = array(
			'Subject'						=> "
				DELETE ce, e, co
				FROM 
					__TablePrefix__ChildEntity ce, 
					__TablePrefix__Entity e, 
					__TablePrefix__Comment co
				WHERE 
					ce.__ColumnPrefix__id = e.__ColumnPrefix__id AND 
					e.__ColumnPrefix__id = co.__ColumnPrefix__id AND
					e.__ColumnPrefix__entityType = 'GalleryComment' AND
					co.__ColumnPrefix__subject LIKE '__?__'
			",
			'Body'							=> "
				DELETE ce, e, co
				FROM 
					__TablePrefix__ChildEntity ce, 
					__TablePrefix__Entity e, 
					__TablePrefix__Comment co
				WHERE 
					ce.__ColumnPrefix__id = e.__ColumnPrefix__id AND 
					e.__ColumnPrefix__id = co.__ColumnPrefix__id AND
					e.__ColumnPrefix__entityType = 'GalleryComment' AND
					co.__ColumnPrefix__comment LIKE '__?__'
			",
			'Author'						=> "
				DELETE ce, e, co
				FROM 
					__TablePrefix__ChildEntity ce, 
					__TablePrefix__Entity e, 
					__TablePrefix__Comment co
				WHERE 
					ce.__ColumnPrefix__id = e.__ColumnPrefix__id AND 
					e.__ColumnPrefix__id = co.__ColumnPrefix__id AND
					e.__ColumnPrefix__entityType = 'GalleryComment' AND
					co.__ColumnPrefix__author LIKE '__?__'
			",
			'IP'							=> "
				DELETE ce, e, co
				FROM 
					__TablePrefix__ChildEntity ce, 
					__TablePrefix__Entity e, 
					__TablePrefix__Comment co
				WHERE 
					ce.__ColumnPrefix__id = e.__ColumnPrefix__id AND 
					e.__ColumnPrefix__id = co.__ColumnPrefix__id AND
					e.__ColumnPrefix__entityType = 'GalleryComment' AND
					co.__ColumnPrefix__host LIKE '__?__'
			",
			'ID'							=> "
				DELETE ce, e, co
				FROM 
					__TablePrefix__ChildEntity ce, 
					__TablePrefix__Entity e, 
					__TablePrefix__Comment co
				WHERE 
					ce.__ColumnPrefix__id = e.__ColumnPrefix__id AND 
					e.__ColumnPrefix__id = co.__ColumnPrefix__id AND
					e.__ColumnPrefix__entityType = 'GalleryComment' AND
					co.__ColumnPrefix__id = '__?__'
			",
			'All'							=> "
				DELETE ce, e, co
				FROM 
					__TablePrefix__ChildEntity ce, 
					__TablePrefix__Entity e, 
					__TablePrefix__Comment co
				WHERE 
					ce.__ColumnPrefix__id = e.__ColumnPrefix__id AND 
					e.__ColumnPrefix__id = co.__ColumnPrefix__id AND
					e.__ColumnPrefix__entityType = 'GalleryComment'
			",
			'CommentCount'					=> "
				SELECT Count(*) FROM __TablePrefix__Comment
			",
			'Find'							=> "
				SELECT *
				FROM __TablePrefix__Comment
				WHERE __ColumnPrefix____Column__ LIKE '__?__'
				LIMIT __Max__
			",
			'FindDuplicates'				=> "
				SELECT __ColumnPrefix____Column__, count(__ColumnPrefix____Column__) as cnt
				FROM __TablePrefix__Comment 
				WHERE __ColumnPrefix____Column__ LIKE '__Value__'
				GROUP BY __ColumnPrefix____Column__ 
				HAVING cnt >= __Count__
				ORDER BY cnt DESC
				LIMIT __Max__

			"
		);

		// Initialize map_TemplateToFile.
		self::$map_TemplateToFile = array(
			'About'							=> 'tpl-About.html',
			'Batch'							=> 'tpl-Batch.html',
			'Bones'							=> 'tpl-Bones.html',
			'Confirm'						=> 'tpl-Confirm.html',
			'Confirmed'						=> 'tpl-Confirmed.html',
			'DuplRecAuthor'					=> 'tpl-DuplRecAuthor.html',
			'DuplRecBody'					=> 'tpl-DuplRecBody.html',
			'DuplRecIP'						=> 'tpl-DuplRecIP.html',
			'DuplRecSubject'				=> 'tpl-DuplRecSubject.html',
			'FindRec'						=> 'tpl-FindRec.html',
			'Header'						=> 'tpl-Header.html',
			'License'						=> 'tpl-License.html',
			'Main'							=> 'tpl-Main.html',
			'Menu'							=> 'tpl-Menu.html',
			'Msg'							=> 'tpl-Msg.html',
			'MsgFile'						=> 'tpl-MsgFile.html',
			'Error'							=> 'tpl-Error.html',
			'SysReq'						=> 'tpl-SysReq.html',
			'Usage'							=> 'tpl-Usage.html',
			'CSS'							=> 'main.css',
			'JS'							=> 'main.js'
		);

		// Initialize map_ListToFile.
		self::$map_ListToFile = array(
			'SubjectList'					=> 'cc-SubjectList.txt',		// Stores Subject spam strings
			'BodyList'						=> 'cc-BodyList.txt',			// Stores Body spam strings
			'AuthorList'					=> 'cc-AuthorList.txt',			// Stores bad Author names
			'IPList'						=> 'cc-IPList.txt'				// Stores bad IP addresses
		);

		// Initialize GlobalTokens.
		self::$GlobalTokens = array(
			'Script_Name'					=> $_SERVER['SCRIPT_NAME'],
			'PageURL_About'					=> 'http://urlbam.com/ha/M002s',
			'PageURL_Main'					=> $_SERVER['PHP_SELF'] . '?page=Main',
			'PageURL_License'				=> 'http://urlbam.com/ha/M002r',
			'PageURL_SysReq'				=> 'http://urlbam.com/ha/M002q',
			'PageURL_Usage'					=> 'http://urlbam.com/ha/M002o'
		);

		// Initialize FieldsToPersist.
		self::$FieldsToPersist = array(
			'DBServer',
			'DBUsername',
			'DBPassword',
			'DBName',
			'TablePrefix',
			'ColumnPrefix',
			'FindComments_Column',
			'FindComments_Value',
			'FindComments_Max',
			'FindDuplicates_Column',
			'FindDuplicates_Value',
			'FindDuplicates_Count',
			'FindDuplicates_Max'
		);

		// Initialize map_StatusToMsg.
		self::$map_StatusToMsg = array(
			STATUS_InvalidListName						=> 'The specified list name is invalid.',
			STATUS_ErrorReadingListFile					=> 'An error occurred reading the list file.',
			STATUS_ErrorWritingListFile					=> 'An error occurred writing the list file.',
			STATUS_InvalidDataSet						=> 'An invalid dataset was specified.',
			STATUS_ErrorSortingList						=> 'An error occured sorting the list.',
			STATUS_DatabaseParameterMissing				=> 'A required database parameter is missing.',
			STATUS_ErrorConnectingToDatabase			=> 'An error occurred connecting to the database.',
			STATUS_ErrorSelectingDatabase				=> 'An error occurred selecting the database.',
			STATUS_ErrorQueryingDatabase				=> 'An error occurred querying the database.',
			STATUS_ErrorDetectingTablePrefix			=> 'An error occurred detecting the table prefix.',
			STATUS_ErrorDetectingColumnPrefix			=> 'An error occurred detecting the column prefix.',
			STATUS_ErrorReadingFile						=> 'An error occurred reading a file.',
			STATUS_ErrorWritingFile						=> 'An error occurred writing a file.',
			STATUS_InvalidIndex							=> 'The specified index is invalid.',
			STATUS_InvalidParameter						=> 'A specified parameter is invalid.'
		);

		// Initialize map_StatusToHint.
		self::$map_StatusToHint = array(
			STATUS_InvalidListName						=> '',
			STATUS_ErrorReadingListFile					=> 'Check the file permissions of the list files.',
			STATUS_ErrorWritingListFile					=> 'Check the file permissions of the list files.',
			STATUS_InvalidDataSet						=> '',
			STATUS_ErrorSortingList						=> '',
			STATUS_DatabaseParameterMissing				=> 'Make sure all database parameters are specified.',
			STATUS_ErrorConnectingToDatabase			=> 'Make sure database credentials are correct.',
			STATUS_ErrorSelectingDatabase				=> 'Make sure the database name is correct.',
			STATUS_ErrorQueryingDatabase				=> 'Make sure table and column prefixes are correct.',
			STATUS_ErrorDetectingTablePrefix			=> '',
			STATUS_ErrorDetectingColumnPrefix			=> '',
			STATUS_ErrorReadingFile						=> '',
			STATUS_ErrorWritingFile						=> '',
			STATUS_InvalidIndex							=> '',
			STATUS_InvalidParameter						=> ''
		);

		// Common Messages
		self::$Msgs = array(
			'ListChanged'								=> 'The list has changed.<br />Remember to save it.',
			'DeleteComments'							=> "
				__CountQueries__ queries were executed.<br />\n
				<br />\n
				__CommentsDeleted__ comments were deleted.<br />\n
				__RowsDeleted__ rows were deleted (total including all tables).<br />\n
				<br />\n
				Number of comments before delete: __CountBefore__<br />\n
				Number of comments after delete: __CountAfter__<br />\n
			",
			'DeleteComments_Complete'					=> "
				The batch-delete process has completed successfully.<br />\n<br />\n
			",
			'DeleteComments_Cancelled'					=> "
				The batch-delete process was cancelled.<br />\n<br />\n
			",
			'DeleteComments_InProcess'					=> "
				Currently batch-processing \"__String__\"<br />\n
				<br />\n
				Please wait...<br />\n
				When working with large tables, this could take some time.
			",
			'DeleteCommentsAll_Complete'				=> "
				The delete-all-comments process has completed successfully.<br />\n<br />\n
			"
		);

		// Templates
		self::$Templates = array(
			'About'							=> '

<p class="Text">Forwarding to Comment Cruncher <a href="__PageURL_About__" class="Text_A">About</a>...</p>

<form name="MyForm" method="post" action="__PageURL_About__">
</form>

<script type="text/javascript">
setTimeout(\'document.MyForm.submit();\', 100);
</script>
',
			'Batch'							=> '
<form action="__Script_Name__" method="post" name="MyForm" class="form_FORM" id="MyForm">

<script type="text/javascript">

var TimeoutID = 0;

function SubmitMyForm() {

	// Hide the submit button. This prevents cancelling in the middle of a database query.
	var SubmitButton = document.getElementById(\'Submit_Cancel\');
	SubmitButton.style.visibility = \'hidden\';
	
	// Submit the form.
	document.MyForm.submit();
}

// We need to do this to be sure the form is not submitted twice:
// First by the user, and then while we\'re waiting for a response, by the timeout().
function CancelTimeout() {
	clearTimeout(TimeoutID);
}

// The the user a few seconds to hit cancel.
TimeoutID = setTimeout(\'SubmitMyForm();\', __SubmitDelay__);

</script>

<table border="0" align="center" cellpadding="0" cellspacing="0" class="form_TABLE" style="margin-top:24px;">
  <tr>
    <td class="form_TD"><p class="Chapter" style="">__Title__</p>
      <p class="Text">__Message__</p>
      <p class="Text">
        <input name="Submit_Cancel" type="submit" class="form_SUBMIT" id="Submit_Cancel" value="Cancel" onClick="CancelTimeout();">
		__FormFields__
      </p></td>
  </tr>
</table>
</form>',
			'Bones'							=> '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
<link href="main.css" rel="stylesheet" type="text/css">
<style type="text/css">
<!--
body {
	margin-left: 0px;
	margin-top: 0px;
	margin-right: 0px;
	margin-bottom: 0px;
}
-->
</style>
__HEAD__
<title>__TITLE__</title>
<style type="text/css">
.Header {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 18px;
	color: #F4E9DD;
	background-color: #684522;
	text-align: center;
	padding-top: 4px;
	padding-right: 0px;
	padding-bottom: 4px;
	padding-left: 0px;
	font-weight: bold;
	margin: 0px;
}
.SubHeader {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 14px;
	font-weight: bold;
	color: #F4E9DD;
	background-color: #BE7E3F;
	padding-top: 2px;
	padding-right: 0px;
	padding-bottom: 2px;
	padding-left: 0px;
	text-align: center;
	margin-top: 0px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.Title {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 16px;
	font-weight: bold;
	color: #F4E9DD;
	background-color: #BE7E3F;
	text-align: center;
	padding-top: 4px;
	padding-right: 0px;
	padding-bottom: 4px;
	padding-left: 0px;
	margin-top: 16px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
	border: 1px none #684522;
}

.Chapter {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 14px;
	font-weight: bold;
	color: #684522;
	background-color: #F4E9DD;
	padding-top: 2px;
	padding-right: 0px;
	padding-bottom: 2px;
	padding-left: 8px;
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.Chapter_Error {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 14px;
	font-weight: bold;
	color: #684522;
	background-color: #EDC5C2;
	padding-top: 2px;
	padding-right: 0px;
	padding-bottom: 2px;
	padding-left: 8px;
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.Chapter_Confirmed {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 14px;
	font-weight: bold;
	color: #684522;
	background-color: #C2E2D7;
	padding-top: 2px;
	padding-right: 0px;
	padding-bottom: 2px;
	padding-left: 8px;
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.Chapter_Link_A {
	color: #BE7E3F;
	cursor: hand;
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-style: normal;
	font-weight: normal;
	padding-top: 0px;
	padding-right: 8px;
	padding-bottom: 0px;
	padding-left: 8px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: solid;
	border-top-color: #BE7E3F;
	border-right-color: #BE7E3F;
	border-bottom-color: #BE7E3F;
	border-left-color: #BE7E3F;
	float: right;
	margin-top: 2px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}

.Chapter_Menu_SUBMIT {
	color: #BE7E3F;
	background-color: transparent;
	cursor: pointer;
	cursor: hand;
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-style: normal;
	font-weight: normal;
	padding-top: 0px;
	padding-right: 2px;
	padding-bottom: 0px;
	padding-left: 2px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: solid;
	border-top-color: #BE7E3F;
	border-right-color: #BE7E3F;
	border-bottom-color: #BE7E3F;
	border-left-color: #BE7E3F;
	float: right;
	margin: 0px;
}
.Chapter_Menu_SUBMIT_NoBorder {
	color: #BE7E3F;
	cursor: hand;
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-style: normal;
	font-weight: normal;
	padding-top: 0px;
	padding-right: 2px;
	padding-bottom: 0px;
	padding-left: 2px;
	margin: 0px;
	float: right;
	background-color: transparent;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
}
.Chapter_Menu {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	color: #BE7E3F;
	font-weight: normal;
	float: right;
	margin-top: 0px;
	margin-right: 12px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.Chapter_Menu_A {
	color: #BE7E3F;
}

.Section {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: bold;
	color: #684522;
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 20px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: solid;
	border-left-style: none;
	border-top-color: #684522;
	border-right-color: #684522;
	border-bottom-color: #684522;
	border-left-color: #684522;
}

.Subsection {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: bold;
	color: #684522;
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 20px;
}

.Text {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	color: #684522;
	margin-top: 8px;
	margin-right: 20px;
	margin-bottom: 0px;
	margin-left: 20px;
}
.Text_A {
	color: #669933;
}
.Menu {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #BE7E3F;
	text-align: center;
	margin-top: 0px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
	padding-top: 4px;
	padding-right: 0px;
	padding-bottom: 4px;
	padding-left: 0px;
}
.Menu_A {
	color: #BE7E3F;
}
.Code {
	font-family: "Courier New", Courier, monospace;
	font-size: 11px;
	color: #333333;
	margin-top: 8px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 20px;
}


.form_TABLE {
	border: 2px solid #BE7E3F;
	width: 700px;
	margin-top: 0px;
	margin-right: auto;
	margin-bottom: 0px;
	margin-left: auto;
}
.form_TD {
	padding-top: 0px;
	padding-right: 20px;
	padding-bottom: 12px;
	padding-left: 20px;
}
.form_FORM {
	padding: 0px;
	margin: 0px;
}
.form_Field_TABLE {
	padding: 0px;
	margin-top: 2px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
	width: 620px;
}
.form_Field-Label_TD {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: bold;
	color: #BE7E3F;
	text-align: right;
	padding-top: 0px;
	padding-right: 8px;
	padding-bottom: 0px;
	padding-left: 0px;
	width: 220px;
}
.form_Field-Data_TD {
	width: 450px;
	padding: 0px;
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #666666;
}
.form_Field-Desc {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #999999;
	margin-top: 0px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 8px;
}
.form_Field-TEXT {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #666666;
	border: 1px solid #BE7E3F;
	width: 200px;
}
.form_Field-TEXT_NoWidth {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #666666;
	border: 1px solid #BE7E3F;
}
.form_Field-TEXT-Chapter {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #BE7E3F;
	border: 1px solid #BE7E3F;
	float: right;
	margin-top: 1px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.form_Field-TEXTAREA {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #666666;
	height: 100px;
	width: 200px;
	border: 1px solid #BE7E3F;
	text-align: left;
}
.form_Field-SELECT {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #666666;
	border: 1px solid #BE7E3F;
}

.form_SUBMIT {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #684522;
	width: 50px;
}
.form_SUBMIT_NoWidth {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #684522;
}





.Top {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 7pt;
	font-style: italic;
	font-weight: bold;
	color: #3F7C79;
	display: block;
	margin: 0px;
}
.Top_A {
	color: #3F7C79;
}
.Credit {
	font-family: Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #F4E9DD;
	float: right;
	text-align: right;
	margin-top: 3px;
	margin-right: 12px;
	margin-bottom: 0px;
	margin-left: -200px;
	width: 200px;
}
.Credit_A {
	color: #F4E9DD;
}
.Copyright {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-style: normal;
	font-weight: normal;
	color: #A26B35;
	text-align: center;
}



ul {
	list-style-type: disc;
	margin-top: 8px;
	margin-bottom: 0px;
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #684522;
	list-style-position: inside;
}
li {
}

ol {
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #684522;
}
blockquote {
	margin-top: 8px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 20px;
}
.Chapter_Menu_SUBMIT:hover {
	text-decoration: underline;
}


a:link {
    text-decoration: none;
}

a:visited {
    text-decoration: none;
}

a:hover {
    text-decoration: underline;
}

a:active {
    text-decoration: underline;
}

</style>
<script type="text/javascript">

function Checkboxes_CountState(FormName, State) {
	/*
	Purpose
		Used to count the number of checkboxes within a specified form
		that are set to a specified state (true or false).
	Parameters
		FormName (string) = Only checkboxes within this form are counted.
		State (optional boolean) = To count the number of checked checkboxes,
			set this to true. To count unchecked checkboxes, set to false.
			Default: true
	Return (integer)
		Return the total number of checkboxes that are set to the same state as
		indicated by State.
	Version History
		1.00 2009.6.25 Tested by Russ Tanner on IE6/FF3.
	*/
	
	// Default State to true.
	if (typeof State == \'undefined\') State = true;
	
	var frm = document.forms[FormName];
	var c = frm.elements.length;
	var r = 0;
	for(var i=0; i<c; i++) {
		if (frm.elements[i].type == \'checkbox\') r += (frm.elements[i].checked == State) ? 1 : 0;
	}
	
	return r;
}


function Checkboxes_InvertAll(FormName) {
	/*
	Purpose
		Used to invert the value of all checkboxes in the specified form.
	Parameters
		FormName (string) = The name of the form we\'re working with.
	Version History
		1.00 2009.6.24 Tested by Russ Tanner on IE6/FF3.
	*/
	
	var frm = document.forms[FormName]
	var c = frm.elements.length;
	for(var i=0; i<c; i++) {
		if (frm.elements[i].type == \'checkbox\') frm.elements[i].checked = !frm.elements[i].checked;
	}
}


function Checkboxes_SetAll(FormName, Value) {
	/*
	Purpose
		Used to set the value of all checkboxes in the specified form.
	Parameters
		FormName (string) = The name of the form we\'re working with.
		Value (boolean) = The value we\'re going to set all checkboxes to .
	Version History
		1.00 2009.6.24 Tested by Russ Tanner on IE6/FF3.
	*/
	
	var frm = document.forms[FormName]
	var c = frm.elements.length;
	for(var i=0; i<c; i++) {
		if (frm.elements[i].type == \'checkbox\') frm.elements[i].checked = Value;
	}
}


function Form_Append_HiddenElement(FormName, Name, Value) {
	/*
	Purpose
		Used to append a hidden form field to an existing form.
	Parameters
		FormName (string) = The name of the form we\'re inserting this form field into.
		Name (string) = The "name" attribute that we\'ll assign to the field.
		Value (string) = The "value" attribute that we\'ll assign to the field.
	Version History
		1.00 2009.6.24 Tested by Russ Tanner on IE6/FF3.
	*/
	
	// Reference to the element we\'re building.
	var o;

	// Create the element.
	o = document.createElement(\'input\');
	
	// Define attributes for the element.
	o.type = \'hidden\';
	o.name = Name;
	o.value = Value;
	o.defaultValue = Value;
	
	// Insert the element into the form.
	document.forms[FormName].appendChild(o);
}


function Form_Submit(FormName, Action){
	/*
	Purpose
		Submits a form in a simple, generic way.
	Parameters
		FormName (string) = The name of the form we want to submit.
		Action (optional string) = If specified this value is assigned to the .action method
			of the form before it is submitted.
	Version History
		1.00 2009.6.24 Tested by Russ Tanner on IE6/FF3.
	*/

	// Get a rererence to the specified form.
	var frm = document.forms[FormName]
	
	// Set the form\'s action and submit it.
	if (typeof Action != \'undefined\') frm.action = Action;
	frm.submit();
}


function KeyPress_Filter(e, Key, Action) {
	/*
	Purpose
		Used to cancel specified key presses.
		Call this function from a form elements\' onKeyPress event and it will return false
		when any of the specified keys are pressed.
	Parameters
		e (object) = The event object provided by the element we\'re working with.
		Key (optional string, number, array) = The key or array of keys we want to filter.
			In the context of this parameter, "filter" means to either allow or disallow,
				depending on the value of Action.
			If this is a string, if the key is pressed that generates the specified string,
				then that key is filtered (allowed/disallowed depending on Action).
			If this is a number, if the key is pressed that generates a keycode matching
				this number, then that key is filtered (allowed/disallowed depending on Action).
			If this is an array, then array elements can be either strings or numbers.
				In this case each element represents a key that is either allowed or 
				disallowed depending on the value of Action.
			Default: 13
		Action (optional boolean) = This is the value that this function returns when a pressed key
			matches a key specified in Key.
			Default: false
	Return (boolean)
		If the specified key (or any of the keys specified in an array) is pressed,
			function returns whatever value is specified in Action.
	Version History
		1.00 2009.6.28 Unfinished
	*/
	
	// Default parameters.
	if (typeof Key == \'undefined\') Key = 13;
	if (typeof Action == \'undefined\') Action = false;

	// Get the event object (cross-browser).
	var evt = (evt) ? evt : ((event) ? event : null);

	// Get the source object (cross-browser).
	var node = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : null);

	alert(node.name);
	
	return false;
	
}


function IgnoreEnterKey(e) {
	/*
	Purpose
		Used to cause a form element to ignore the Enter key.
	Return (boolean)
		Returns false when the enter key is hit on a hooked element.
	Version History
		1.00 2009.06.29 Tested by Russ Tanner on IE6/FF3.
	*/

	// Get a common event variable (cross-browser).
	if (!e) var e = window.event;

	// Get the key code that was pressed (cross-browser).	
	if (e.keyCode) kc = e.keyCode;
	else if (e.which) kc = e.which;

	// Don\'t let this event propagate (performance optimization).
	if (e.stopPropagation) e.stopPropagation();

	// Return false if the enter key was pressed.
	return ! (kc == 13);
	
}


</script>
</head>

<body onLoad="if (typeof window.Init == \'function\') { Init(); }">
__Header__ __Menu__ __Body__
<p class="Copyright">Copyright 2009, Russ Tanner, All Rights Reserved </p>
</body>
</html>
',
			'Confirm'						=> '
<table border="0" align="center" cellpadding="0" cellspacing="0" class="form_TABLE" style="margin-top:24px;">
  <tr>
    <td class="form_TD"><p class="Chapter" style="">__Title__</p>
      <p class="Text">__Message__</p>
      <p class="Text"><a href="__URL1__" class="Text_A">__Label1__</a> | <a href="__URL2__" class="Text_A">__Label2__</a></p></td>
  </tr>
</table>',
			'Confirmed'						=> '
<table border="0" align="center" cellpadding="0" cellspacing="0" class="form_TABLE" style="margin-top:24px;">
  <tr>
    <td class="form_TD"><p class="Chapter_Confirmed">__Title__</p>
    <p class="Text">__Message__</p>
    <p class="Text">Click here to <a href="__URLContinue__" class="Text_A">continue...</a></p></td>
  </tr>
</table>',
			'DuplRecAuthor'					=> '
<p class="Chapter" style="">
	<a href="__PageURL_Main__" class="Chapter_Link_A">Main Page</a>
	<a href="#" class="Chapter_Link_A" onclick=\'if (! Checkboxes_CountState("Main")) { alert("No items are checked."); return false; } if (confirm("Are you sure you want to delete all checked comments?")) { Form_Append_HiddenElement("Main","do","FindDuplicatesDelete"); Form_Submit("Main"); } return false;\'>Delete All Checked</a>
	<input name="max__SN__" type="text" class="form_Field-TEXT-Chapter" id="max__SN__" style="margin-right:8px;	text-align: right;" value="__CountInt__" size="5" maxlength="12" onkeypress="kc=(event.which)?event.which:event.keyCode; if(kc==13) {document.getElementById(\'View__SN__\').onclick(); return false;}">
	<a href="#" class="Chapter_Link_A" id="View__SN__" name="View__SN__" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'FindComments\');Form_Append_HiddenElement(\'Main\',\'value\',\'__Author__\');Form_Append_HiddenElement(\'Main\',\'max\',document.forms[\'Main\'].elements[\'max__SN__\'].value);Form_Submit(\'Main\');return false;">View</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_InvertAll(\'Main\');return false;">Invert All</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_SetAll(\'Main\',false);return false;">Uncheck All</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_SetAll(\'Main\',true);return false;">Check All</a>
	<input name="value[]" type="checkbox" id="value[]" value="__Author__">
# __SN__</p>
<p class="Text"><strong>Author:</strong> __Author__<br>
  <strong>Number of Duplicates:</strong> __Count__</p>',
			'DuplRecBody'					=> '
<p class="Chapter" style="">
	<a href="__PageURL_Main__" class="Chapter_Link_A">Main Page</a>
	<a href="#" class="Chapter_Link_A" onclick=\'if (! Checkboxes_CountState("Main")) { alert("No items are checked."); return false; } if (confirm("Are you sure you want to delete all checked comments?")) { Form_Append_HiddenElement("Main","do","FindDuplicatesDelete"); Form_Submit("Main"); } return false;\'>Delete All Checked</a>
	<input name="max__SN__" type="text" class="form_Field-TEXT-Chapter" id="max__SN__" style="margin-right:8px;	text-align: right;" value="__CountInt__" size="5" maxlength="12" onkeypress="kc=(event.which)?event.which:event.keyCode; if(kc==13) {document.getElementById(\'View__SN__\').onclick(); return false;}">
	<a href="#" class="Chapter_Link_A" id="View__SN__" name="View__SN__" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'FindComments\');Form_Append_HiddenElement(\'Main\',\'value\',\'__Body__\');Form_Append_HiddenElement(\'Main\',\'max\',document.forms[\'Main\'].elements[\'max__SN__\'].value);Form_Submit(\'Main\');return false;">View</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_InvertAll(\'Main\');return false;">Invert All</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_SetAll(\'Main\',false);return false;">Uncheck All</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_SetAll(\'Main\',true);return false;">Check All</a>
	<input name="value[]" type="checkbox" id="value[]" value="__Body__">
# __SN__</p>
<p class="Text"><strong>Number of Duplicates:</strong> __Count__</p>
<p class="Text">__Body__</p>',
			'DuplRecIP'						=> '
<p class="Chapter" style="">
	<a href="__PageURL_Main__" class="Chapter_Link_A">Main Page</a>
	<a href="#" class="Chapter_Link_A" onclick=\'if (! Checkboxes_CountState("Main")) { alert("No items are checked."); return false; } if (confirm("Are you sure you want to delete all checked comments?")) { Form_Append_HiddenElement("Main","do","FindDuplicatesDelete"); Form_Submit("Main"); } return false;\'>Delete All Checked</a>
	<input name="max__SN__" type="text" class="form_Field-TEXT-Chapter" id="max__SN__" style="margin-right:8px;	text-align: right;" value="__CountInt__" size="5" maxlength="12" onkeypress="kc=(event.which)?event.which:event.keyCode; if(kc==13) {document.getElementById(\'View__SN__\').onclick(); return false;}">
	<a href="#" class="Chapter_Link_A" id="View__SN__" name="View__SN__" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'FindComments\');Form_Append_HiddenElement(\'Main\',\'value\',\'__IP__\');Form_Append_HiddenElement(\'Main\',\'max\',document.forms[\'Main\'].elements[\'max__SN__\'].value);Form_Submit(\'Main\');return false;">View</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_InvertAll(\'Main\');return false;">Invert All</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_SetAll(\'Main\',false);return false;">Uncheck All</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_SetAll(\'Main\',true);return false;">Check All</a>
	<input name="value[]" type="checkbox" id="value[]" value="__IP__">
# __SN__</p>
<p class="Text"><strong>IP Address:</strong> __IP__<br>
  <strong>Number of Duplicates:</strong> __Count__</p>',
			'DuplRecSubject'				=> '
<p class="Chapter" style="">
	<a href="__PageURL_Main__" class="Chapter_Link_A">Main Page</a>
	<a href="#" class="Chapter_Link_A" onclick=\'if (! Checkboxes_CountState("Main")) { alert("No items are checked."); return false; } if (confirm("Are you sure you want to delete all checked comments?")) { Form_Append_HiddenElement("Main","do","FindDuplicatesDelete"); Form_Submit("Main"); } return false;\'>Delete All Checked</a>
	<input name="max__SN__" type="text" class="form_Field-TEXT-Chapter" id="max__SN__" style="margin-right:8px;	text-align: right;" value="__CountInt__" size="5" maxlength="12" onkeypress="kc=(event.which)?event.which:event.keyCode; if(kc==13) {document.getElementById(\'View__SN__\').onclick(); return false;}">
	<a href="#" class="Chapter_Link_A" id="View__SN__" name="View__SN__" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'FindComments\');Form_Append_HiddenElement(\'Main\',\'value\',\'__Subject__\');Form_Append_HiddenElement(\'Main\',\'max\',document.forms[\'Main\'].elements[\'max__SN__\'].value);Form_Submit(\'Main\');return false;">View</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_InvertAll(\'Main\');return false;">Invert All</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_SetAll(\'Main\',false);return false;">Uncheck All</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_SetAll(\'Main\',true);return false;">Check All</a>
	<input name="value[]" type="checkbox" id="value[]" value="__Subject__">
# __SN__</p>
<p class="Text"><strong>Subject:</strong> __Subject__<br>
  <strong>Number of Duplicates:</strong> __Count__</p>',
			'FindRec'						=> '
<p class="Chapter" style="">
	<a href="__PageURL_Main__" class="Chapter_Link_A">Main Page</a>
	<a href="#" class="Chapter_Link_A" onclick="if (! Checkboxes_CountState(\'Main\')) { alert(\'No items are checked.\'); return false; } if (confirm(\'Are you sure you want to delete all checked comments?\')) { Form_Append_HiddenElement(\'Main\',\'do\',\'FindCommentsDelete\');Form_Submit(\'Main\')} return false;">Delete All Checked</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_InvertAll(\'Main\');return false;">Invert All</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_SetAll(\'Main\',false);return false;">Uncheck All</a>
	<a href="#" class="Chapter_Link_A" onclick="Checkboxes_SetAll(\'Main\',true);return false;">Check All</a>
    <input name="id[]" type="checkbox" id="id[]" value="__ID__">
# __SN__</p>
<p class="Text"><strong>Subject:</strong> __Subject__<br>
<p class="Text"><strong>Author:</strong> __Author__<br>
<strong>Date:</strong> __Date__<br>
<strong>IP Address:</strong> __IP__<br>
<strong>Comment ID:</strong> __ID__</p>
<p class="Text">__Body__</p>',
			'Header'						=> '

<p class="Header"><span class="Credit"><a href="http://urlbam.com/ha/M002c" target="_blank" class="Credit_A">by Russ Tanner</a></span>Gallery Comment Cruncher</p>
<p class="SubHeader">Version 1.00</p>
',
			'License'						=> '

<p class="Text">Forwarding to Comment Cruncher <a href="__PageURL_License__" class="Text_A">License</a>...</p>

<form name="MyForm" method="post" action="__PageURL_License__">
</form>

<script type="text/javascript">
setTimeout(\'document.MyForm.submit();\', 100);
</script>
',
			'Main'							=> '

<table border="0" align="center" cellpadding="0" cellspacing="0" class="form_TABLE" style="margin-top:24px;">
  <tr>
    <td class="form_TD"><form action="__Script_Name__" method="post" name="Main" class="form_FORM" id="Main">
        <p class="Title" style="">Comment Management</p>
        <p class="Chapter" style="">Overview</p>
        <p class="Text">Comment Cruncher removes unwanted comments from your Gallery. Comments can be removed based on the  subject, body, author, or IP address.</p>
        <p class="Text">Please see <a href="__PageURL_License__" target="_blank" class="Text_A">licensing</a> and <a href="__PageURL_SysReq__" target="_blank" class="Text_A">system requirements</a>. For help, see the <a href="__PageURL_Usage__" target="_blank" class="Text_A">usage</a> page.</p>
        <p class="Chapter"><a href="#"class="Chapter_Link_A" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'RestoreDefaults\'); Form_Append_HiddenElement(\'Main\',\'q\',\'DBCredentials\'); Form_Submit(\'Main\'); return false;">Restore Defaults</a>Database Credentials</p>
        <p class="Text">Enter the credentials necessary to access the database that Gallery is using.</p>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE" style="margin-top:12px;">
            <tr>
              <td class="form_Field-Label_TD">Database Server</td>
              <td class="form_Field-Data_TD"><input name="DBServer" type="text" class="form_Field-TEXT" id="DBServer" value="__DBServer__" maxlength="100"></td>
            </tr>
        </table>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE">
            <tr>
              <td class="form_Field-Label_TD">Database Username</td>
              <td class="form_Field-Data_TD"><input name="DBUsername" type="text" class="form_Field-TEXT" id="DBUsername" value="__DBUsername__" maxlength="100">              </td>
            </tr>
        </table>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE">
            <tr>
              <td class="form_Field-Label_TD">Database Password</td>
              <td class="form_Field-Data_TD"><input name="DBPassword" type="text" class="form_Field-TEXT" id="DBPassword" value="__DBPassword__" maxlength="100">              </td>
            </tr>
        </table>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE">
            <tr>
              <td class="form_Field-Label_TD">Database Name</td>
              <td class="form_Field-Data_TD"><input name="DBName" type="text" class="form_Field-TEXT" id="DBName" value="__DBName__" maxlength="100"></td>
            </tr>
        </table>
        <p class="Chapter">
		<a href="#"class="Chapter_Link_A" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'RestoreDefaults\'); Form_Append_HiddenElement(\'Main\',\'q\',\'Prefixes\'); Form_Submit(\'Main\'); return false;">Restore Defaults</a>
		<a href="#"class="Chapter_Link_A" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'AutoDetect\'); Form_Append_HiddenElement(\'Main\',\'q\',\'Prefixes\'); Form_Submit(\'Main\'); return false;">Auto Detect </a>
        Table &amp; Column Prefix</p>
          <p class="Text">Enter the prefix that is appended to your table names and column names.</p>
          <p class="Text">By default, many Gallery installations use &quot;g2_&quot; as the default  prefix for both tables and columns (don\'t forget the underscore). You can use the &quot;Auto Detect&quot; button to let Comment Cruncher detect prefixes for you.</p>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE" style="margin-top:12px;">
            <tr>
              <td class="form_Field-Label_TD">Table Prefix </td>
              <td class="form_Field-Data_TD"><input name="TablePrefix" type="text" class="form_Field-TEXT" id="TablePrefix" value="__TablePrefix__" maxlength="100"></td>
            </tr>
        </table>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE">
            <tr>
              <td class="form_Field-Label_TD">Column Prefix </td>
              <td class="form_Field-Data_TD"><input name="ColumnPrefix" type="text" class="form_Field-TEXT" id="ColumnPrefix" value="__ColumnPrefix__" maxlength="100"></td>
            </tr>
        </table>
        <p class="Chapter">
		<a href="#" class="Chapter_Link_A" onclick="if (confirm(\'Are you sure you want to remove duplicates from the (SubjectList) list?\')) { Form_Append_HiddenElement(\'Main\',\'do\',\'RemoveDuplicates\'); Form_Append_HiddenElement(\'Main\',\'q\',\'SubjectList\'); Form_Submit(\'Main\'); } return false;">Remove Duplicates</a>
		<a href="#" class="Chapter_Link_A" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'SortList\'); Form_Append_HiddenElement(\'Main\',\'q\',\'SubjectList\'); Form_Submit(\'Main\'); return false;">Sort List</a>
		<a href="#" class="Chapter_Link_A" onclick="if (confirm(\'Are you sure you want to clear the (SubjectList) list?\')) { Form_Append_HiddenElement(\'Main\',\'do\',\'ClearList\'); Form_Append_HiddenElement(\'Main\',\'q\',\'SubjectList\'); Form_Submit(\'Main\'); } return false;">Clear List</a>
        Delete By Subject</p>
          <p class="Text">This feature deletes comments that contain  specified strings in the subject.</p>
          <p class="Text">The &quot;Save&quot; button simply saves this list to a file (cc-SubjectList.txt) in the directory this script is running.</p>
          <p class="Text"> The &quot;Delete&quot; button deletes all comments having a subject matching any item in the list. SQL &quot;LIKE&quot; wildcards (&quot;%&quot;, &quot;_&quot;) are allowed. </p>
          <p class="Text">The &quot;Batch&quot; button does the same thing as the &quot;Delete&quot; button, one item at a time. This is useful for avoiding PHP timeout errors when processing large lists on large databases. See the <a href="__PageURL_Usage__" target="_blank" class="Text_A">usage</a> page for additional details.</p>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE" style="margin-top:12px;">
            <tr>
              <td valign="top" class="form_Field-Label_TD">Undesired Strings
              <p id="SubjectList_Msg">&nbsp;</p></td>
              <td class="form_Field-Data_TD"><textarea name="SubjectList" class="form_Field-TEXTAREA" id="SubjectList" onkeyup="SayListChanged(\'SubjectList\')" onchange="SayListChanged(\'SubjectList\')" onFocus="setInterval(\'SayListChanged(\\\'SubjectList\\\')\',2000);">__SubjectList__</textarea></td>
            </tr>
        </table>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE">
            <tr>
              <td class="form_Field-Label_TD">&nbsp;</td>
              <td class="form_Field-Data_TD"><input name="Submit_SaveList_SubjectList" type="submit" class="form_SUBMIT" id="Submit_SaveList_SubjectList" value="Save">
                <input name="Submit_DeleteComments_SubjectList" type="submit" class="form_SUBMIT" id="Submit_DeleteComments_SubjectList" value="Delete" style="margin-left:45px;">
                <input name="Submit_DeleteBatch_SubjectList" type="submit" class="form_SUBMIT" id="Submit_DeleteBatch_SubjectList" value="Batch"></td>
            </tr>
        </table>
          <p class="Chapter">
		<a href="#" class="Chapter_Link_A" onclick="if (confirm(\'Are you sure you want to remove duplicates from the (BodyList) list?\')) { Form_Append_HiddenElement(\'Main\',\'do\',\'RemoveDuplicates\'); Form_Append_HiddenElement(\'Main\',\'q\',\'BodyList\'); Form_Submit(\'Main\'); } return false;">Remove Duplicates</a>
		<a href="#" class="Chapter_Link_A" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'SortList\'); Form_Append_HiddenElement(\'Main\',\'q\',\'BodyList\'); Form_Submit(\'Main\'); return false;">Sort List</a>
		<a href="#" class="Chapter_Link_A" onclick="if (confirm(\'Are you sure you want to clear the (BodyList) list?\')) { Form_Append_HiddenElement(\'Main\',\'do\',\'ClearList\'); Form_Append_HiddenElement(\'Main\',\'q\',\'BodyList\'); Form_Submit(\'Main\'); } return false;">Clear List</a>
          Delete By Body</p>
          <p class="Text">This feature deletes comments that contain  specified strings in the body. <br>
          The associated file is cc-BodyList.txt. See the <a href="__PageURL_Usage__" target="_blank" class="Text_A">usage</a> page for additional details.</p>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE" style="margin-top:12px;">
            <tr>
              <td valign="top" class="form_Field-Label_TD">Undesired Strings
                <p id="BodyList_Msg">&nbsp;</p></td>
              <td class="form_Field-Data_TD"><textarea name="BodyList" class="form_Field-TEXTAREA" id="BodyList" onkeyup="SayListChanged(\'BodyList\')" onchange="SayListChanged(\'BodyList\')" onFocus="setInterval(\'SayListChanged(\\\'BodyList\\\')\',2000);">__BodyList__</textarea></td>
            </tr>
          </table>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE">
            <tr>
              <td class="form_Field-Label_TD">&nbsp;</td>
              <td class="form_Field-Data_TD"><input name="Submit_SaveList_BodyList" type="submit" class="form_SUBMIT" id="Submit_SaveList_BodyList" value="Save">
                <input name="Submit_DeleteComments_BodyList" type="submit" class="form_SUBMIT" id="Submit_DeleteComments_BodyList" value="Delete" style="margin-left:45px;">
                <input name="Submit_DeleteBatch_BodyList" type="submit" class="form_SUBMIT" id="Submit_DeleteBatch_BodyList" value="Batch"></td>
            </tr>
          </table>
          <p class="Chapter">
		<a href="#" class="Chapter_Link_A" onclick="if (confirm(\'Are you sure you want to remove duplicates from the (AuthorList) list?\')) { Form_Append_HiddenElement(\'Main\',\'do\',\'RemoveDuplicates\'); Form_Append_HiddenElement(\'Main\',\'q\',\'AuthorList\'); Form_Submit(\'Main\'); } return false;">Remove Duplicates</a>
		<a href="#" class="Chapter_Link_A" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'SortList\'); Form_Append_HiddenElement(\'Main\',\'q\',\'AuthorList\'); Form_Submit(\'Main\'); return false;">Sort List</a>
		<a href="#" class="Chapter_Link_A" onclick="if (confirm(\'Are you sure you want to clear the (AuthorList) list?\')) { Form_Append_HiddenElement(\'Main\',\'do\',\'ClearList\'); Form_Append_HiddenElement(\'Main\',\'q\',\'AuthorList\'); Form_Submit(\'Main\'); } return false;">Clear List</a>
            Delete By Author</p>
          <p class="Text">This feature deletes comments that were made by any of the authors you specify. <br>
          The associated file is cc-AuthorList.txt. See the <a href="__PageURL_Usage__" target="_blank" class="Text_A">usage</a> page for additional details.</p>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE" style="margin-top:12px;">
            <tr>
              <td valign="top" class="form_Field-Label_TD">Undesired Authors
              <p id="AuthorList_Msg">&nbsp;</p></td>
              <td class="form_Field-Data_TD"><textarea name="AuthorList" class="form_Field-TEXTAREA" id="AuthorList" onkeyup="SayListChanged(\'AuthorList\')" onchange="SayListChanged(\'AuthorList\')" onFocus="setInterval(\'SayListChanged(\\\'AuthorList\\\')\',2000);">__AuthorList__</textarea>              </td>
            </tr>
        </table>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE">
            <tr>
              <td class="form_Field-Label_TD">&nbsp;</td>
              <td class="form_Field-Data_TD"><input name="Submit_SaveList_AuthorList" type="submit" class="form_SUBMIT" id="Submit_SaveList_AuthorList" value="Save">
              <input name="Submit_DeleteComments_AuthorList" type="submit" class="form_SUBMIT" id="Submit_DeleteComments_AuthorList" value="Delete" style="margin-left:45px;">
              <input name="Submit_DeleteBatch_AuthorList" type="submit" class="form_SUBMIT" id="Submit_DeleteBatch_AuthorList" value="Batch"></td>
            </tr>
        </table>
          <p class="Chapter">
		<a href="#" class="Chapter_Link_A" onclick="if (confirm(\'Are you sure you want to remove duplicates from the (IPList) list?\')) { Form_Append_HiddenElement(\'Main\',\'do\',\'RemoveDuplicates\'); Form_Append_HiddenElement(\'Main\',\'q\',\'IPList\'); Form_Submit(\'Main\'); } return false;">Remove Duplicates</a>
		<a href="#" class="Chapter_Link_A" onclick="Form_Append_HiddenElement(\'Main\',\'do\',\'SortList\'); Form_Append_HiddenElement(\'Main\',\'q\',\'IPList\'); Form_Submit(\'Main\'); return false;">Sort List</a>
		<a href="#" class="Chapter_Link_A" onclick="if (confirm(\'Are you sure you want to clear the (IPList) list?\')) { Form_Append_HiddenElement(\'Main\',\'do\',\'ClearList\'); Form_Append_HiddenElement(\'Main\',\'q\',\'IPList\'); Form_Submit(\'Main\'); } return false;">Clear List</a>
          Delete By IP Address</p>
          <p class="Text">This feature deletes comments made by users from a specified IP addresses. <br>
          The associated file is cc-IPList.txt. See the <a href="__PageURL_Usage__" target="_blank" class="Text_A">usage</a> page for additional details.</p>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE" style="margin-top:12px;">
            <tr>
              <td valign="top" class="form_Field-Label_TD">Undesired IP Addresses 
              <p id="IPList_Msg">&nbsp;</p></td>
              <td class="form_Field-Data_TD"><textarea name="IPList" class="form_Field-TEXTAREA" id="IPList" onkeyup="SayListChanged(\'IPList\')" onchange="SayListChanged(\'IPList\')" onFocus="setInterval(\'SayListChanged(\\\'IPList\\\')\',2000);">__IPList__
</textarea></td>
            </tr>
        </table>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE">
            <tr>
              <td class="form_Field-Label_TD">&nbsp;</td>
              <td class="form_Field-Data_TD"><input name="Submit_SaveList_IPList" type="submit" class="form_SUBMIT" id="Submit_SaveList_IPList" value="Save">
              <input name="Submit_DeleteComments_IPList" type="submit" class="form_SUBMIT" id="Submit_DeleteComments_IPList" value="Delete" style="margin-left:45px;">
              <input name="Submit_DeleteBatch_IPList" type="submit" class="form_SUBMIT" id="Submit_DeleteBatch_IPList" value="Batch"></td>
            </tr>
        </table>
          <p class="Chapter">Delete All Comments</p>
          <p class="Text">This feature deletes all comments in the Gallery.</p>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE" style="margin-top:12px;">
            <tr>
              <td class="form_Field-Label_TD">&nbsp;</td>
              <td class="form_Field-Data_TD"><input name="Submit_DeleteComments_All" type="submit" class="form_SUBMIT_NoWidth" id="Submit_DeleteComments_All" value="Delete All Comments"></td>
            </tr>
          </table>
          <p class="Chapter">Analytics</p>
          <p class="Section">Find Comments </p>
          <p class="Text"> Search the Comment\'s 
            <select name="FindComments_Column" class="form_Field-SELECT" id="FindComments_Column">
              <option value="subject" selected>Subject</option>
              <option value="comment">Body</option>
              <option value="author">Author</option>
              <option value="host">IP Address</option>
              <option value="id">Comment ID</option>
            </select>            
            <br>
            Display a comment if this value equals<span class="form_Field-Data_TD">:           
            <input name="FindComments_Value" type="text" class="form_Field-TEXT_NoWidth" id="FindComments_Value" value="__FindComments_Value__" size="20" maxlength="100">
        </span> (Wildcards &quot;%&quot; and &quot;_&quot; are allowed.)<br>
          Limit the returned list to a maximum of <span class="form_Field-Data_TD">
          <input name="FindComments_Max" type="text" class="form_Field-TEXT_NoWidth" id="FindComments_Max" value="__FindComments_Max__" size="8" maxlength="10">
          </span>comments.</p>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE" style="margin-top:12px;">
            <tr>
              <td class="form_Field-Label_TD">&nbsp;</td>
              <td class="form_Field-Data_TD"><input name="Submit_FindComments" type="submit" class="form_SUBMIT_NoWidth" id="Submit_FindComments" value="Find Comments"></td>
            </tr>
          </table>
          <p class="Section">Find Duplicates</p>
          <p class="Text"> Search the Comment\'s
            <select name="FindDuplicates_Column" class="form_Field-SELECT" id="FindDuplicates_Column">
              <option value="subject" selected>Subject</option>
              <option value="comment">Body</option>
              <option value="author">Author</option>
              <option value="host">IP Address</option>
            </select>
            <br>
            Display a comment if this value equals<span class="form_Field-Data_TD">:
            <input name="FindDuplicates_Value" type="text" class="form_Field-TEXT_NoWidth" id="FindDuplicates_Value" value="__FindDuplicates_Value__" size="20" maxlength="100">
            </span> (Wildcards &quot;%&quot; and &quot;_&quot;  are allowed.)<br>
            Display a comment if this field contains<span class="form_Field-Data_TD">
            <input name="FindDuplicates_Count" type="text" class="form_Field-TEXT_NoWidth" id="FindDuplicates_Count" value="__FindDuplicates_Count__" size="8" maxlength="10" style="margin-left:1px;">
            </span>or more duplicates.<br>
            Limit the returned list to a maximum of <span class="form_Field-Data_TD">
            <input name="FindDuplicates_Max" type="text" class="form_Field-TEXT_NoWidth" id="FindDuplicates_Max" value="__FindDuplicates_Max__" size="8" maxlength="10">
        </span>comments.</p>
          <table width="500" border="0" cellpadding="0" cellspacing="0" class="form_Field_TABLE" style="margin-top:12px;">
            <tr>
              <td class="form_Field-Label_TD">&nbsp;</td>
              <td class="form_Field-Data_TD"><input name="Submit_FindDuplicates" type="submit" class="form_SUBMIT_NoWidth" id="Submit_FindDuplicates" value="Find Duplicates"></td>
            </tr>
          </table>
    </form></td>
  </tr>
</table>

<script language="javascript">

// Globals //

// The original contents of a list. This is used to determine if a list has actually changed or not.
// This is necessary because the "onchange" event does not fire at the right time.
var SubjectList_Contents = \'\';
var BodyList_Contents = \'\';
var AuthorList_Contents = \'\';
var IPList_Contents = \'\';

// Flags to indicate if a "List Change" message is being displayed or not.
var SubjectList_Changed = false;
var BodyList_Changed = false;
var AuthorList_Changed = false;
var IPList_Changed = false;


// Functions //

function Init() {
	/*
	Purpose
		Used to initialize Javascript on this page.
	*/
	
	// Get a reference to our "Main" form for convenience.
	var frm = document.forms[\'Main\'];

	// Load *_Contents variables.
	SubjectList_Contents = document.getElementById(\'SubjectList\').value;
	BodyList_Contents = document.getElementById(\'BodyList\').value;
	AuthorList_Contents = document.getElementById(\'AuthorList\').value;
	IPList_Contents = document.getElementById(\'IPList\').value;

	// Set the default values in drop-downs.
	Select_SetSelected(\'FindComments_Column\', \'__FindComments_Column__\');
	Select_SetSelected(\'FindDuplicates_Column\', \'__FindDuplicates_Column__\');

	// Trap the Enter key when made in the form.
	//document.forms[\'Main\'].onkeypress = DisableEnterKey;
	

	// Register event handlers...
	
	// Don\'t submit the form if Enter is pressed while these textboxes have focus.

	frm.elements[\'DBServer\'].onkeypress = IgnoreEnterKey;
	if (frm.elements[\'DBServer\'].captureEvents) frm.elements[\'DBServer\'].captureEvents(Event.KEYPRESS);

	frm.elements[\'DBUsername\'].onkeypress = IgnoreEnterKey;
	if (frm.elements[\'DBUsername\'].captureEvents) frm.elements[\'DBUsername\'].captureEvents(Event.KEYPRESS);

	frm.elements[\'DBPassword\'].onkeypress = IgnoreEnterKey;
	if (frm.elements[\'DBPassword\'].captureEvents) frm.elements[\'DBPassword\'].captureEvents(Event.KEYPRESS);

	frm.elements[\'DBName\'].onkeypress = IgnoreEnterKey;
	if (frm.elements[\'DBName\'].captureEvents) frm.elements[\'DBName\'].captureEvents(Event.KEYPRESS);

	frm.elements[\'TablePrefix\'].onkeypress = IgnoreEnterKey;
	if (frm.elements[\'TablePrefix\'].captureEvents) frm.elements[\'TablePrefix\'].captureEvents(Event.KEYPRESS);

	frm.elements[\'ColumnPrefix\'].onkeypress = IgnoreEnterKey;
	if (frm.elements[\'ColumnPrefix\'].captureEvents) frm.elements[\'ColumnPrefix\'].captureEvents(Event.KEYPRESS);
	
	// FIND: Submit the form with injected form values if Enter is pressed while these textboxes have focus.
	
	frm.elements[\'FindComments_Value\'].onkeypress = h_Form_Submit_Find;
	if (frm.elements[\'FindComments_Value\'].captureEvents) frm.elements[\'FindComments_Value\'].captureEvents(Event.KEYPRESS);

	frm.elements[\'FindComments_Max\'].onkeypress = h_Form_Submit_Find;
	if (frm.elements[\'FindComments_Max\'].captureEvents) frm.elements[\'FindComments_Max\'].captureEvents(Event.KEYPRESS);

	// FIND DUPLICATES: Submit the form with injected form values if Enter is pressed while these textboxes have focus.
	
	frm.elements[\'FindDuplicates_Value\'].onkeypress = h_Form_Submit_FindDuplicates;
	if (frm.elements[\'FindDuplicates_Value\'].captureEvents) frm.elements[\'FindDuplicates_Value\'].captureEvents(Event.KEYPRESS);

	frm.elements[\'FindDuplicates_Count\'].onkeypress = h_Form_Submit_FindDuplicates;
	if (frm.elements[\'FindDuplicates_Count\'].captureEvents) frm.elements[\'FindDuplicates_Count\'].captureEvents(Event.KEYPRESS);

	frm.elements[\'FindDuplicates_Max\'].onkeypress = h_Form_Submit_FindDuplicates;
	if (frm.elements[\'FindDuplicates_Max\'].captureEvents) frm.elements[\'FindDuplicates_Max\'].captureEvents(Event.KEYPRESS);

}


function HasListChanged(ListName) {
	/*
	Purpose
		Used to determine if a list has changed from it\'s original value.
		This is necessary because the INPUT elements\'s onchange does not fire at the right time.
	Parameters	
		ListName (string) = The name of the list we\'re working with.
	Return (boolean)
		Returns true of the specified list has been changed.
		Otherwise, returns false.
	Notes
		This should be called by an input element\'s "onkeydown" event.
	Version History
		1.00 2009.6.10 Tested by Russ Tanner
	*/
	
	if (ListName == \'SubjectList\') {
		if (SubjectList_Contents == document.getElementById(ListName).value) return false;
		else return true;
	}
	if (ListName == \'BodyList\') {
		if (BodyList_Contents == document.getElementById(ListName).value) return false;
		else return true;
	}
	if (ListName == \'AuthorList\') {
		if (AuthorList_Contents == document.getElementById(ListName).value) return false;
		else return true;
	}
	if (ListName == \'IPList\') {
		if (IPList_Contents == document.getElementById(ListName).value) return false;
		else return true;
	}
	
	
}


function SayListChanged(ListName) {
	/*
	Purpose
		Used to display an indicator that text has changed in the specified List.
	Parameters	
		ListName (string) = The name of the list we\'re working with.
	Version History
		1.00 2009.6.10 Tested by Russ Tanner
	*/
	
	// Determine if the list has changed or not.
	if (! HasListChanged(ListName)) return;
	
	// If the specified InputID was already done, just exit. There\'s no need to risk flicker.
	// If the ID was not done, set the corresponding flag to true.
	if (ListName == \'SubjectList\') {
		if (SubjectList_Changed) return;
		SubjectList_Changed = true;
	}
	if (ListName == \'BodyList\') {
		if (BodyList_Changed) return;
		BodyList_Changed = true;
	}
	if (ListName == \'AuthorList\') {
		if (AuthorList_Changed) return;
		AuthorList_Changed = true;
	}
	if (ListName == \'IPList\') {
		if (IPList_Changed) return;
		IPList_Changed = true;
	}

	// Config
	var Msg = \'The list has changed.<br />Remember to save it.\';		// The message to display.
	var MsgColor = \'red\';			// The color of the message text.
	var BGColor = \'#FFEEEE\';		// The background color to set the input element to.
	
	// Map the list name to: (1) A message display object, and (2) An input element.
	var oMsg = document.getElementById(ListName + \'_Msg\');
	var oInput = document.getElementById(ListName);

	// Display the message.
	oMsg.innerHTML = Msg;
	oMsg.style.color = MsgColor;
	
	// Change the background color of the input element.
	oInput.style.backgroundColor = BGColor;
	
}


function Select_SetSelected(ID, DefaultOption) {
	/*
	Purpose
		Used to select the proper items in drop-down boxes (select tags).
	Parameters
		ID (string) = The ID of the drop-down we\'re working with.
		DefaultOption (string) = The option we want selected.
	Return (boolean)
		If the option was successfully set, function returns true.
		If the opeion was not set, function return false.
	Version History
		1.00 2009.6.23 Tested by Russ Tanner on IE/FF.
	*/
	
	// Get a reference to the object we\'re working with.
	var o = document.getElementById(ID);
	
	// Iterate all of the options in the specified drop-down.
   	for (var i = 0; i < o.options.length; i++) {
		// Found the option we\'re looking for. Make it the selected option.
       	if (o.options[i].value == DefaultOption) {
           	o.options[i].selected = true;
            return true;
   	    }
    }
	
	// Matching value was not found, return false.
	return false;

}


function DisableEnterKey(evt) {

   // Get the event object (cross-browser).
   var evt = (evt) ? evt : ((event) ? event : null);

   // Get the source object (cross-browser).
   var node = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : null);

   // If Enter pressed on a Textbox, return false.
   if ((evt.keyCode == 13) && (node.type=="text"))  {return false;}

   return true;

}


function h_Form_Submit_Find(e) {
	/*
	Purpose
		Event handler used specifically to submit the form from any of the Find textboxes.
	Version History
		1.00 2009.6.29
	*/
	
	// Get a common event variable (cross-browser).
	if (!e) var e = window.event;

	// Get the key code that was pressed (cross-browser).	
	if (e.keyCode) kc = e.keyCode;
	else if (e.which) kc = e.which;

	// The Enter key was pressed
	if (kc == 13) {

		// Make the server think the Submit_FindComments submit button was clicked.
		Form_Append_HiddenElement(\'Main\',\'Submit_FindComments\',\'Submit_FindComments\');
	
		// Submit the form.
		Form_Submit(\'Main\')

		// Don\'t perform the default action.
		return false;
	}
	
	// The Enter key was not pressed. Go ahead with the default action.
	return true;
	
}


function h_Form_Submit_FindDuplicates(e) {
	/*
	Purpose
		Event handler used specifically to submit the form from any of the Find-Duplicates textboxes.
	Version History
		1.00 2009.6.29
	*/
	
	// Get a common event variable (cross-browser).
	if (!e) var e = window.event;

	// Get the key code that was pressed (cross-browser).	
	if (e.keyCode) kc = e.keyCode;
	else if (e.which) kc = e.which;

	// The Enter key was pressed
	if (kc == 13) {

		// Make the server think the Submit_FindDuplicates submit button was clicked.
		Form_Append_HiddenElement(\'Main\',\'Submit_FindDuplicates\',\'Submit_FindDuplicates\');
	
		// Submit the form.
		Form_Submit(\'Main\')

		// Don\'t perform the default action.
		return false;
	}
	
	// The Enter key was not pressed. Go ahead with the default action.
	return true;
	
}

</script>
',
			'Menu'							=> '

<p class="Menu"><a href="__PageURL_Main__" class="Menu_A">Comment Management</a> | <a href="__PageURL_Usage__" target="_blank" class="Menu_A">Usage</a> | <a href="__PageURL_SysReq__" target="_blank" class="Menu_A">System Requirements</a> | <a href="__PageURL_License__" target="_blank" class="Menu_A">License</a> | <a href="__PageURL_About__" target="_blank" class="Menu_A">About</a> </p>',
			'Msg'							=> '
<table border="0" align="center" cellpadding="0" cellspacing="0" class="form_TABLE" style="margin-top:24px;">
  <tr>
    <td class="form_TD"><p class="Chapter" style="">__Title__</p>
      <p class="Text">__Message__</p>
      <p class="Text">Click here to <a href="__URLContinue__" class="Text_A">continue...</a></p></td>
  </tr>
</table>',
			'MsgFile'						=> '
<form action="__PageURL_Main__" method="post" id="Main" name="Main"><table border="0" align="center" cellpadding="0" cellspacing="0" class="form_TABLE" style="margin-top:24px;">
__FormFields__
  <tr>
    <td class="form_TD"><p class="Chapter" style="">__Title__</p>
        <p class="Text">__PreBody__</p>
      <p class="Text">
        <?php if (is_file(\'__FileName__\')) readfile(\'__FileName__\'); ?>
      </p>
      <p class="Text">__PostBody__</p></td>
  </tr>
</table></form>',
			'Error'							=> '
<table border="0" align="center" cellpadding="0" cellspacing="0" class="form_TABLE" style="margin-top:24px;">
  <tr>
    <td class="form_TD"><p class="Chapter_Error">__Title__</p>
    <p class="Text">__Message__</p>
    <p class="Text">Click here to <a href="__URLContinue__" class="Text_A">continue...</a></p></td>
  </tr>
</table>',
			'SysReq'						=> '

<p class="Text">Forwarding to Comment Cruncher <a href="__PageURL_SysReq__" class="Text_A">System Requirements </a>...</p>

<form name="MyForm" method="post" action="__PageURL_SysReq__">
</form>

<script type="text/javascript">
setTimeout(\'document.MyForm.submit();\', 100);
</script>
',
			'Usage'							=> '

<p class="Text">Forwarding to Comment Cruncher <a href="__PageURL_Usage__" class="Text_A">Usage</a>...</p>

<form name="MyForm" method="post" action="__PageURL_Usage__">
</form>

<script type="text/javascript">
setTimeout(\'document.MyForm.submit();\', 100);
</script>
',
			'CSS'							=> '.Header {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 18px;
	color: #F4E9DD;
	background-color: #684522;
	text-align: center;
	padding-top: 4px;
	padding-right: 0px;
	padding-bottom: 4px;
	padding-left: 0px;
	font-weight: bold;
	margin: 0px;
}
.SubHeader {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 14px;
	font-weight: bold;
	color: #F4E9DD;
	background-color: #BE7E3F;
	padding-top: 2px;
	padding-right: 0px;
	padding-bottom: 2px;
	padding-left: 0px;
	text-align: center;
	margin-top: 0px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.Title {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 16px;
	font-weight: bold;
	color: #F4E9DD;
	background-color: #BE7E3F;
	text-align: center;
	padding-top: 4px;
	padding-right: 0px;
	padding-bottom: 4px;
	padding-left: 0px;
	margin-top: 16px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
	border: 1px none #684522;
}

.Chapter {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 14px;
	font-weight: bold;
	color: #684522;
	background-color: #F4E9DD;
	padding-top: 2px;
	padding-right: 0px;
	padding-bottom: 2px;
	padding-left: 8px;
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.Chapter_Error {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 14px;
	font-weight: bold;
	color: #684522;
	background-color: #EDC5C2;
	padding-top: 2px;
	padding-right: 0px;
	padding-bottom: 2px;
	padding-left: 8px;
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.Chapter_Confirmed {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 14px;
	font-weight: bold;
	color: #684522;
	background-color: #C2E2D7;
	padding-top: 2px;
	padding-right: 0px;
	padding-bottom: 2px;
	padding-left: 8px;
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.Chapter_Link_A {
	color: #BE7E3F;
	cursor: hand;
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-style: normal;
	font-weight: normal;
	padding-top: 0px;
	padding-right: 8px;
	padding-bottom: 0px;
	padding-left: 8px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: solid;
	border-top-color: #BE7E3F;
	border-right-color: #BE7E3F;
	border-bottom-color: #BE7E3F;
	border-left-color: #BE7E3F;
	float: right;
	margin-top: 2px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}

.Chapter_Menu_SUBMIT {
	color: #BE7E3F;
	background-color: transparent;
	cursor: pointer;
	cursor: hand;
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-style: normal;
	font-weight: normal;
	padding-top: 0px;
	padding-right: 2px;
	padding-bottom: 0px;
	padding-left: 2px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: solid;
	border-top-color: #BE7E3F;
	border-right-color: #BE7E3F;
	border-bottom-color: #BE7E3F;
	border-left-color: #BE7E3F;
	float: right;
	margin: 0px;
}
.Chapter_Menu_SUBMIT_NoBorder {
	color: #BE7E3F;
	cursor: hand;
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-style: normal;
	font-weight: normal;
	padding-top: 0px;
	padding-right: 2px;
	padding-bottom: 0px;
	padding-left: 2px;
	margin: 0px;
	float: right;
	background-color: transparent;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: none;
	border-left-style: none;
}
.Chapter_Menu {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	color: #BE7E3F;
	font-weight: normal;
	float: right;
	margin-top: 0px;
	margin-right: 12px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.Chapter_Menu_A {
	color: #BE7E3F;
}

.Section {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: bold;
	color: #684522;
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 20px;
	border-top-width: 1px;
	border-right-width: 1px;
	border-bottom-width: 1px;
	border-left-width: 1px;
	border-top-style: none;
	border-right-style: none;
	border-bottom-style: solid;
	border-left-style: none;
	border-top-color: #684522;
	border-right-color: #684522;
	border-bottom-color: #684522;
	border-left-color: #684522;
}

.Subsection {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: bold;
	color: #684522;
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 20px;
}

.Text {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	color: #684522;
	margin-top: 8px;
	margin-right: 20px;
	margin-bottom: 0px;
	margin-left: 20px;
}
.Text_A {
	color: #669933;
}
.Menu {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #BE7E3F;
	text-align: center;
	margin-top: 0px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
	padding-top: 4px;
	padding-right: 0px;
	padding-bottom: 4px;
	padding-left: 0px;
}
.Menu_A {
	color: #BE7E3F;
}
.Code {
	font-family: "Courier New", Courier, monospace;
	font-size: 11px;
	color: #333333;
	margin-top: 8px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 20px;
}


.form_TABLE {
	border: 2px solid #BE7E3F;
	width: 700px;
	margin-top: 0px;
	margin-right: auto;
	margin-bottom: 0px;
	margin-left: auto;
}
.form_TD {
	padding-top: 0px;
	padding-right: 20px;
	padding-bottom: 12px;
	padding-left: 20px;
}
.form_FORM {
	padding: 0px;
	margin: 0px;
}
.form_Field_TABLE {
	padding: 0px;
	margin-top: 2px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
	width: 620px;
}
.form_Field-Label_TD {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: bold;
	color: #BE7E3F;
	text-align: right;
	padding-top: 0px;
	padding-right: 8px;
	padding-bottom: 0px;
	padding-left: 0px;
	width: 220px;
}
.form_Field-Data_TD {
	width: 450px;
	padding: 0px;
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #666666;
}
.form_Field-Desc {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #999999;
	margin-top: 0px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 8px;
}
.form_Field-TEXT {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #666666;
	border: 1px solid #BE7E3F;
	width: 200px;
}
.form_Field-TEXT_NoWidth {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #666666;
	border: 1px solid #BE7E3F;
}
.form_Field-TEXT-Chapter {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #BE7E3F;
	border: 1px solid #BE7E3F;
	float: right;
	margin-top: 1px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
}
.form_Field-TEXTAREA {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #666666;
	height: 100px;
	width: 200px;
	border: 1px solid #BE7E3F;
	text-align: left;
}
.form_Field-SELECT {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #666666;
	border: 1px solid #BE7E3F;
}

.form_SUBMIT {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #684522;
	width: 50px;
}
.form_SUBMIT_NoWidth {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #684522;
}





.Top {
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 7pt;
	font-style: italic;
	font-weight: bold;
	color: #3F7C79;
	display: block;
	margin: 0px;
}
.Top_A {
	color: #3F7C79;
}
.Credit {
	font-family: Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #F4E9DD;
	float: right;
	text-align: right;
	margin-top: 3px;
	margin-right: 12px;
	margin-bottom: 0px;
	margin-left: -200px;
	width: 200px;
}
.Credit_A {
	color: #F4E9DD;
}
.Copyright {
	font-family: Tahoma, Verdana, sans-serif;
	font-size: 11px;
	font-style: normal;
	font-weight: normal;
	color: #A26B35;
	text-align: center;
}



ul {
	list-style-type: disc;
	margin-top: 8px;
	margin-bottom: 0px;
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #684522;
	list-style-position: inside;
}
li {
}

ol {
	margin-top: 12px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 0px;
	font-family: Verdana, Arial, Helvetica, sans-serif;
	font-size: 11px;
	font-weight: normal;
	color: #684522;
}
blockquote {
	margin-top: 8px;
	margin-right: 0px;
	margin-bottom: 0px;
	margin-left: 20px;
}
.Chapter_Menu_SUBMIT:hover {
	text-decoration: underline;
}


a:link {
    text-decoration: none;
}

a:visited {
    text-decoration: none;
}

a:hover {
    text-decoration: underline;
}

a:active {
    text-decoration: underline;
}
',
			'JS'							=> '
function Checkboxes_CountState(FormName, State) {
	/*
	Purpose
		Used to count the number of checkboxes within a specified form
		that are set to a specified state (true or false).
	Parameters
		FormName (string) = Only checkboxes within this form are counted.
		State (optional boolean) = To count the number of checked checkboxes,
			set this to true. To count unchecked checkboxes, set to false.
			Default: true
	Return (integer)
		Return the total number of checkboxes that are set to the same state as
		indicated by State.
	Version History
		1.00 2009.6.25 Tested by Russ Tanner on IE6/FF3.
	*/
	
	// Default State to true.
	if (typeof State == \'undefined\') State = true;
	
	var frm = document.forms[FormName];
	var c = frm.elements.length;
	var r = 0;
	for(var i=0; i<c; i++) {
		if (frm.elements[i].type == \'checkbox\') r += (frm.elements[i].checked == State) ? 1 : 0;
	}
	
	return r;
}


function Checkboxes_InvertAll(FormName) {
	/*
	Purpose
		Used to invert the value of all checkboxes in the specified form.
	Parameters
		FormName (string) = The name of the form we\'re working with.
	Version History
		1.00 2009.6.24 Tested by Russ Tanner on IE6/FF3.
	*/
	
	var frm = document.forms[FormName]
	var c = frm.elements.length;
	for(var i=0; i<c; i++) {
		if (frm.elements[i].type == \'checkbox\') frm.elements[i].checked = !frm.elements[i].checked;
	}
}


function Checkboxes_SetAll(FormName, Value) {
	/*
	Purpose
		Used to set the value of all checkboxes in the specified form.
	Parameters
		FormName (string) = The name of the form we\'re working with.
		Value (boolean) = The value we\'re going to set all checkboxes to .
	Version History
		1.00 2009.6.24 Tested by Russ Tanner on IE6/FF3.
	*/
	
	var frm = document.forms[FormName]
	var c = frm.elements.length;
	for(var i=0; i<c; i++) {
		if (frm.elements[i].type == \'checkbox\') frm.elements[i].checked = Value;
	}
}


function Form_Append_HiddenElement(FormName, Name, Value) {
	/*
	Purpose
		Used to append a hidden form field to an existing form.
	Parameters
		FormName (string) = The name of the form we\'re inserting this form field into.
		Name (string) = The "name" attribute that we\'ll assign to the field.
		Value (string) = The "value" attribute that we\'ll assign to the field.
	Version History
		1.00 2009.6.24 Tested by Russ Tanner on IE6/FF3.
	*/
	
	// Reference to the element we\'re building.
	var o;

	// Create the element.
	o = document.createElement(\'input\');
	
	// Define attributes for the element.
	o.type = \'hidden\';
	o.name = Name;
	o.value = Value;
	o.defaultValue = Value;
	
	// Insert the element into the form.
	document.forms[FormName].appendChild(o);
}


function Form_Submit(FormName, Action){
	/*
	Purpose
		Submits a form in a simple, generic way.
	Parameters
		FormName (string) = The name of the form we want to submit.
		Action (optional string) = If specified this value is assigned to the .action method
			of the form before it is submitted.
	Version History
		1.00 2009.6.24 Tested by Russ Tanner on IE6/FF3.
	*/

	// Get a rererence to the specified form.
	var frm = document.forms[FormName]
	
	// Set the form\'s action and submit it.
	if (typeof Action != \'undefined\') frm.action = Action;
	frm.submit();
}


function KeyPress_Filter(e, Key, Action) {
	/*
	Purpose
		Used to cancel specified key presses.
		Call this function from a form elements\' onKeyPress event and it will return false
		when any of the specified keys are pressed.
	Parameters
		e (object) = The event object provided by the element we\'re working with.
		Key (optional string, number, array) = The key or array of keys we want to filter.
			In the context of this parameter, "filter" means to either allow or disallow,
				depending on the value of Action.
			If this is a string, if the key is pressed that generates the specified string,
				then that key is filtered (allowed/disallowed depending on Action).
			If this is a number, if the key is pressed that generates a keycode matching
				this number, then that key is filtered (allowed/disallowed depending on Action).
			If this is an array, then array elements can be either strings or numbers.
				In this case each element represents a key that is either allowed or 
				disallowed depending on the value of Action.
			Default: 13
		Action (optional boolean) = This is the value that this function returns when a pressed key
			matches a key specified in Key.
			Default: false
	Return (boolean)
		If the specified key (or any of the keys specified in an array) is pressed,
			function returns whatever value is specified in Action.
	Version History
		1.00 2009.6.28 Unfinished
	*/
	
	// Default parameters.
	if (typeof Key == \'undefined\') Key = 13;
	if (typeof Action == \'undefined\') Action = false;

	// Get the event object (cross-browser).
	var evt = (evt) ? evt : ((event) ? event : null);

	// Get the source object (cross-browser).
	var node = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : null);

	alert(node.name);
	
	return false;
	
}


function IgnoreEnterKey(e) {
	/*
	Purpose
		Used to cause a form element to ignore the Enter key.
	Return (boolean)
		Returns false when the enter key is hit on a hooked element.
	Version History
		1.00 2009.06.29 Tested by Russ Tanner on IE6/FF3.
	*/

	// Get a common event variable (cross-browser).
	if (!e) var e = window.event;

	// Get the key code that was pressed (cross-browser).	
	if (e.keyCode) kc = e.keyCode;
	else if (e.which) kc = e.which;

	// Don\'t let this event propagate (performance optimization).
	if (e.stopPropagation) e.stopPropagation();

	// Return false if the enter key was pressed.
	return ! (kc == 13);
	
}

'
		);


	} // End of function Init()

}



// Handlers //


function h_DeleteComments() {
	/*
	Purpose
		Handler for deleting comments from the Gallery using a specified list (ListName).
		This handler should also save the comments for the specified list.
	HTTP Parameters
		q (GET/POST string) = This is the name of the list we want to use.
			This is not a file name. It is a proper list name which gets mapped to a file name.
			Valid values:
				"SubjectList", "BodyList", "AuthorList", "IPList"
		(List) = The list we are working with must be specified because it is automatically saved
			by this function. The name of the specified list must match the name specified in "q".
			Each item in the list must be delimited by 10, 13, or 13/10.
		(Database Parameters) = All standard database parameters must be specified.
	Return (integer)
		Returns a status code:
			STATUS_OK
			STATUS_ParameterMissing
			STATUS_InvalidListName
			STATUS_ErrorReadingListFile
			STATUS_ErrorWritingListFile
			STATUS_DatabaseParameterMissing
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// The title for all dialogs.
	$TITLE = 'Delete Comments';
	
	// Retrieve HTTP parameter: "q"
	$ListName = GetInput('q');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($ListName)) {
		SayPage__Error_ByStatus('Parameter Missing: "q"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Validate the list name, save the passed list, and retrieve the list as a clean array.
	$ListArray = ListName_ValidateAndSave($ListName);
	// Error Trap: The specified list is invalid. Display an error message and return a status code.
	if (is_int($ListArray)) return SayPage__Error_ByStatus($ListArray, $TITLE);

	// Refine the title now that we know it's valid.
	$TITLE = "Delete Comments Using the \"$ListName\" List";

	// Get the database parameters.
	$DB = Parm_Get_DB();
	// Error Trap: Display an error message and return a status code.
	if (is_int($DB)) return SayPage__Error_ByStatus($DB, $TITLE);

	// Delete comments from Gallery.
	$e = Gallery_Comments_Delete($DB, $ListName);
	// An error occurred. Display an error message.
	if (is_int($e)) return SayPage__Error_ByStatus($e, $TITLE);

	// Display status page and return status code OK.
	$Msg = Msg_Build('DeleteComments', array(
		'ListName'			=> $ListName,
		'CountQueries'		=> number_format($e['QueryCount']),
		'CommentsDeleted'	=> number_format($e['CountBefore'] - $e['CountAfter']),
		'RowsDeleted'		=> number_format($e['CountDeleted']),
		'CountDeleted'		=> number_format($e['CountDeleted']),
		'CountBefore'		=> number_format($e['CountBefore']),
		'CountAfter'		=> number_format($e['CountAfter'])
	));
	SayPage__Confirmed($Msg, $TITLE);

	return STATUS_OK;

}


function h_DeleteBatch() {
	/*
	Purpose
		Handler for deleting comments from Gallery in batch mode.
		This function provides a solution for situations where PHP times out before an entire list can be processed.
		The function h_DeleteBatch_Begin() begins this process. This function continues each step
			until the batch process is complete.
		This handler iterates each item in the specified list and generates an auto-submitting page for each one.
	Return (integer)
		Returns a status code:
			STATUS_OK
			STATUS_InvalidListName
			STATUS_ErrorReadingListFile
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Config
	// The title for all dialogs.
	$TITLE = 'Delete Comments (batch mode)';

	// Retrieve all of the POSTed variables that we're interested in.
	$FormFields = array(
		'do'					=> null,
		'ListName'				=> null,
		'DBServer'				=> null,
		'DBUsername'			=> null,
		'DBPassword'			=> null,
		'DBName'				=> null,
		'TablePrefix'			=> null,
		'ColumnPrefix'			=> null,
		'Index'					=> null,
		'Title'					=> null,
		'CountQuery'			=> null,
		'CountDeleted'			=> null,
		'CountCommentsBegin'	=> null
	);
	$FormFields = Parm_FillArray($FormFields, null, null, 'P');

	// Trap: The user clicked "Cancel".
	$x = GetPOST('Submit_Cancel');
	if (! is_null($x)) {

		// Display the final message.
		$e = h_DeleteBatch_End($FormFields, true);
		// Error Trap: Display an error message and return a status code.
		if ($e != STATUS_OK) return SayPage__Error_ByStatus($e, $TITLE);

		return STATUS_OK;

	}

	// Error Trap: The specified list name is invalid (security). Display an error message and return a status code.
	if (! ListName_IsValid($FormFields['ListName'])) return SayPage__Error_ByStatus(STATUS_InvalidListName, $TITLE);

	// Get the current line from the list file.
	$List = List_Line_GetCurrent($FormFields['ListName'], $FormFields['Index']);
	// Error Trap: Display an error message and return a status code.
	if (is_int($List)) return SayPage__Error_ByStatus($List, $TITLE);

	// Delete comments.
	$e = Gallery_Comments_Delete_ByMethod($FormFields, cfg::$map_ListToMethod[$FormFields['ListName']], $List);
	// Error Trap: Display an error message and return a status code.
	if ($e['Status'] != STATUS_OK) return SayPage__Error_ByStatus($e['Status'], $TITLE);

	// Success. Comments were deleted. Increment counts.
	$FormFields['CountQuery'] += 1;
	$FormFields['CountDeleted'] += $e['CountDeleted'];

	// Get the next line from the specified list that contains a value.
	$List = List_Line_GetNext($FormFields['ListName'], $FormFields['Index']);
	// Error Trap: Error reading list file. Display an error message and return a status code.
	if (is_int($List)) return SayPage__Error_ByStatus($List, $TITLE);

	// No lines remain in the list (they've all been processed).
	// Display the batch's final message.
	if (is_null($List['Line'])) {

		// Display the final message.
		$e = h_DeleteBatch_End($FormFields);
		// Error Trap: Display an error message and return a status code.
		if ($e != STATUS_OK) return SayPage__Error_ByStatus($e, $TITLE);

		return STATUS_OK;

	}

	// Lines to remain in the list to be deleted.
	// Update the index to point to the next one.
	$FormFields['Index'] = $List['Index'];

	// Create the HTML form fields we're embedding into the "Batch" form.
	$FormFieldsTags = Tags_Build_INPUT_Hidden($FormFields);

	// Build the "Message" token for the "Batch" template.
	$MsgToken = Msg_Build('DeleteComments_InProcess', array(
		'String'		=> htmlspecialchars($List['Line']),
		'ListName'		=> htmlspecialchars($FormFields['ListName'])
	));

	// Build the "Batch" template.
	$Template = Template_Get('Batch');
	$Tokens = array_merge(cfg::$GlobalTokens, array(
		'Title'				=> htmlspecialchars($FormFields['Title']),
		'Message'			=> $MsgToken,
		'FormFields'		=> $FormFieldsTags,
		'SubmitDelay'		=> cfg::$def_Batch_SubmitDelay
	));
	$BatchTemplate = ExpandTokens($Template, $Tokens);

	// Display the "Bones" template.
	print Template_Build__Bones($BatchTemplate);

	return STATUS_OK;

}


function h_DeleteBatch_Begin() {
	/*
	Purpose
		Used to begin a batch-mode comment-delete operation.
	HTTP Parameters
		q (GET/POST string) = This is the name of the list we're using.
			This is not a file name. It is a proper list name which gets mapped to a file name.
			Valid values:
				"SubjectList", "BodyList", "AuthorList", "IPList"
	Return (integer)
		Returns a status code.
		Possible values are:
			STATUS_OK
			STATUS_ParameterMissing
			STATUS_DatabaseParameterMissing
			STATUS_InvalidListName
			STATUS_ErrorReadingListFile
			STATUS_ErrorWritingListFile
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
			STATUS_ListIsEmpty
	Notes
		This function stuffs all of the information necessary to complete the batch process into form INPUT variables.
			List of variables that are posted:
				"do" = The command that we want to use to tell the command processor that we're processing a batch file.
				"ListName" = The name of the list we're using to delete comments.
					This can be any valid list name as specified in cfg:: map_ListToMethod.
				"Index" = The zero-based index into the list (ListName) that we are currently processing.
				"Title" = The title used for all dialogs.
				"CountQuery" = The running total of queries that have been performed.
				"CountDeleted" = The running total number of rows that have been deleted in the database.
				"CountCommentsBegin" = The total number of comments that existed at the beginning of the batch.
					This is used to calculate the total number of comments that were deleted at the end of the batch.
				All of the database parameters returned by Parm_Get_DB().
		The resulting page will post itself using Javascript.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Calculate the title for all dialogs.
	$TITLE = 'Delete Comments in Batch Mode';

	// Retrieve HTTP parameter: "q"
	$ListName = GetInput('q');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($ListName)) {
		SayPage__Error('Parameter Missing: "q"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Validate the list name, save the passed list, and retrieve the list as a clean array.
	$ListArray = ListName_ValidateAndSave($ListName);
	// Error Trap: The specified list is invalid. Display an error message and return a status code.
	if (is_int($ListArray)) return SayPage__Error_ByStatus($ListArray, $TITLE);

	// Create a better title. Do this after the list name has been validated.
	$TITLE = "Delete Comments in Batch Mode Using the \"$ListName\" List";

	// We need all the database parameters in order to build the hidden form fields.
	$DB = Parm_Get_DB();
	// Error Trap: Database parameter missing. Display an error message and return a status code.
	if (is_int($DB)) return SayPage__Error_ByStatus($DB, $TITLE);

	// Get the beginning comment count.
	$e = Gallery_Comments_GetCount($DB, $DB['TablePrefix']);
	// Error Trap: Display an error message and return a status code.
	if ($e['Status'] != STATUS_OK) return SayPage__Error_ByStatus($e['Status'], $TITLE);

	// Define the list of form fields we want to have.
	$FormFields = array(
		'do'					=> '_DeleteBatch',
		'ListName'				=> $ListName,
		'DBServer'				=> $DB['DBServer'],
		'DBUsername'			=> $DB['DBUsername'], 
		'DBPassword'			=> $DB['DBPassword'], 
		'DBName'				=> $DB['DBName'], 
		'TablePrefix'			=> $DB['TablePrefix'], 
		'ColumnPrefix'			=> $DB['ColumnPrefix'], 
		'Index'					=> '-1',
		'Title'					=> $TITLE,
		'CountQuery'			=> '0',
		'CountDeleted'			=> '0',
		'CountCommentsBegin'	=> $e['CommentCount']
	);

	// Get the first line from the specified list that contains a value.
	$List = List_Line_GetNext($FormFields['ListName'], $FormFields['Index']);
	// Error Trap: Error reading list file. Display an error message and return a status code.
	if (is_int($List)) return SayPage__Error_ByStatus($List, $TITLE);
	// Error Trap: The list is empty. Display an error message and return a status code.
	if (is_null($List['Line'])) {
		$eListName = htmlspecialchars($FormFields['ListName']);
		SayPage__Error("Cannot perform batch-delete because the \"{$eListName}\" list is empty.", $TITLE);
		return STATUS_ListIsEmpty;
	}
	// Set the index to the first line in the list that has a value.
	$FormFields['Index'] = $List['Index'];

	// Create the message for the batch dialog.
	$MsgToken = Msg_Build('DeleteComments_InProcess', array(
		'String'		=> htmlspecialchars($List['Line']),
		'ListName'		=> htmlspecialchars($FormFields['ListName'])
	));

	// Build the "Batch" template.
	$Template = Template_Get('Batch');
	$Tokens = array_merge(cfg::$GlobalTokens, array(
		'Title'				=> htmlspecialchars($FormFields['Title']),
		'Message'			=> $MsgToken,
		'FormFields'		=> Tags_Build_INPUT_Hidden($FormFields),
		'SubmitDelay'		=> cfg::$def_Batch_SubmitDelay
	));
	$BatchTemplate = ExpandTokens($Template, $Tokens);

	// Display the template and return OK status.
	print Template_Build__Bones($BatchTemplate);
	return STATUS_OK;

}


function h_DeleteBatch_End($FormFields, $Cancelled = false) {
	/*
	Purpose
		Used to display the final message for the batch-delete process.
	Parameters
		FormFields (associative array) = The form fields passed to h_DeleteBatch().
		Cancelled (optional boolean) = If true, the "Cancelled" message is prepended to the displayed message.
			Default: false
	Return (integer)
		Return a status code.
			Possible return values:
				STATUS_OK
				STATUS_ErrorConnectingToDatabase
				STATUS_ErrorSelectingDatabase
				STATUS_ErrorQueryingDatabase
	Version History
		1.00 2009.6.18
	*/

	// Get the ending comment count.
	$e = Gallery_Comments_GetCount($FormFields, $FormFields['TablePrefix']);	// FormFields contains DB credentials.
	// Error Trap: Return a status code.
	if ($e['Status'] != STATUS_OK) return $e['Status'];

	// Display status page.
	$Msg = Msg_Build('DeleteComments', array(
		'ListName'			=> htmlspecialchars($FormFields['ListName']),
		'CountQueries'		=> number_format(htmlspecialchars($FormFields['CountQuery'])),
		'CommentsDeleted'	=> number_format(htmlspecialchars($FormFields['CountCommentsBegin'] - $e['CommentCount'])),
		'RowsDeleted'		=> number_format(htmlspecialchars($FormFields['CountDeleted'])),
		'CountBefore'		=> number_format(htmlspecialchars($FormFields['CountCommentsBegin'])),
		'CountAfter'		=> number_format($e['CommentCount'])
	), ($Cancelled ? cfg::$Msgs['DeleteComments_Cancelled'] : cfg::$Msgs['DeleteComments_Complete']));
	SayPage__Confirmed($Msg, htmlspecialchars($FormFields['Title']));

	return STATUS_OK;

}


function h_DeleteComments_All() {
	/*
	Purpose
		Handler for clearing all comments in the Gallery database.
	HTTP Parameters
		(Database Parameters) = All standard database parameters must be specified.
	Return (integer)
		Returns a status code.
		Valid values:
			STATUS_OK
			STATUS_DatabaseParameterMissing
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
	Notes
		Before the list is actually cleared, a yes/no confirmation page is displayed to prevent accidents.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Config
	$TITLE = 'Delete All Comments';			// The title used for dialogs.

	// We are confirmed.
	if (GetInput('confirmed') === 'yes') {

		// Get the database parameters.
		$DB = Parm_Get_DB();
		// Error Trap: Display an error message and return a status code.
		if (is_int($DB)) return SayPage__Error_ByStatus($DB, $TITLE);

		// Delete all comments.
		$e = Gallery_Comments_DeleteAll($DB);
		// Error Trap: Display an error message and return a status code.
		if (is_int($e)) return SayPage__Error_ByStatus($e, $TITLE);

		// Success. Display confirmation message and return OK status code.
		$Msg = cfg::$Msgs['DeleteCommentsAll_Complete'] . ExpandTokens(cfg::$Msgs['DeleteComments'], array(
			'CountQueries'		=> number_format($e['QueryCount']),
			'CommentsDeleted'	=> number_format((int) $e['CountBefore'] - (int) $e['CountAfter']),
			'RowsDeleted'		=> number_format($e['CountDeleted']),
			'CountBefore'		=> number_format($e['CountBefore']),
			'CountAfter'		=> number_format($e['CountAfter'])
		));
		SayPage__Confirmed($Msg, $TITLE);
		return STATUS_OK;

	}

	// We are not confirmed. Display the "Confirm" page.
	else {

		// Get the database parameters.
		// We need these so they can be passed in the GET query string if the user clicks "Yes".
		$DB = Parm_Get_DB();
		// Error Trap: Return a status code.
		if (is_int($DB)) return SayPage__Error_ByStatus($DB, $TITLE);

		// Display the confirmation dialog and return OK status.
		SayPage__Confirm(
			$TITLE,
			'Are you sure you want to delete all comments in the database?',
			'Yes',
			$_SERVER['PHP_SELF'] . '?' . http_build_query(array_merge(
				array('do'=>'DeleteCommentsAll', 'confirmed'=>'yes'),
				$DB
			)),
			'No',
			$_SERVER['PHP_SELF']
		);
		return STATUS_OK;
	}

}


function h_FindComments() {
	/*
	Purpose
		Handler for finding and displaying comments from the Gallery.
	HTTP Parameters
		column (string) = The name of the column we're searching.
			This is the actual database field name without a prefix.
			Valid values: "subject", "comment", "author", "host"
				If any other value is specified, an error message will be displayed.
		value (string) = The value we're searching "column" for.
		max (integer) = The maximum number of results to return.
			An error will be displayed if:
				- This is < 1
				- This is > 2,000,000
				- This is not numeric
		(Database Parameters) = All standard database parameters must be specified.
	Return (integer)
		Returns a status code:
			STATUS_OK
			STATUS_ParameterMissing
			STATUS_InvalidParameter
			STATUS_ErrorReadingListFile
			STATUS_ErrorWritingListFile
			STATUS_DatabaseParameterMissing
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
	Version History
		1.00 2009.6.22 Tested by Russ Tanner in PHP 5.1.6.
	*/

	// The title for all dialogs.
	$TITLE = 'Find Comments';

	// Retrieve HTTP parameter: "column"
	$Column = GetInput('column');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($Column)) {
		SayPage__Error('Parameter Missing: "column"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Retrieve HTTP parameter: "value"
	$Value = GetInput('value');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($Value)) {
		SayPage__Error('Parameter Missing: "value"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Retrieve HTTP parameter: "max"
	$Max = GetInput('max');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($Max)) {
		SayPage__Error('Parameter Missing: "max"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Error Trap: The specified column name is invalid. Display an error message and return a status code.
	if (! array_key_exists($Column, cfg::$map_ColumnToFriendly)) {
		SayPage__Error('Invalid Parameter: Column', $TITLE);
		return SayPage__Error_ByStatus(STATUS_InvalidParameter, $TITLE);
	}

	// Error Trap: Invalid Parameter: Max, Display an error message and return a status code.
	if ((! is_numeric($Max)) || $Max < 1 || $Max > 2000000) {
		SayPage__Error('Invalid Parameter: Max<br />Max must be a value between 1 and 2,000,000.', $TITLE);
		return SayPage__Error_ByStatus(STATUS_InvalidParameter, $TITLE);
	}

	// Get the database parameters.
	$DB = Parm_Get_DB();
	// Error Trap: Display an error message and return a status code.
	if (is_int($DB)) return SayPage__Error_ByStatus($DB, $TITLE);

	// Open the database. We need a database link id in this function.
	$hDB = DB_Open_ByArray($DB);
	// Error Trap: Return a status code.
	if (is_int($hDB)) return SayPage__Error_ByStatus($hDB, $TITLE);

	// Get the SQL template we're using.
	$SQL = cfg::$map_MethodToSQL['Find'];

	// Build SQL from the SQL template.
	$SQL = str_replace('__TablePrefix__', mysql_real_escape_string($DB['TablePrefix'], $hDB), $SQL);
	$SQL = str_replace('__ColumnPrefix__', mysql_real_escape_string($DB['ColumnPrefix'], $hDB), $SQL);
	$SQL = str_replace('__Column__', mysql_real_escape_string($Column, $hDB), $SQL);
	$SQL = str_replace('__Max__', $Max, $SQL);
	$SQL = str_replace('__?__', mysql_real_escape_string($Value, $hDB), $SQL);

	// Query the database.
	$q = mysql_query($SQL, $hDB);
	// Error Trap: Error querying database. Return a status code.
	if (! $q) return SayPage__Error_ByStatus(STATUS_ErrorQueryingDatabase, $TITLE);

	// Optimize variables.
	$p = $DB['ColumnPrefix'];		// We're going to use this a lot later.

	// Load the template used to generate each record's HTML.
	$Template = Template_Get('FindRec');

	// General counter provides a value for the SN (serial number) token.
	$SN = 1;		// The first record is 1, not 0 (zero).

	// Empty the buffer file.
	file_put_contents(cfg::$Buffer_FileName, '');

	// Iterate the returned records.
	// We go through a lot of trouble here to avoid PHP memory limits on large result sets.
	// We will build each row's template and write it to a file instead of a variable (as we would normally do).
	while ($row = mysql_fetch_assoc($q)) {

		// Create a token list for the current record.
		$Tokens = array(
			'SN'			=> number_format($SN),
			'Subject'		=> strlen(trim($row[$p.'subject'])) > 0 ? htmlspecialchars($row[$p.'subject']) : '(no subject)',
			'Author'		=> htmlspecialchars($row[$p.'author']),
			'Date'			=> date(cfg::$DateFormat, $row[$p.'date']),
			'IP'			=> htmlspecialchars($row[$p.'host']),
			'ID'			=> htmlspecialchars($row[$p.'id']),
			'Body'			=> htmlspecialchars($row[$p.'comment']),
			'PageURL_Main'	=> cfg::$GlobalTokens['PageURL_Main']
		);
		
		// Build the template for the current record.
		$rowHTML = ExpandTokens($Template, $Tokens);

		// Append the current row's HTML to the buffer file (instead of memory).
		file_put_contents(cfg::$Buffer_FileName, $rowHTML, FILE_APPEND);

		// Increment our serial number.
		$SN += 1;

	}

	// Create search-criteria string for diplay at the top of the results page.
	$SearchCriteria = 'You searched for ' . cfg::$map_ColumnToFriendly[$Column] . ' = "' . htmlspecialchars($Value) . '"';

	// Trap: No matching records. Display message and return OK status code.
	if ($SN == 1) {
		SayPage__Msg($SearchCriteria . "<br />\nNo matching comments were found.", $TITLE);
		return STATUS_OK;
	}

	// Tokens for MsgFile template...
	// Create a match-count string.
	$MatchCount = number_format($SN - 1) . ' matches have been found.';
	// This is the "PreBody" token.
	$PreBody = $SearchCriteria . "<br />\n" . $MatchCount;
	// Make a simple "PostBody" token.
	$PostBody = '';
//	Save for possible update later.
//	$PostBody = '<p class="Text">Click here to <a href="__URLContinue__" class="Text_A">continue...</a></p>'."\n";
//	$PostBody = str_replace('__URLContinue__', cfg::$GlobalTokens['PageURL_Main'], $PostBody);

	// Create tokens.
	$Tokens = array(
		'Title'			=> $TITLE,
		'PreBody'		=> $PreBody,
		'PostBody'		=> $PostBody,
		'FileName'		=> cfg::$Buffer_FileName,
		'PageURL_Main'	=> cfg::$GlobalTokens['PageURL_Main'],
		'FormFields'	=> Tags_Build_INPUT_Hidden($DB)
	);
	$MsgFileTemplate = ExpandTokens(Template_Get('MsgFile'), $Tokens);

	// Build the bones and save it to a temporary PHP file.
	file_put_contents(cfg::$PHPTemp_FileName, Template_Build__Bones($MsgFileTemplate));

	// Redirect to the temporary PHP script.
	header('Location:' . cfg::$PHPTemp_FileName);

	return STATUS_OK;

}


function h_FindComments_Delete() {
	/*
	Purpose
		Handler for deleting comments from the "Find Comments" list.
	HTTP Parameters
		id (POST array) = An array of IDs for each comment we want to delete.
		(Database Parameters) = All standard database parameters must be specified.
	Return (GET/POST integer)
		Returns a status code:
			STATUS_OK
			STATUS_DatabaseParameterMissing
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
	Version History
		1.00 2009.6.23 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// The title for all dialogs.
	$TITLE = 'Delete Checked Comments';

	// Trap: No comments were checked. Display a message and return an OK status code.
	if (! array_key_exists('id', $_POST)) {
		SayPage__Msg('No comments were checked.<br />No comments have been deleted.', $TITLE);
		return STATUS_OK;
	}

	// Get the database parameters.
	$DB = Parm_Get_DB();
	// Error Trap: Display an error message and return a status code.
	if (is_int($DB)) return SayPage__Error_ByStatus($DB, $TITLE);

	// Open the database.
	$hDB = DB_Open_ByArray($DB, true);
	// Error Trap: Display an error message and return a status code.
	if (is_int($hDB)) return SayPage__Error_ByStatus($hDB, $TITLE);

	// Determine the number of comments before the delete.
	$e = Gallery_Comments_GetCount($hDB, $DB['TablePrefix']);
	// Error Trap: Display an error message and return a status code.
	if ($e['Status'] != STATUS_OK) return SayPage__Error_ByStatus($e['Status'], $TITLE);
	// Save the count.
	$CountBefore = $e['CommentCount'];

	// Used to track the total number of delete queries, rows, and comments.
	$CountQueries = 0;
	$RowsDeleted = 0;
	$CountComments = 0;		// This is separate form CountQueries to accomodate future optimization.

	// Iterate all passed IDs.
	foreach ($_POST['id'] as $id) {

		// Delete the current id.
		$e = Gallery_Comments_Delete_ByMethod($hDB, 'ID', $id, $DB['TablePrefix'], $DB['ColumnPrefix']);
		// Error Trap: Display an error message and return a status code.
		if ($e['Status'] != STATUS_OK) return SayPage__Error_ByStatus($e['Status'], $TITLE);

		// Update tracking counts.
		$CountQueries += 1;
		$RowsDeleted += $e['CountDeleted'];
		$CountComments += 1;

	}

	// Determine the number of comments after the delete.
	$e = Gallery_Comments_GetCount($hDB, $DB['TablePrefix']);
	// Error Trap: Display an error message and return a status code.
	if ($e['Status'] != STATUS_OK) return SayPage__Error_ByStatus($e['Status'], $TITLE);
	// Save the count.
	$CountAfter = $e['CommentCount'];

	// Display status page and return status code OK.
	$Msg = Msg_Build('DeleteComments', array(
		'CountQueries'		=> number_format($CountQueries),
		'CommentsDeleted'	=> number_format((int) $CountBefore - (int) $CountAfter),
		'RowsDeleted'		=> number_format($RowsDeleted),
		'CountDeleted'		=> number_format($CountComments),
		'CountBefore'		=> number_format($CountBefore),
		'CountAfter'		=> number_format($CountAfter)
	));
	SayPage__Confirmed($Msg, $TITLE);

	return STATUS_OK;

}


function h_FindDuplicates() {
	/*
	Purpose
		Handler for finding and displaying duplicate comments from the Gallery.
	HTTP Parameters
		column (string) = The name of the column we're searching.
			This is the actual table's column name witout a prefix.
			Valid values: "subject", "comment", "author", "host"
				If any other value is specified, an error message will be displayed.
		value (string) = The value we're searching the specified column for.
			This is used in a SQL LIKE statment, so wildcards are accepted ("%", "_").
		count (integer) = The number of duplicates we must find (in the specified column) in order for the comment
			to qualify to be displayed.
			An error will be displayed if:
				- This is < 1
				- This is > 2,000,000
				- This is not numeric
		max (integer) = The maximum number of results to return.
			An error will be displayed if:
				- This is < 1
				- This is > 2,000,000
				- This is not numeric
		(Database Parameters) = All standard database parameters must be specified.
	Return (integer)
		Returns a status code:
			STATUS_OK
			STATUS_InvalidParameter
			STATUS_ErrorReadingListFile
			STATUS_ErrorWritingListFile
			STATUS_DatabaseParameterMissing
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
	Version History
		1.00 2009.6.22 Tested by Russ Tanner in PHP 5.1.6.
	*/

	// The title for all dialogs.
	$TITLE = 'Find Duplicates';

	// Retrieve HTTP parameter: "column"
	$Column = GetInput('column');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($Column)) {
		SayPage__Error('Parameter Missing: "column"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Retrieve HTTP parameter: "value"
	$Value = GetInput('value');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($Value)) {
		SayPage__Error('Parameter Missing: "value"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Retrieve HTTP parameter: "count"
	$Count = GetInput('count');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($Value)) {
		SayPage__Error('Parameter Missing: "count"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Retrieve HTTP parameter: "max"
	$Max = GetInput('max');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($Max)) {
		SayPage__Error('Parameter Missing: "max"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Error Trap: The specified column name is invalid. Display an error message and return a status code.
	if (! array_key_exists($Column, cfg::$map_ColumnToFriendly)) {
		SayPage__Error('Invalid Parameter: Column', $TITLE);
		return SayPage__Error_ByStatus(STATUS_InvalidParameter, $TITLE);
	}

	// Error Trap: Invalid Parameter: Count, Display an error message and return a status code.
	if ((! is_numeric($Count)) || $Count < 1 || $Count > 2000000) {
		SayPage__Error('Invalid Parameter: Count<br />Count must be a value between 1 and 2,000,000.', $TITLE);
		return SayPage__Error_ByStatus(STATUS_InvalidParameter, $TITLE);
	}

	// Error Trap: Invalid Parameter: Max, Display an error message and return a status code.
	if ((! is_numeric($Max)) || $Max < 1 || $Max > 2000000) {
		SayPage__Error('Invalid Parameter: Max<br />Max must be a value between 1 and 2,000,000.', $TITLE);
		return SayPage__Error_ByStatus(STATUS_InvalidParameter, $TITLE);
	}

	// Get the database parameters.
	$DB = Parm_Get_DB();
	// Error Trap: Display an error message and return a status code.
	if (is_int($DB)) return SayPage__Error_ByStatus($DB, $TITLE);

	// Open the database. We need a database link id in this function.
	$hDB = DB_Open_ByArray($DB);
	// Error Trap: Return a status code.
	if (is_int($hDB)) return SayPage__Error_ByStatus($hDB, $TITLE);

	// Get the SQL template we're using.
	$SQL = cfg::$map_MethodToSQL['FindDuplicates'];

	// Build SQL from the SQL template.
	$SQL = str_replace('__TablePrefix__', mysql_real_escape_string($DB['TablePrefix'], $hDB), $SQL);
	$SQL = str_replace('__ColumnPrefix__', mysql_real_escape_string($DB['ColumnPrefix'], $hDB), $SQL);
	$SQL = str_replace('__Column__', $Column, $SQL);
	$SQL = str_replace('__Value__', mysql_real_escape_string($Value, $hDB), $SQL);
	$SQL = str_replace('__Count__', $Count, $SQL);
	$SQL = str_replace('__Max__', $Max, $SQL);

	// Query the database.
	$q = mysql_query($SQL, $hDB);

	// Error Trap: Error querying database. Return a status code.
	if (! $q) return SayPage__Error_ByStatus(STATUS_ErrorQueryingDatabase, $TITLE);

	// Optimize variables.
	$p = $DB['ColumnPrefix'];		// We're going to use this a lot later on.

	// Load the template used to generate each record's HTML.
	$Template = Template_Get(cfg::$map_ColumnToTemplate[$Column]);

	// General counter provides a value for the SN (serial number) token.
	$SN = 1;		// The first record is 1, not 0 (zero).

	// Empty the buffer file.
	file_put_contents(cfg::$Buffer_FileName, '');

	// Iterate the returned records.
	// We go through a lot of trouble here to avoid PHP memory limits on large result sets.
	// We will build each row's template and write it to a file instead of a variable (as we would normally do).
	while ($row = mysql_fetch_assoc($q)) {

		// Create a token list for the current record.
	$Tokens = array(
			'SN'			=> number_format($SN),
			'Count'			=> number_format($row['cnt']),
			'CountInt'		=> $row['cnt'],
			'PageURL_Main'	=> cfg::$GlobalTokens['PageURL_Main']
		);
		// Conditionally build more tokens depending on the column selected.
		switch($Column) {
		case 'subject':
			$Tokens['Subject'] = strlen(trim($row[$p.'subject'])) > 0 ? htmlspecialchars($row[$p.'subject']) : '(no subject)';
			break;
		case 'comment':
			$Tokens['Body'] = htmlspecialchars($row[$p.'comment']);
			break;
		case 'author':
			$Tokens['Author'] = htmlspecialchars($row[$p.'author']);
			break;
		case 'host':
			$Tokens['IP'] = htmlspecialchars($row[$p.'host']);
			break;
		}
		
		// Build the template for the current record.
		$rowHTML = ExpandTokens($Template, $Tokens);

		// Append the current row's HTML to the buffer file (instead of memory).
		file_put_contents(cfg::$Buffer_FileName, $rowHTML, FILE_APPEND);

		// Increment our serial number.
		$SN += 1;

	}

	// Create search-criteria string for diplay at the top of the results page.
	$SearchCriteria = 'You searched for ' . cfg::$map_ColumnToFriendly[$Column] . ' = "' . htmlspecialchars($Value) . '"';

	// Trap: No matching records. Display message and return OK status code.
	if ($SN == 1) {
		SayPage__Msg($SearchCriteria . "<br />\nNo matching records were found.", $TITLE);
		return STATUS_OK;
	}

	// Tokens for MsgFile template...
	// Create a match-count string.
	$MatchCount = number_format($SN - 1) . ' matches have been found.';
	// This is the "PreBody" token.
	$PreBody = $SearchCriteria . "<br />\n" . $MatchCount;
	// Make a simple "PostBody" token.
	$PostBody = '';
//	Save for possible later update.
//	$PostBody = '<p class="Text">Click here to <a href="__URLContinue__" class="Text_A">continue...</a></p>'."\n";
//	$PostBody = str_replace('__URLContinue__', cfg::$GlobalTokens['PageURL_Main'], $PostBody);

	// Build the hidden tags we need to pass to the result page.
	$HiddenTags = Tags_Build_INPUT_Hidden(array_merge($DB, array(
		'column'		=> $Column
	)));

	// Create tokens.
	$Tokens = array(
		'Title'			=> $TITLE,
		'PreBody'		=> $PreBody,
		'PostBody'		=> $PostBody,
		'FileName'		=> cfg::$Buffer_FileName,
		'PageURL_Main'	=> cfg::$GlobalTokens['PageURL_Main'],
		'FormFields'	=> $HiddenTags
	);
	$MsgFileTemplate = ExpandTokens(Template_Get('MsgFile'), $Tokens);

	// Build the bones and save it to a temporary PHP file.
	file_put_contents(cfg::$PHPTemp_FileName, Template_Build__Bones($MsgFileTemplate));

	// Redirect to the temporary PHP script.
	header('Location:' . cfg::$PHPTemp_FileName);

	return STATUS_OK;

}


function h_FindDuplicates_Delete() {
	/*
	Purpose
		Handler for deleting comments from the "Find Duplicates" list.
	HTTP Parameters
		value (POST array) = An array of values for each comment we want to delete.
			If a row's column (as specified in Column) matches any string, the row (Comment) is deleted.
		column (string) = The name of the column we're searching.
			This is the actual table's column name witout a prefix.
			Valid values: "subject", "comment", "author", "host"
				If any other value is specified, an error message will be displayed.
		(Database Parameters) = All standard database parameters must be specified.
	Return (GET/POST integer)
		Returns a status code:
			STATUS_OK
			STATUS_DatabaseParameterMissing
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
	Version History
		1.00 2009.6.23 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// The title for all dialogs.
	$TITLE = 'Delete Checked Comments';

	// Trap: No comments were checked. Display a message and return an OK status code.
	if (! array_key_exists('value', $_POST)) {
		SayPage__Msg('No comments were checked.<br />No comments have been deleted.', $TITLE);
		return STATUS_OK;
	}

	// Error Trap: The column was not POSTed. Display a message and return a status code.
	if (! array_key_exists('column', $_POST)) {
		SayPage__Error('A required parameter is missing: column', $TITLE);
		return STATUS_OK;
	}
	// Error Trap: The specified column is invalid. Display a message and return a status code.
	if (! array_key_exists($_POST['column'], cfg::$map_ColumnToMethod)) {
		SayPage__Error('A required parameter is missing: column', $TITLE);
		return STATUS_OK;
	}

	// Now we know the specified column is good. Save it (convenience) and map it to a Method.
	$Column = $_POST['column'];
	$Method = cfg::$map_ColumnToMethod[$Column];

	// Get the database parameters.
	$DB = Parm_Get_DB();
	// Error Trap: Display an error message and return a status code.
	if (is_int($DB)) return SayPage__Error_ByStatus($DB, $TITLE);

	// Open the database.
	$hDB = DB_Open_ByArray($DB, true);
	// Error Trap: Display an error message and return a status code.
	if (is_int($hDB)) return SayPage__Error_ByStatus($hDB, $TITLE);

	// Determine the number of comments before the delete.
	$e = Gallery_Comments_GetCount($hDB, $DB['TablePrefix']);
	// Error Trap: Display an error message and return a status code.
	if ($e['Status'] != STATUS_OK) return SayPage__Error_ByStatus($e['Status'], $TITLE);
	// Save the count.
	$CountBefore = $e['CommentCount'];

	// Used to track the total number of delete queries, rows, and comments.
	$CountQueries = 0;
	$RowsDeleted = 0;
	$CountComments = 0;		// This is separate form CountQueries to accomodate future optimization.

	// Iterate all passed values.
	foreach ($_POST['value'] as $value) {

		// Make sure "value" is properly unescaped if necessary.
		$value = get_magic_quotes_gpc() ? stripslashes($value) : $value;

		// Delete the current id.
		$e = Gallery_Comments_Delete_ByMethod($hDB, $Method, $value, $DB['TablePrefix'], $DB['ColumnPrefix']);
		// Error Trap: Display an error message and return a status code.
		if ($e['Status'] != STATUS_OK) return SayPage__Error_ByStatus($e['Status'], $TITLE);

		// Update tracking counts.
		$CountQueries += 1;
		$RowsDeleted += $e['CountDeleted'];
		$CountComments += 1;

	}

	// Determine the number of comments after the delete.
	$e = Gallery_Comments_GetCount($hDB, $DB['TablePrefix']);
	// Error Trap: Display an error message and return a status code.
	if ($e['Status'] != STATUS_OK) return SayPage__Error_ByStatus($e['Status'], $TITLE);
	// Save the count.
	$CountAfter = $e['CommentCount'];

	// Display status page and return status code OK.
	$Msg = Msg_Build('DeleteComments', array(
		'CountQueries'		=> number_format($CountQueries),
		'CommentsDeleted'	=> number_format((int) $CountBefore - (int) $CountAfter),
		'RowsDeleted'		=> number_format($RowsDeleted),
		'CountDeleted'		=> number_format($CountComments),
		'CountBefore'		=> number_format($CountBefore),
		'CountAfter'		=> number_format($CountAfter)
	));
	SayPage__Confirmed($Msg, $TITLE);

	return STATUS_OK;

}


function h_ClearList() {
	/*
	Purpose
		Handler for clearing a list.
	HTTP Parameters
		q (GET/POST string) = This is the name of the list we want to clear.
			This is not a file name. It is a proper list name which gets mapped to a file name.
			Valid values:
				"SubjectList", "BodyList", "AuthorList", "IPList"
	Return (integer)
		Returns a status code.
		Valid values:
			STATUS_OK
			STATUS_ParameterMissing
			STATUS_InvalidListName
			STATUS_ErrorWritingListFile
	Notes
		Before the list is actually cleared, a yes/no confirmation page is displayed to prevent accidents.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Config
	$TITLE = 'Clear List';			// The title used for dialogs.

	// Retrieve HTTP parameter: "q"
	$ListName = GetInput('q');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($ListName)) {
		SayPage__Error('Parameter Missing: "q"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Error Trap: Invalid list name. Display an error message and return a status code.
	if (! ListName_IsValid($ListName)) return SayPage__Error_ByStatus(STATUS_InvalidListName, $TITLE);

	// Clear the list
	$e = List_Clear($ListName);
	// Error Trap: Display an error message and return a status code.
	if ($e != STATUS_OK) return SayPage__Error_ByStatus($e, $TITLE);

	// Success. Display confirmation message and return OK status code.
	SayPage__Confirmed("The \"$ListName\" list has been cleared.", $TITLE);
	return STATUS_OK;

}


function h_RemoveDuplicates() {
	/*
	Purpose
		Handler for removing duplicates from a list.
	HTTP Parameters
		q (GET/POST string) = This is the name of the list we want to remove duplicates from.
			This is not a file name. It is a proper list name which gets mapped to a file name.
			Valid values:
				"SubjectList", "BodyList", "AuthorList", "IPList"
		(Database Parameters) = All standard database parameters must be specified.
	Return (integer)
		Returns a status code.
		Valid values:
			STATUS_OK
			STATUS_InvalidListName
			STATUS_ErrorReadingListFile
			STATUS_ErrorWritingListFile
	Notes
		This displays a message page reporting the status of the request.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Config
	$TITLE = 'Remove Duplicates';			// The title used for all result messages.

	// Retrieve HTTP parameter: "q"
	$ListName = GetInput('q');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($ListName)) {
		SayPage__Error('Parameter Missing: "q"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Error Trap: Invalid list name. Display an error message and return a status code.
	if (! ListName_IsValid($ListName)) return SayPage__Error_ByStatus(STATUS_InvalidListName, $TITLE);

	// Remove duplicates.
	$e = List_RemoveDuplicates($ListName);
	// Error Trap: Display an error message and return.
	if ($e != STATUS_OK) return SayPage__Error_ByStatus($e, $TITLE);

	// Success. Display confirmation message and return OK status code.
	SayPage__Confirmed("The \"$ListName\" list has been purged of duplicates.", $TITLE);
	return STATUS_OK;

}


function h_SaveList() {
	/*
	Purpose
		Handler for saving a list of values to a list file.
	HTTP Parameters
		q (GET/POST string) = This is the name of the list we want to sort.
			This is not a file name. It is a proper list name which gets mapped to a file name.
			Valid values:
				"SubjectList", "BodyList", "AuthorList", "IPList"
	Return (integer)
		Returns a status code.
		Valid values:
			STATUS_OK
			STATUS_ParameterMissing
			STATUS_InvalidListName
			STATUS_ErrorWritingListFile
	Notes
		This displays a message page reporting the status of the request.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Config
	$TITLE = 'Save List';			// The title used for all result messages.

	// Retrieve HTTP parameter: "q"
	$ListName = GetInput('q');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($ListName)) {
		SayPage__Error('Parameter Missing: "q"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Error Trap: Invalid list name. Display an error message and return a status code.
	if (! ListName_IsValid($ListName)) return SayPage__Error_ByStatus(STATUS_InvalidListName, $TITLE);

	// Get the raw list we want to save from current parameters.
	$ListString = GetInput($ListName);
	// Error Trap: Invalid list name (the list was not passed). Display an error message and return a status code.
	if (is_null($ListString)) return SayPage__Error_ByStatus(STATUS_InvalidListName, $TITLE);

	// Convert the list to an array properly.
	$ListArray = HTML_TextArea_ToArray($ListString);

	// Save the list.
	$e = List_Save_FromArray($ListName, $ListArray);
	// Error Trap: Display an error message and return a status code.
	if ($e != STATUS_OK) return SayPage__Error_ByStatus($e, $TITLE);

	// Success. Display a confirmation page and return OK status code.
	SayPage__Confirmed("The \"$ListName\" list has been saved.", $TITLE);
	return STATUS_OK;

}


function h_SayPage() {
	/*
	Purpose
		Used to display the specified page.
	HTTP Parameters
		page (optional string) = The name of the page we want to display.
			This is the proper mapped name of the page, not a file name.
			Default: "Main"
	Return (integer)
		Returns a status code.
			Possible return values:
				STATUS_OK
				STATUS_InvalidParameter
	Version History
		1.00 2009.6.26 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Retrieve HTTP parameter: "page"
	$Page = GetInput('page');
	// Trap: Parameter missing. Default it.
	if (is_null($Page)) $Page = "Main";

	// Vector to the correct page-display function.
	switch($Page) {
	case 'About':
		SayPage__About();
		break;
	case 'License':
		SayPage__License();
		break;
	case 'Main':
		SayPage__Main();
		break;
	case 'SysReq':
		SayPage__SysReq();
		break;
	case 'Usage':
		SayPage__Usage();
		break;
	default:
		SayPage__Error('Invalid Parameter: "page"');
		return STATUS_InvalidParameter;
	}

	return STATUS_OK;
}


function h_SortList() {
	/*
	Purpose
		Handler for sorting a list of values to a list file.
	HTTP Parameters
		q (GET/POST string) = This is the name of the list we want to sort.
			This is not a file name. It is a proper list name which gets mapped to a file name.
			Valid values:
				"SubjectList", "BodyList", "AuthorList", "IPList"
	Return (integer)
		Returns a status code.
		Valid values:
			STATUS_OK
			STATUS_ParameterMissing
			STATUS_InvalidListName
			STATUS_ErrorReadingListFile
			STATUS_ErrorSortingList
			STATUS_ErrorWritingListFile
	Notes
		This displays a message page reporting the status of the request.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Config
	$TITLE = 'Sort List';			// The title used for all result messages.

	// Retrieve HTTP parameter: "q"
	$ListName = GetInput('q');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($ListName)) {
		SayPage__Error('Parameter Missing: "q"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Error Trap: Invalid list name. Display an error message and return a status code.
	if (! ListName_IsValid($ListName)) return SayPage__Error_ByStatus(STATUS_InvalidListName, $TITLE);

	// Sort the list
	$e = List_Sort($ListName);
	// Error Trap: Display an error message and return a status code.
	if ($e != STATUS_OK) return SayPage__Error_ByStatus($e, $TITLE);

	// Success. Display a confirmation message and return OK status code.
	SayPage__Confirmed("The \"$ListName\" list has been sorted.", $TITLE);
	return STATUS_OK;

}


function h_AutoDetect() {
	/*
	Purpose
		Handler for automatically detecting table and column prefixes.
	HTTP Parameters
		q (GET/POST string) = Determines which set of data gets auto-detected.
			Valid values:
				"Prefixes" - Auto-detect table/column prefixes.
		(Database Parameters) = The following database parameters are required:
			DBServer, DBUsername, DBPassword, DBName
	Return (integer)
		Returns a status code:
			STATUS_OK
			STATUS_ParameterMissing
			STATUS_DatabaseParameterMissing
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
			STATUS_ErrorDetectingTablePrefix
			STATUS_ErrorDetectingColumnPrefix
	Version History
		1.00 2009.6.9 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Config
	$TITLE = 'Auto-Detect Table & Column Prefixes';			// The title used for all result messages.

	// Retrieve HTTP parameter: "q"
	$_q = GetInput('q');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($_q)) {
		SayPage__Error('Parameter Missing: "q"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Detect Table prefix.
	$TablePrefix = Gallery_Table_GetPrefix();
	// An error occured. Display an error message and return a status code.
	if (is_int($TablePrefix)) return SayPage__Error_ByStatus($TablePrefix, $TITLE);

	// Detect Column Prefix.
	$ColumnPrefix = Gallery_Column_GetPrefix($TablePrefix);
	// An error occured. Display an error message and return a status code.
	if (is_int($ColumnPrefix)) return SayPage__Error_ByStatus($ColumnPrefix, $TITLE);

	// Display a confirmation page.
	SayPage__Confirm(
		$TITLE,
		"
			The following prefixes have been detected:<br />\n
			<br />\n
			Table Prefix: \"$TablePrefix\".<br />\n
			Column Prefix: \"$ColumnPrefix\".<br />\n
			<br />\n
			Would you like to use these prefixes?
		", 
		'Yes',
		$_SERVER['PHP_SELF'] . '?' . http_build_query(array('page'=>'Main', 'TablePrefix'=>$TablePrefix, 'ColumnPrefix'=>$ColumnPrefix)),
		'No',
		$_SERVER['PHP_SELF'] . '?' . http_build_query(array('page'=>'Main'))
	);

	return STATUS_OK;

}


function h_RestoreDefaults() {
	/*
	Purpose
		Handler for resetting field values to their default values.
	HTTP Parameters
		q (GET/POST string) = Determines which set of data gets reset to default values.
			Valid values:
				"DBCredentials" - Resets all fields in the "Database Credentials" section.
				"Prefixes" - Resets all field in the "Table & Column Prefix" section.
	Return (integer)
		Returns a status code.
		Valid values:
			STATUS_OK
			STATUS_ParameterMissing
			STATUS_InvalidDataSet
	Notes
		This does not display a status page. It simply redisplays the main page.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// The title used for all dialogs.
	$TITLE = 'Restore Defaults';

	// Retrieve HTTP parameter: "q"
	$_q = GetInput('q');
	// Error Trap: Parameter missing. Display an error message and return a status code.
	if (is_null($_q)) {
		SayPage__Error('Parameter Missing: "q"', $TITLE);
		return STATUS_ParameterMissing;
	}

	// Reset correct data set
	switch($_q) {
	case 'DBCredentials':
		Input_Unset_FormFields(array(
			'DBServer',
			'DBUsername',
			'DBPassword',
			'DBName'
		));
		break;
	case 'Prefixes':
		Input_Unset_FormFields(array(
			'TablePrefix',
			'ColumnPrefix'
		));
		break;
	// Invalid data set. Display an error message.
	default:
		SayPage__Error('Invalid Parameter: "q"');
		return STATUS_InvalidDataSet;
	}

	// Display the main page
	SayPage__Main();

	return STATUS_OK;

}



// Functions //


function AA_Split($MyAA) {
	/*
	Purpose
		Used to convert an associative array into two normal arrays.
	Parameters
		MyAA (Associative Array) = The associative array we want to convert.
	Return (Array of Arrays)
		Returns an array with two elements.
			The first element contains an array of Keys from the specified associative array (MyAA).
			The second elements contains an array of Values from the specified associative array (MyAA).
		If the specified associative array (MyAA) is empty, function returns two empty arrays.
	Version History
		1.0 8/12/2004 Tested by Russ Tanner on PHP 4.3.4.
	*/

	// Initiate variables
	$R['Keys'] = array(); // Return buffer for keys
	$R['Values'] = array(); // Return buffer for values

	// Build both arrays
	while (list($Key, $Value) = each ($MyAA)) {
		$R['Keys'][] = $Key;
		$R['Values'][] = $Value;
	}
	
	// Return the result
	return $R;

}


function AA_ExtendKV($Arr, $KeyPre = '', $KeyApp = '', $ValPre = '', $ValApp = '') {
	/*
	Purpose
		Used to prepend and append a value to each key and value in an array.
	Parameters
		Arr (Array) = The array we're working with.
		KeyPre (Optional String) = This string is prepended to each key in the array.
		KeyApp (Optional String) = This string is appended to each key in the array.
		ValPre (Optional String) = This string is prepended to each value in the array.
		ValApp (Optional String) = This string is appended to each value in the array.
	Return (Associative Array)
		Returns the specified array modified according to the specified parameters.
		Returns an empty array if:
			- The specified array (Arr) is empty.
	Version History
		1.0	4/28/2007 11:44pm Tested by Russ Tanner on PHP 5.0.1.
	*/

	// Initiate variables
	$r = array(); // return buffer

	// Iterate the entire array
	foreach($Arr as $k => $v) $r[$KeyPre . $k . $KeyApp] = $ValPre . $v . $ValApp;

	// Return the result
	return $r;

}


function Arr_Delete_Whitespace($Arr) {
	/*
	Purpose
		Used to remove all elements from an array that contain only whitespace.
	Parameters
		Arr (array) = The array we're working with.
	Return (array)
		Returns the specified array (Arr) with all elements that contain only whitespace removed.
	Version History
		1.00 2008.4.20 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Unset elements containing only whitespace
	foreach($Arr as $k => $v) if(trim($v) == '') unset($Arr[$k]);

	// Rebuild the array and return it.
	return array_values($Arr); 

}


function Arr_EscHTML($Arr) {
	/*
	Purpose
		Used to escape an array using either htmlspecialchars() or htmlentities(), as specified by EscLevel.
	Parameters
		Arr (Array) = The array we're going to escape.
			An an AA is passed, the keys will be preserved.
	Return (Array)
		Returns the specified array (Arr) escaped as specified.
		If the specified EscLevel is invalid (<0 or >2), function returns the original array unchanged.
	Version History
		1.00 2008.6.23 by Russ Tanner
	*/

	$r = array(); // return buffer

	// Escape the specified array
	foreach ($Arr as $k=>$v) $r[$k] = htmlspecialchars($v, cfg::$QuoteStyle, cfg::$CharSet);

	return $r;

}


function Arr_Trim_Whitespace($Arr) {
	/*
	Purpose
		Used to remove all whitespace from all elements in an array.
	Parameters
		Arr (array) = The array we're working with.
	Return (array)
		Returns the specified array (Arr) with all elements fully trimmed of whitespace.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	$r = array();		// Return buffer.

	// Iterate all elements appending trimmed elements to the return buffer.
	foreach($Arr as $v) $r[] = trim($v);

	return $r;

}


function BatchControl_Load() {
	/*
	Purpose
		Used to load and parse a batch control file into an associative array.
	Return (associative array)
		Returns an associative array with the following elements:
			"Vars" (array of strings) = Returns an array of variables from the batch control file.
			"Status (integer) = Returns a status code.
				Possible Values:
					STATUS_OK
					STATUS_ErrorReadingBatchControlFile
	Version History
		1.00 2009.06.15
	*/

	// Return buffer.
	$r = array(
		'Vars'		=> '',
		'Status'	=> 0
	);

	// No batch control file exists. Initiate one.

	// Load the file to an array.
	$ControlFile = File_AsArray(cfg::$Batch_ControlFile);
	// Error Trap: Error reading batch control file. Return a status code.
	if ($ControlFile['Status'] != STATUS_OK) {
		$r['Status'] = STATUS_ErrorReadingBatchControlFile;
		return $r;
	}

}


function BatchFile_Parse($FileSpec = null) {
	/*
	Purpose
		Used to parse the specified batch file and return the current parameters.
	Parameters
		FileSpec (optional string) = The filespec of the batch file we're parsing.
			Default if null: cfg::Batch_FileName
			Default: null
	Return (associative array)
		Returns an associative array containing the following elements:
			"Vars" (array of strings) = This contains all variables that were defined on the current line of the batch file.
			"Status" (integer) = Contains a status code.
				Possible Values:
					STATUS_OK
					STATUS_ErrorReadingBatchFile
					STATUS_ErrorWritingBatchFile
	Notes
		Following is the structure of each line in a batch file:
			<key> = <value>
		Lines that begin with 
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Return buffer
	$r = array(
		'_Status' 		=> 0
	);

	// Default parameters
	if (is_null($BatchFileName)) $BatchFileName = cfg::$Batch_FileName;

	// A batch-control file exists. Load it.

	// Load the batch file
	$File = file_get_contents($BatchFileName);
	// Error Trap: Error reading batch file. Returns a status code.
	if ($File === false) {
		$r['_Status'] = STATUS_ErrorReadingBatchFile;
		return $r;
	}

	// Convert the list to a clean array.
	$r = explode($Delimiter, $List);			// Return buffer.

	// Remove any elements containing only whitespace.
	$r = Arr_Delete_Whitespace($r);

	// Trim all whitespace from all elements. This also removes the trailing 13 (if it exists).
	$r = Arr_Trim_Whitespace($r);

	return $r;

}


function DB_Open($Server, $Username, $Password, $Name, $NewConnection = false) {
	/*
	Purpose
		Used to open the database and get a resource ID.
	Parameters
		Server (string) = The server name of the database we're opening.
		Username (string) = The username of the database we're opening.
		Password (string) = The password of the databse we're opening.
		Name (string) = The name of the database we're opening.
		NewConnection (optional boolean) = Used to determine if we want to ensure a new connection is made.
			This is sent as the fourth parameter of mysql_connect().
			Default: false
	Returns (database link id, integer)
		Returns a database link id if the database is successfully opened.
		Returns an (integer) status code if an error occurs.
			Possible integer return values:
				STATUS_ErrorConnectingToDatabase
				STATUS_ErrorSelectingDatabase
	Exit Conditions
		If an error occurs, the database is not open on exit.
	Version History
		1.00 2009.6.11 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Connect to the database.
	$hDB = @mysql_connect($Server, $Username, $Password, $NewConnection);
	// Error Trap: Error connecting to database. Return status code.
	if (! $hDB) return STATUS_ErrorConnectingToDatabase;

	// Select the database.
	// Error Trap: Error selecting database. Return a status code.
	if (! @mysql_select_db($Name, $hDB)) {
		mysql_close($hDB);
		return STATUS_ErrorSelectingDatabase;
	}

	// We successfully opened the database. Return the database link id.
	return $hDB;

}


function DB_Open_ByArray($Credentials, $NewConnection = false) {
	/*
	Purpose
		Used to open the database and get a resource ID.
	Parameters
		Credentials (associative array) = This contains all necessary database credentials.
			Element names: DBServer, DBUsername, DBPassword, DBName
		NewConnection (optional boolean) = Used to determine if we want to ensure a new connection is made.
			This is sent as the fourth parameter of mysql_connect().
			Default: false
	Returns (database link id, integer)
		Returns a database link id if the database is successfully opened.
		Returns an (integer) status code if an error occurs.
			Possible integer return values:
				STATUS_ErrorConnectingToDatabase
				STATUS_ErrorSelectingDatabase
	Exit Conditions
		If an error occurs, the database is not open on exit.
	Version History
		1.00 2009.6.11 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Let DB_Open() do our work.
	return DB_Open($Credentials['DBServer'], $Credentials['DBUsername'], $Credentials['DBPassword'], $Credentials['DBName'], $NewConnection);

}


function DB_Open_ByParms($NewConnection = false) {
	/*
	Purpose
		Used to open the database and get a resource ID.
	Parameters
		NewConnection (optional boolean) = Used to determine if we want to ensure a new connection is made.
			This is sent as the fourth parameter of mysql_connect().
			Default: false
	Returns (database resource id, integer)
		Returns a database link id if the database is successfully opened.
		Returns an (integer) status code if an error occurs.
			Possible integer return values:
				STATUS_DatabaseParameterMissing
				STATUS_ErrorConnectingToDatabase
				STATUS_ErrorSelectingDatabase
	Exit Conditions
		If an error occurs, the database is not open on exit.
	Version History
		1.00 2009.6.11 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Get all necessary database parameters.
	$DB = Parm_Get_DB(true);
	// Error Trap: Database parameter missing. Return a status code.
	if (is_int($DB)) return STATUS_DatabaseParameterMissing;

	// Connect to (and select) the database. Return a (integer) status code if an error occurs.
	return DB_Open($DB['DBServer'], $DB['DBUsername'], $DB['DBPassword'], $DB['DBName'], $NewConnection);

}


function Err_GetFriendlyMap() {
	/*
	Purpose
		Used to get the cfg::map_StatusToMsg map with hints from cfg::map_StatusToHint appeneded to each message.
	Return (associative array)
		Returns cfg::map_StatusToMsg with hints from cfg::map_StatusToHint appended.
	Version History
		1.00 2009.6.9 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Config
	$Glue = "<br />\n";		// The HTML code that is inserted between the error message and the hint.

	// Load the primary message map into the return buffer.
	$r = cfg::$map_StatusToMsg;

	// Iterate all map entries.
	foreach($r as $k => $v) {

		// A hint exists for this map. Append it to the return buffer.
		if (strlen(cfg::$map_StatusToHint[$k]) > 0) $r[$k] = $r[$k] . $Glue . cfg::$map_StatusToHint[$k];

	}

	return $r;

}


function ExpandTokens($MyString, $Fields, $TokenBegin = '__', $TokenEnd = '__') {
	/*
	Purpose
		Parses the specified String (MyString) and replaces all fields in it with the values specified
			by the passed associative array.
	Parameters
		MyString (String) = This is the string we are processing.
		Fields (Associative Array) = This contains a list of field values (Strings) that will be used to
			replace fields in the specified string (MyString). Each Key is a token which will be replaced
			with the corresponding value.
		TokenBegin (Optional String) = A string that is prepended to each token (Key in the associative array)
			before it is replaced in the specified text (MyString).
			For example, if Key = "Title", TokenBegin = "&&", and TokenEnd = "**", then the string "$$Title**" is
			considered the actual token to be replaced.
			Default: '__'
		TokenEnd (Optional String) = A string that is appended to each token (Key in the associative array)
			before it is replaced in the specified text (MyString).
			Default: '__'
	Return (String)
		Returns the specified string (MyString) with all fields replaced with the specified field list (Fields).
			For example if MyString is:
				"You may send Quota_QMsgCount messages within Quota_QMsgHours hours."
			And if Fields contains this array:
				Quota_QMsgCount => 100, Quota_QMsgHours => 24
			Than the resulting message will read:
				"You may send 100 messages within 24 hours."
	Version History
		1.0 8/12/2004 Tested by Russ Tanner on PHP 4.3.4.
		2.0	4/28/2007 Verified by Russ Tanner on PHP 5.0.1
			Added parameters: TokenBegin, TokenEnd
			Added additional code to create an intermediate array with the beginning and ending tokens attached
				before allowing str_replace() to process the arrays.
	*/
	// Modify the array by prepending/appending the token delimiters to each key
	$x = AA_ExtendKV($Fields, $TokenBegin, $TokenEnd);

	// Split the Fields array into 2 separate arrays for use in the str_replace() function
	$y = AA_Split($x);
	
	// Now, replace all tokens in MyString and return the result
	return str_replace($y['Keys'], $y['Values'], $MyString);

}


function Field_GetDisplay_List($ListName) {
	/*
	Purpose
		Used to get a list of values associated with the specified list.
		The value returned is intended to be displayed in the associated ListBox.
	Parameters
		ListName (string) = The list name representing the list we want to return.
			This is not a file name. It is a proper list name which gets mapped to a file name.
	Return (string)
		Returns the values associated with the specified list as a string.
		This string is loaded from the list's associated file.
		If an error occurs reading the file, the list is loaded from the POST array.
		If the value does not exist in the POST array, the value is loaded from the GET array.
		If the value does not exist in the GET array, the value is loaded from the session.
		If the value does not exist in the SESSION, the default value is returned instead.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Attempt to load the list from its associated file.
	$x = List_Load($ListName);
	
	// List loaded successfully, return it.
	if (is_string($x)) return $x;

	// List did not load successfully. Try getting it from parameter-input and from the session.
	$x = Input_Get_Ordered($ListName);

	// List loaded successfully, return it.
	if (! is_null($x)) return $x;

	// List did not load successfully. Just return the default value.
	return cfg::$DefaultFieldValues[$ListName];

}


function Fields_LoadPersistent() {
	/*
	Purpose
		Used to load all persistent fields from the session.
	Return (associative array)
		Returns a list containing all persistent fields.
		The key is the name of the field as contained in cfg::FieldsToPersist.
		The value is the value to be displayed.
	Notes
		Only fields listed in cfg::FieldsToPersist are loaded.
		If a persistent field is not found in the session, then it's default value is loaded.
		If a default value does not exist for the field, then it default to a zero-length string.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	$r = array(); 	// Return buffer

	// Iterate the list of fields to persist to the session.
	foreach (cfg::$FieldsToPersist as $FieldName) {

		$x = GetInput($FieldName);

		// A GET or POST variable has been found.
		// We do this first to allow users to use query strings to prefill form fields.
		if (! is_null($x)) {
			$r[$FieldName] = $x;
		}
		// A session variable is found for the current field. Save it to the output array.
		elseif (array_key_exists(cfg::$SessionPrefix_FormFields . $FieldName, $_SESSION)) {
			$r[$FieldName] = $_SESSION[cfg::$SessionPrefix_FormFields . $FieldName];
		}
		// No session variable was found for the current field.
		// There is a default value for the current field. Save it to the output array.
		elseif (array_key_exists($FieldName, cfg::$DefaultFieldValues)) {
			$r[$FieldName] = cfg::$DefaultFieldValues[$FieldName];
		}
		// No default value was found for the current field.
		// Save a zero-length string to the output array.
		else {
			$r[$FieldName] = '';
		}

	}

	return $r;

}


function Fields_SavePersistent() {
	/*
	Purpose
		Used to save all persistent fields to the session.
	Notes
		Only fields listed in cfg::FieldsToPersist are saved.
		If a field is both GETed and POSTed, the GETed variable is used.
		Fields are only saved if they contain non-whitespace.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Iterate the list of fields to persist to the session.
	foreach (cfg::$FieldsToPersist as $FieldName) {

		// A GET parameter was passed. Save it to the session.
		if (array_key_exists($FieldName, $_GET)) {
			$_SESSION[cfg::$SessionPrefix_FormFields . $FieldName] = trim(GetGET($FieldName));
		}
		// A POST parameter was passed. Save it to the session.
		elseif (array_key_exists($FieldName, $_POST)) {
			$_SESSION[cfg::$SessionPrefix_FormFields . $FieldName] = trim(GetPOST($FieldName));
		}

	}

}


function File_FromArray($FileSpec, $Arr, $EOL = null) {
	/*
	Purpose
		Used to convert an array into a file.
		This function is cross platform compatible.
	Parameters
		FileSpec (string) = The filespec of the file we're saving into.
		Arr (array) = The array we're saving to a file.
		EOL (string, null) = The string we're using for end-of-line sequences.
			Default if null: PHP_EOL (the correct end-of-line for the platform we're running on)
			Default: null
	Return (integer)
		Returns a standard status code.
			Possible return values:	
				STATUS_OK
				STATUS_ErrorWritingFile
	Version History
		1.00 2009.06.15
	*/

	// Default parameters.
	if (is_null($EOL)) $EOL = PHP_EOL;

	// Convert the array to a string.
	$File = implode(PHP_EOL, $Arr);

	// Save the specified file.
	$e = file_put_contents($FileSpec, $File);
	// Error Trap: Error writing file. Return a status code.
	if ($e === false) return STATUS_ErrorWritingFile;

	// Return OK status code.
	return STATUS_OK;

}


function File_PopLine($FileSpec) {
	/*
	Purpose
		Used to pop (remove and return) the top line from a file.
	Parameters
		FileSpec (string) = The filespec of the file we're working with.
	Return (associative array)
		Returns an associative array with the following elements:
			"Line" (string) = Returns the top line of the specified file.
				This returns null if:
					- If the specified file is empty, or
					- A status condition occurs.
			"Status" (integer) = Return a status code.
				Possible return values:
					STATUS_OK
					STATUS_ErrorReadingFile
					STATUS_ErrorWritingFile
	Exit Conditions
		The top line of the specified file is removed from the file.
	Version History
		1.00 2009.06.15
	*/

	// Return buffer.
	$r = array(
		'Line'		=> null,
		'Status'	=> STATUS_OK
	);

	// Open the specified file.
	$x = File_AsArray($FileSpec);
	// Error Trap: Error reading file. Return status code.
	if ($x['Status'] != STATUS_OK) {
		$r['Status'] = STATUS_ErrorReadingFile;
		return $r;
	}

	// Get the value off the top line and pop (shift) the array.
	// This returns null if the array is empty.
	$r['Line'] = array_shift($x['Lines']);

	// Save the specified file.
	$x = File_FromArray($FileSpec, $x['Lines']);
	// Error Trap: Error writing file. Return status code.
	if ($x != STATUS_OK) {
		$r['Line'] = null;		// Need to return null if a status condition occurs.
		$r['Status'] = STATUS_ErrorWritingFile;
		return $r;
	}

	return $r;

}


function File_AsArray($FileSpec, $DeleteWS = false, $TrimWS = true) {
	/*
	Purpose
		Used to open a file and return it as an array.
		This function is cross platform compatible.
	Parameters
		FileSpec (string) = The filespec of the file we're working with.
		DeleteWS (optional boolean) = If true, elements in the returned array containing only whitespace are removed.
			Default: false
		TrimWS (optional boolean) = If true, all elements in the returned array are trimmed of all 
			leading and following whitespace.
			Note: If the file being processed contains EOLs that are not matched with the current platform	
				PHP is running on, then it's possible that, if TrimWS is false, certain EOL characters may remain
				in elements of the returned array. For this reason, it's generally best that this value is true
				as it gets rid of any extra EOLs that may linger. The only time to set this false would be if
				leading or trailing whitespace is important.
			Default: true
	Return (array, integer)
		Returns an array containing each line of the specified file (FileSpec).
		If an error occurs, return a status code (integer).
			Possible return values:	
				STATUS_ErrorReadingFile
	Version History
		1.00 2009.06.14 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Load the specified file.
	$File = file_get_contents($FileSpec);
	// Error Trap: Error reading file. Return a status code.
	if ($File === false) return STATUS_ErrorReadingFile;
	
	// Convert the list to an array.
	$r = explode(PHP_EOL, $File);

	// Remove any elements containing only whitespace.
	if ($DeleteWS) $r = Arr_Delete_Whitespace($r);

	// Trim all whitespace from all elements. This also removes any EOL characters if they remain.
	if ($TrimWS) $r = Arr_Trim_Whitespace($r);

	return $r;

}


function Gallery_Column_GetPrefix($TablePrefix) {
	/*
	Purpose
		Used to get the column prefix in the Gallery table.
	Parameters
		TablePrefix (string) = The prefix used for all tables in the database.
	Returns (string, integer)
		Returns the column prefix used in the Gallery database.
		Returns a zero-length string if no column prefix is used.
		If an error occurs, function returns a status code (integer):
			STATUS_OK
			STATUS_DatabaseParameterMissing
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
			STATUS_ErrorDetectingColumnPrefix
	Notes
		The necessary database credentials are table/column prefix are fetched from GETed or POSTed parameters (in that order).
		If these values are not GETed or POSTed, an error is returned.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Open the database.
	$hDB = DB_Open_ByParms();
	// Error Trap: Error opening database. Return a status code.
	if (is_int($hDB)) return $hDB;

	// Query for the "comment" column in the Comment table.
	$q = mysql_query("SHOW COLUMNS FROM " . $TablePrefix . "Comment  LIKE '%comment'", $hDB);
	// Error Trap: Error querying database. Return a status code.
	if (! $q) return STATUS_ErrorQueryingDatabase;

	// Get the total number of rows returned.
	$x = mysql_num_rows($q);
	// Error Trap: No columns like "%comment" found. Return a status code.
	if ($x === false) return STATUS_ErrorDetectingColumnPrefix;
	// Error Trap: No matching column found or too many matches found. Return a status code.
	if ($x == 0 || $x > 1) return STATUS_ErrorDetectingColumnPrefix;

	// Get the name of the comment column.
	$Row = mysql_fetch_row($q);
	$ColumnName = $Row[0];

	// Parse out the prefix from the column name.
	$Prefix = substr($ColumnName, 0, strlen($ColumnName) - strlen('comment'));

	return $Prefix;

}


function Gallery_Comments_Delete($DB, $ListName) {
	/*
	Purpose
		Used to delete comments from the Gallery database based on the specified list (ListName).
	Parameters
		DB (associative array) = This contains all database credentials, plus "TablePrefix" and "ColumnPrefix"
		ListName (string) = The name of the list we are using.
			The list name implies the method that will be used to delete comments.
			Example:
				If ListName is "SBList", then comments will be deleted based on the subject and body of the comment.
				If ListName is "AuthorList", then comments will be deleted based on its author.
				If ListName is "IPList", the comments will be deleted based on the IP address of the author.
	Returns (associative array, integer)
		Returns an associative array containing the following elements:
			"CountBefore" - The total number of comments that existed before the delete.
			"CountAfter" - The total number of comments that existed after the delete.
			"CountDeleted" - The total number of records that were deleted.
			"QueryCount" - The total number of queries we ran against the database.
		If an error occurs, returns a status code (integer):
			STATUS_OK
			STATUS_InvalidListName
			STATUS_ErrorReadingListFile
			STATUS_DatabaseParameterMissing
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
	Notes
		The necessary database credentials and table/column prefix are fetched from GETed or POSTed parameters (in that order).
		If these values are not GETed or POSTed, an error is returned.
	Version History
		1.00 2009.6.21 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Return buffer
	$r = array(
		'CountBefore'		=> 0,
		'CountAfter'		=> 0,
		'QueryCount'		=> 0,
		'CountDeleted'		=> 0
	);

	// Load the list file.
	$ListArray = List_Load_AsArray($ListName);
	// Error Trap: An error occured. Return a status code.
	if (is_int($ListArray)) return $ListArray;

	// Determine which method name we're using based on the current list name.
	$MethodName = cfg::$map_ListToMethod[$ListName];

	// Determine the number of comments before the delete.
	$e = Gallery_Comments_GetCount($DB);
	// Error Trap: Error getting count. Return a status code.
	if ($e['Status'] != STATUS_OK) return $e['Status'];

	// We have a valid count, remember it.
	$r['CountBefore'] = $e['CommentCount'];

	// Iterate the list-array.
	foreach($ListArray as $v) {

		// The current line contains a value.
		if (List_Line_GetType($v) == LINETYPE_Value) {

			// Execute the query.
			$e = Gallery_Comments_Delete_ByMethod($DB, $MethodName, $v);
			// Error Trap: Return a status code.
			if ($e['Status'] != STATUS_OK) die("aaaaaaaaaaaaaaaaaa"); //return $e['Status'];

			// Increment the query count.
			$r['QueryCount'] += 1;
	
			// Maintain the total number of records affected.
			$r['CountDeleted'] += $e['CountDeleted'];

		}

	}

	// Determine the number of comments after the delete.
	$e = Gallery_Comments_GetCount($DB);
	// Error Trap: Error getting count. Return a status code.
	if ($e['Status'] != STATUS_OK) return $e['Status'];
	
	// We have a valid count, remember it.
	$r['CountAfter'] = $e['CommentCount'];

	return $r;

}


function Gallery_Comments_DeleteAll($DB) {
	/*
	Purpose
		Used to delete all comments from the Gallery database.
	Parameters
		DB (associative array) = This contains all database credentials, plus "TablePrefix" and "ColumnPrefix"
	Returns (associative array, integer)
		Returns an associative array containing the following elements:
			"CountBefore" - The total number of comments that existed before the delete.
			"CountAfter" - The total number of comments that existed after the delete.
			"CountDeleted" - The total number of records that were deleted.
			"QueryCount" - The total number of queries we ran against the database.
		If an error occurs, function returns a status code (integer):
			STATUS_OK
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
	Notes
		The necessary database credentials and table/column prefix are fetched from GETed or POSTed parameters (in that order).
			If these values are not GETed or POSTed, an error is returned.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Return buffer
	$r = array(
		'CountBefore'		=> 0,
		'CountAfter'		=> 0,
		'QueryCount'		=> 1,		// This always returns 1 since we're only performing 1 query.
		'CountDeleted'		=> 0
	);

	// Open the database. We need a database link id in this function.
	$hDB = DB_Open_ByArray($DB);
	// Error Trap: Return a status code.
	if (is_int($hDB)) return $hDB;

	// Determine the number of comments before the delete.
	$e = Gallery_Comments_GetCount($hDB, $DB['TablePrefix']);
	// Error Trap: Error getting count. Return a status code.
	if ($e['Status'] != STATUS_OK) return $e['Status'];

	// We have a valid count, remember it.
	$r['CountBefore'] = $e['CommentCount'];

	// Get the SQL template we're using based on the current MethodName.
	$SQL = cfg::$map_MethodToSQL['All'];

	// Build SQL from the SQl template.
	$SQL = str_replace('__TablePrefix__', mysql_real_escape_string($DB['TablePrefix'], $hDB), $SQL);
	$SQL = str_replace('__ColumnPrefix__', mysql_real_escape_string($DB['ColumnPrefix'], $hDB), $SQL);

	// Delete all comments.
	$q = mysql_query($SQL, $hDB);
	// Error Trap: Error querying database. Return a status code.
	if (! $q) return STATUS_ErrorQueryingDatabase;

	// Get the total number of records deleted.
	$r['CountDeleted'] = mysql_affected_rows($hDB);

	// Determine the number of comments after the delete.
	$e = Gallery_Comments_GetCount($hDB, $DB['TablePrefix']);
	// Error Trap: Error getting count. Return a status code.
	if ($e['Status'] != STATUS_OK) return $e['Status'];
	
	// We have a valid count, remember it.
	$r['CountAfter'] = $e['CommentCount'];

	return $r;

}


function Gallery_Comments_Delete_ByMethod($hDB, $Method, $String, $TablePrefix = null, $ColumnPrefix = null) {
	/*
	Purpose
		Used to delete comments from the Gallery database based on the specified method (Method) and string (String).
	Parameters
		hDB (database link identifier, associative array) = 
			If this is a database link identifier, it is used to correspond with the database.
				The database is not closed when this function exits.
			If this is a list of database credentials:
				Following is a list of expected elements in the associative array:
					"DBServer", "DBUsername", "DBPassword", "DBName"
				The following elements may be specified in this array. If they are, they do not need to be passed to
					their corresponding parameters: "TablePrefix", "ColumnPrefix"
					If they are passed to both, the values passed to the explicit parameters override the ones passed 
					in the hDB array.
		Method (string) = The method we will use to delete comments.
			Valid Values:
				"Subject" "Body" "Author" "IP" "ID"
		String (string) = We will delete all records having this value in the column represented by the specified method.
		TablePrefix (optional string) = The prefix used for all tables.
			Default: null
		ColumnPrefix (optional string) = The prefix used for all columns.
			Default: null
	Returns (associative array)
		Returns an associative array containing the following elements:
			"CountDeleted" - The total number of records that were deleted.
			"Status" - A status code.
				The following status codes may be returned:
					STATUS_OK
					STATUS_ErrorConnectingToDatabase
					STATUS_ErrorSelectingDatabase
					STATUS_ErrorQueryingDatabase
	Entry Conditions
		The specified method (Method) must be a valid method name (as defined in map_MethodToSQL) or this function will crash.
	Version History
		1.00 2009.06.13 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Return buffer
	$r = array(
		'CountDeleted'		=> 0,
		'Status'			=> STATUS_OK
	);

	// Database credentials were specified. Open the database.
	if (is_array($hDB)) {

		// If TablePrefix is not specified, then it must be passed in the hDB array.
		if (is_null($TablePrefix)) $TablePrefix = $hDB['TablePrefix'];
		// If ColumnPrefix is not specified, then it must be passed in the hDB array.
		if (is_null($ColumnPrefix)) $ColumnPrefix = $hDB['ColumnPrefix'];

		// Open the database.
		$hDB = DB_Open_ByArray($hDB, true);
		// Error Trap Return a status code.
		if (is_int($hDB)) {
			$r['Status'] = $hDB;
			return $r;
		}

	}

	// Get the SQL template we're using based on the current MethodName.
	$SQL = cfg::$map_MethodToSQL[$Method];

	// Build SQL from the SQL template for the current list-item.
	$SQL = str_replace('__TablePrefix__', mysql_real_escape_string($TablePrefix, $hDB), $SQL);
	$SQL = str_replace('__ColumnPrefix__', mysql_real_escape_string($ColumnPrefix, $hDB), $SQL);
	$SQL = str_replace('__?__', mysql_real_escape_string($String, $hDB), $SQL);

	// Delete comments.
	$q = mysql_query($SQL, $hDB);
	// Error Trap: Error querying database. Return a status code.
	if (! $q) {
		$r['Status'] = STATUS_ErrorQueryingDatabase;
		return $r;
	}

	// Get the total number of records affected.
	$r['CountDeleted'] = mysql_affected_rows($hDB);

	// Return OK status.
	$r['Status'] = STATUS_OK;

	return $r;

}


function Gallery_Comments_Find($DB, $Field, $Value) {
	/*
	Purpose
		Used to find comments from the Gallery database based on the specified field name and value.
	Parameters
		DB (associative array) = This contains all database credentials, plus "TablePrefix" and "ColumnPrefix"
		Field (string) = The field we're searching. This should be specified without a column prefix.
			Capitalization must be correct.
		Value (string) = The value we're searching for.
	Returns (array of associative array, integer)
		Returns an array of associative array containing a database of records that match the search criteria.
		If an error occurs, returns a status code (integer).
			Possible return values:
				STATUS_DatabaseParameterMissing
				STATUS_ErrorConnectingToDatabase
				STATUS_ErrorSelectingDatabase
				STATUS_ErrorQueryingDatabase
	Notes
		The necessary database credentials and table/column prefix are fetched from GETed or POSTed parameters (in that order).
			If these values are not GETed or POSTed, an error is returned.
	Version History
		1.00 2009.6.21 Tested by Russ Tanner on PHP 5.1.6.
	*/

	$r = array();		// Return buffer
	
	// Open the database. We need a database link id in this function.
	$hDB = DB_Open_ByArray($DB);
	// Error Trap: Return a status code.
	if (is_int($hDB)) return $hDB;

	// Get the SQL template we're using.
	$SQL = cfg::$map_MethodToSQL['Find'];

	// Build SQL from the SQL template.
	$SQL = str_replace('__TablePrefix__', mysql_real_escape_string($DB['TablePrefix'], $hDB), $SQL);
	$SQL = str_replace('__ColumnPrefix__', mysql_real_escape_string($DB['ColumnPrefix'], $hDB), $SQL);
	$SQL = str_replace('__Field__', mysql_real_escape_string($Field, $hDB), $SQL);
	$SQL = str_replace('__?__', mysql_real_escape_string($Value, $hDB), $SQL);

	// Query the database.
	$q = mysql_query($SQL, $hDB);
	// Error Trap: Error querying database. Return a status code.
	if (! $q) return STATUS_ErrorQueryingDatabase;

	// Iterate the returned records, building an in-memory database (array of associative arrays).
	//while ($row = mysql_fetch_assoc($q)) $r[] = $row;
	//while ($row = mysql_fetch_assoc($q)) file_put_contents(cfg::$Buffer_FileName, );

	return $r;

}


function Gallery_Comments_GetCount($hDB, $TablePrefix = null) {
	/*
	Purpose
		Used to get the total number of comments that current exist in the Gallery database.
	Parameters
		hDB (database link identifier, associative array) = 
			If this is a database link identifier, it is used to correspond with the database.
				The database is not closed when this function exits.
			If this is a list of database credentials:
				Following is a list of expected elements in the associative array:
					"DBServer", "DBUsername", "DBPassword", "DBName"
				The following element may be specified. If it is, TablePrefix is not required:
					"TablePrefix"
		TablePrefix (optional string) = The prefix used for all tables.
			If hDB is an array that includes the element TablePrefix, then this parameter need not be passed.
	Return (associative array)
		Returns an associative array with the following elements:
			"CommentCount" (integer) - The total number of comments in the specified database.
				If "Status" is anything other than STATUS_OK, this element returns -1.
			"Status" (integer) - The function's returned status code. Possible values:
				STATUS_OK
				STATUS_ErrorConnectingToDatabase
				STATUS_ErrorSelectingDatabase
				STATUS_ErrorQueryingDatabase
	Entry Conditions
		The database associated with the link identifier (hDB) must be connected and selected or this function will crash.
	Version History
		1.00 2009.6.11 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Initialize the return buffer.
	$r = array(
		'CommentCount'		=> -1,
		'Status'			=> STATUS_OK
	);

	// Database credentials were specified. Open the database.
	if (is_array($hDB)) {

		// If TablePrefix is not specified, then it must be passed in the hDB array.
		if (is_null($TablePrefix)) $TablePrefix = $hDB['TablePrefix'];

		// Open the database.
		$hDB = DB_Open_ByArray($hDB, true);
		// Error Trap: Return a status code.
		if (is_int($hDB)) {
			$r['Status'] = $hDB;
			return $r;
		}

	}

	// Get the SQL template for counting comments.
	$SQL = cfg::$map_MethodToSQL['CommentCount'];

	// Build SQL from the SQL template.
	$SQL = str_replace('__TablePrefix__', mysql_real_escape_string($TablePrefix, $hDB), $SQL);

	// Execute query.
	$q = mysql_query($SQL, $hDB);
	// Error Trap: Error querying database. Return a status code.
	if (! $q) {
		$r['Status'] = STATUS_ErrorQueryingDatabase;
		return $r;
	}

	// Get the count.
	$r['CommentCount'] = mysql_result($q, 0);
	// Error Trap: Error querying database. Return a status code.
	if ($r['CommentCount'] === false) {
		$r['Status'] = STATUS_ErrorQueryingDatabase;
		return $r;
	}

	return $r;

}


function Gallery_Table_GetPrefix() {
	/*
	Purpose
		Used to get the table prefix in the Gallery table.
	Returns (string, integer)
		Returns the table prefix used in the Gallery database.
		Returns a zero-length string if no table prefix is used.
		If an error occurs, function returns a status code (integer):
			STATUS_OK
			STATUS_DatabaseParameterMissing
			STATUS_ErrorConnectingToDatabase
			STATUS_ErrorSelectingDatabase
			STATUS_ErrorQueryingDatabase
			STATUS_ErrorDetectingTablePrefix
	Notes
		The necessary database credentials are table/column prefix are fetched from GETed or POSTed parameters (in that order).
		If these values are not GETed or POSTed, an error is returned.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Open the database.
	$hDB = DB_Open_ByParms();
	// Error Trap: Error opening database. Return a status code.
	if (is_int($hDB)) return $hDB;

	// Query for the comment table.
	$q = mysql_query("SHOW TABLES LIKE '%Comment'", $hDB);
	// Error Trap: Error querying database. Return a status code.
	if (! $q) return STATUS_ErrorQueryingDatabase;

	// Get the total number of rows returned.
	$x = mysql_num_rows($q);
	// Error Trap: No table like "Comment" found. Return a status code.
	if ($x === false) return STATUS_ErrorDetectingTablePrefix;
	// Error Trap: No matching table found, or too many matches found. Return a status code.
	if ($x == 0 || $x > 1) return STATUS_ErrorDetectingTablePrefix;

	// Get the name of the comment table.
	$Row = mysql_fetch_row($q);
	$TableName = $Row[0];

	// Parse out the prefix from the table name.
	$Prefix = substr($TableName, 0, strlen($TableName) - strlen('Comment'));

	return $Prefix;

}


function GetGET($VarName) {
	/*
	Purpose
		Safely retrieves GETed data.
	Parameters
		VarName (string) The name of the GET variable we want to retrieve.
	Return (string, null)
		Returns the GETed element's value properly stripped, if necessary.
		Returns NULL if:
			- The specified variable (VarName) was not specified.
			- The specified variable (VarName) is not a string (it could be an array).
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Trap: Variable not specified, return NULL.
	if (! array_key_exists($VarName, $_GET)) return null;

	// Trap: Variable is not a string, return NULL.
	if (! is_string($_GET[$VarName])) return null;

	// Return properly escaped value.
	return get_magic_quotes_gpc() ? stripslashes($_GET[$VarName]) : $_GET[$VarName];

}


function GET_Unset_FormFields($List) {
	/*
	Purpose
		Used to unset specified form field elements in the _GET array.
	Parameters
		List (array) = This contains a list of elements we want to unset in _GET.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	foreach($List as $k) unset($_GET[$k]);

}


function GetInput($VarName) {
	/*
	Purpose
		Used to retrieve GETed or POSTed data.
	Parameters	
		VarName (string) = The name of the GET or POST variable we want to retrieve.
	Return (string, null)
		Returns the GETed or POSTed element's value properly stripped, if necessary.
		Returns NULL if the specified variable (VarName) was not POSTed or GETed.
		If a variable is both POSTed and GETed, the GETed variable is returned.
	Version History
		1.00 2009.6.11 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Get the GETed variable.
	$r = GetGET($VarName);

	// Variable was GETed, return it now.
	if (! is_null($r)) return $r;

	// The Variable was not GETed. See if it was POSTed.
	$r = GetPOST($VarName);

	// Variable was POSTed, return it now.
	if (! is_null($r)) return $r;

	// Variable was not POSTed or GETed. Return NULL.
	return null;

}


function GetPOST($VarName) {
	/*
	Purpose
		Safely retrieves POSTed data.
	Parameters
		VarName (string) The name of the POST variable we want to retrieve.
	Return (string, null)
		Returns the POSTed element's value properly stripped, if necessary.
		Returns NULL if:
			- The specified variable (VarName) was not specified.
			- The specified variable (VarName) is not a string (it could be an array).
	Version History
		1.00 2009.6.24 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Trap: Variable not specified, return NULL.
	if (! array_key_exists($VarName, $_POST)) return null;

	// Trap: Variable is not a string, return NULL.
	if (! is_string($_POST[$VarName])) return null;

	// Return properly escaped value.
	return get_magic_quotes_gpc() ? stripslashes($_POST[$VarName]) : $_POST[$VarName];

}


function GetSession($VarName) {
	/*
	Purpose
		Safely retrieves session data.
		This is used to eliminate notices on strict systems.
	Parameters
		VarName (string) The name of the session variable we want to retrieve.
	Return (string, null)
		Returns the session variable's value.
		Returns NULL if the specified session variable (VarName) was not specified.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Trap: Variable not specified, return NULL.
	if (! array_key_exists($VarName, $_SESSION)) return null;

	// Session variable exists, return it.
	return $_SESSION[$VarName];

}


function HTML_ConvertIMDB_Comments($IMDB, $Template, $ColumnPrefix) {
	/*
	Purpose
		Used to convert an in-memory database (array of associative arrays) into displayable HTML.
	Parameters
		IMDB (array of associative arrays) = The in-memory database we want to convert.
		Template (string) = The template we will use to build the HTML.
		ColumnPrefix (string) = The prefix used on all columns in the database.
			We need this because the IMDB uses it for keys in the associative arrays it contains.
	Return (string)
		Returns a string of HTML code ready to be displayed.
	Version History
		1.00 2009.6.22 Tested by Russ Tanner on PHP 5.1.6.
	*/

	$r = '';				// Return buffer.

	$p = $ColumnPrefix;		// Convenience.

	// We need to maintain a serial number for the "SN" token.
	$i = 1;

	// Iterate the in-memory database.
	foreach ($IMDB as $v) {

		// Create a token list for the current record.
		$Tokens = array(
			'SN'			=> $i,
			'Subject'		=> strlen(trim($v[$p.'subject'])) > 0 ? htmlspecialchars($v[$p.'subject']) : '(no subject)',
			'Author'		=> htmlspecialchars($v[$p.'author']),
			'Date'			=> date(cfg::$DateFormat, $v[$p.'date']),
			'IP'			=> htmlspecialchars($v[$p.'host']),
			'ID'			=> htmlspecialchars($v[$p.'id']),
			'Body'			=> htmlspecialchars($v[$p.'comment']),
			'PageURL_Main'	=> cfg::$GlobalTokens['PageURL_Main']
		);
		
		// Build the template for the current record and append it to the return buffer.
		$r .= ExpandTokens($Template, $Tokens);

		// Increment our serial number.
		$i += 1;

	}

	return $r;

}


function HTML_TextArea_ToArray($Str, $DeleteWS = false, $TrimWS = true) {
	/*
	Purpose
		Used to convert the raw string from a TEXTAREA to a clean array.
	Parameters
		Str (string) = The raw string we want to convert.
			This list can be delimited by any of the following ASCII character combinations: 13/10, 13, 10
		DeleteWS (optional boolean) = If true, elements in the returned array containing only whitespace are removed.
			Default: false
		TrimWS (optional boolean) = If true, all elements in the returned array are trimmed of all 
			leading and following whitespace.
			Default: true
	Return (array)
		Returns an array of strings containing each line from the specified string (Str).
		Returns an empty array if the specified string is empty.
	Version History
		1.00 2009.6.20 Tested by Russ Tanner on PHP 5.1.6 using IE (Mac/Win), Firefox (Mac/Win), and Opera (Mac).
	*/

	// Convert the list to an array.
	$r = preg_split('/(\r\n|\n|\r)/', $Str);				// Handle all possible combination of line endings (EOLs)

	// Remove any elements containing only whitespace.
	if ($DeleteWS) $r = Arr_Delete_Whitespace($r);

	// Trim all whitespace from all elements. This also removes any EOL characters if they remain.
	if ($TrimWS) $r = Arr_Trim_Whitespace($r);

	return $r;

}


function Input_Get_Ordered($ParmName, $FetchOrder = 'PGS', $DefaultValue = null, $SessionName = null) {
	/*
	Purpose
		Used to get either parameter input (GET/POST) or session data (from the SESSION object)
		in a specified order of precedence.
	Parameters
		ParmName (string) = The name of the parameter we're getting.
			This value is assumed to be the same for GET, POST, or SESSION (unless SESSION is overridden).
		FetchOrder (optional string) = This determines the order of precedence between GET, POST, and SESSION.
			This contains a string of any of the following characters:
				P - POST
				G - GET
				S - SESSION
			Each single character is examined in order from left to right.
			The specified parameter (ParmName) is searched for in each array (GET, POST, or SESSION).
			As soon as it is found, it is returned.
			Default: "PGS"
		DefaultValue (optional string) = The value that is returned if no matching parameters are found.
			Default: NULL
		SessionName (optional string) = The element key (name) to use when searching the session object.
			This parameters enables you to have a session key that is different than the GET or POST key.
			Default if null: Defaults to the same value as specified in ParmName.
			Default: null
	Return (string)
		Returns the parameter value (or SESSION array value) matching the specified parameter name (ParmName).
		The search is made through all array specified in Order.
		The parameter that is found with a matching name is returned.
		If no matching parameter is found, function returns the specified default value (DefaultValue).
	Version History
		1.00 2009.6.16 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Default parameters
	if (is_null($SessionName)) $SessionName = $ParmName;

	// Iterate the Order string
	for ($i=0; $i<strlen($FetchOrder); $i++) {

		// Process the current fetch order code
		switch(substr($FetchOrder, $i, 1)) {
		case 'P':
			if (array_key_exists($ParmName, $_POST)) return GetPOST($ParmName);
			break;
		case 'G':
			if (array_key_exists($ParmName, $_GET)) return GetGET($ParmName);
			break;
		case 'S':
			if (array_key_exists($SessionName, $_SESSION)) return GetSession($SessionName);
			break;
		default:
			SayPage__BugTrap('In Input_Get_Ordered(), Order contains an invalid character');
			exit(0);
		}
		
	}

	// A matching parameter was not found. Return the specified default value.
	return $DefaultValue;

}


function Input_Unset_FormFields($List) {
	/*
	Purpose
		Used to unset specified form field elements in the _GET, _POST, and _SESSION arrays.
	Parameters
		List (array) = This contains a list of elements we want to unset.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	GET_Unset_FormFields($List);
	POST_Unset_FormFields($List);
	SESSION_Unset_FormFields($List);

}


function ListName_IsValid($ListName) {
	/*
	Purpose
		Used to determine if the specifed list name is valid.
	Parameters
		ListName (string) = The name of the list we are testing.
	Return (boolean)
		Returns true if the specified list name is valid.
		Otherwise, returns false.
	Version History
		1.00 2009.6.18 Tested by Russ Tanner on PHP 5.1.6.
	*/

	return array_key_exists($ListName, cfg::$map_ListToFile);

}


function ListName_ValidateAndSave($ListName) {
	/*
	Purpose
		Used to validate and save the passed list that has the specified name (ListName).
	Parameters
		ListName (string) = The name of the list we will validate and save.
	Return (array, integer)
		Returns the contents of the specified list as an array.
			The list is properly retrieved from passed parameters and each element is trimmed.
		If an error occurs, function returns a status code.
			Possible values:
				STATUS_InvalidListName
					- The specified list name was invalid, or
					- The list with the specified name was not passed.
				STATUS_ErrorWritingListFile
	Version History
		1.00 2009.6.18 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Error Trap: The specified list is invalid. Return a status code.
	if (! ListName_IsValid($ListName)) return STATUS_InvalidListName;

	// Get the raw list we want to save from current parameters and convert it to an array (properly).
	// We do this so List_Save_FromArray() will use the correct EOL characters for the current platform.
	$ListString = GetInput($ListName);
	// Error Trap: The list with the specified name was not passed. Return a status code.
	if (is_null($ListString)) return STATUS_InvalidListName;

	// Convert the fetched list to a clean array.
	$ListArray = HTML_TextArea_ToArray($ListString);

	// Save the list and return a status code.
	$e = List_Save_FromArray($ListName, $ListArray);
	// Error Trap: Return a status code.
	if ($e != STATUS_OK) return $e;

	// No errors occurred. Return the cleaned list as an array.
	return $ListArray;

}


function List_Clear($ListName) {
	/*
	Purpose
		Used to clear the specified list.
		This makes the associated list file zero-length (empty).
	Parameters
		ListName (string) = The name of the list we want to clear.
			This is not a file name. It is a proper list name which gets mapped to a file name.
	Return (integer)
		Returns a status code.
		Possible return values:
			STATUS_OK
			STATUS_ErrorWritingListFile
	Entry Conditions
		The specified list name (ListName) must be valid or this function will crash.
			This is an optimization.
	Version History
		1.00 2009.6.18 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Save the list file with nothing in it.
	return List_Save($ListName, '');

}


function List_Create($ListName, $UseDefaults = true) {
	/*
	Purpose
		Used to create a blank list file.
		If the list file already exists, nothing changes.
	Parameters
		ListName (string) = The name of the list we're working with.
		UseDefaults (optional boolean, string) = 
			If this is a boolean:
				If true, if the file is created, it is filled with defaults from the cfg object.
				If false, if the file is created, it is created empty.
			If this is a string:
				If the file is created, it is filled with this value.
			Default: true
	Return (integer)
		Returns a status code.
		Possible return values:
			STATUS_OK
			STATUS_ErrorWritingListFile
	Entry Conditions
		The specified list name (ListName) must be a valid list name or this function may crash.
			This is an optimization.
	Version History
		1.00 2009.6.11 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Trap: The list file already exists. Do nothing. Return OK status.
	if (List_Exists($ListName)) return STATUS_OK;

	// Determine what we are filling the file with.
	if (is_string($UseDefaults)) $File = $UseDefaults;
	else $File = $UseDefaults ? cfg::$DefaultFieldValues[$ListName] : '';

	// The list file does not exist. Create it.
	return List_Save($ListName, $File);

}


function List_Exists($ListName) {
	/*
	Purpose
		Used to determine if the specified list file exists.
	Parameters
		ListName (string) = The name of the list we're working with.
	Return (boolean)
		Returns true if the specified list exists.
		Returns false if the specified list does not exist.
	Entry Conditions
		The specified list name (ListName) must be a valid list name or this function may crash or return an invalid value.
			This is an optimization.
	Version History
		1.00 2009.6.11 Tested by RussTanner on PHP 5.1.6.
	*/

	return is_file(cfg::$map_ListToFile[$ListName]) ? true : false;

}


function List_Line_GetCurrent($ListName, $Index) {
	/*
	Purpose
		Used to get the current line in a list file.
	Parameters
		ListName (string) = The name of the list we're working with.
		Index (integer) = A zero-based index pointing to the current line in the list file.
			We want to fetch this line and return it.
	Return (string, integer)
		Returns the line of the specified list file (ListName) pointed to by the specified (zero-based) index (Index).
		Returns an integer if an error occurs.
			Possible return values:
					STATUS_OK
					STATUS_ErrorReadingListFile
					STATUS_InvalidIndex
	Entry Conditions
		The specified list name (ListName) must be a valid list name or this function may crash or return an invalid value.
			This is an optimization.
	Version History
		1.00 2009.6.16 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Load the list as an array.
	$r = List_Load_AsArray($ListName);
	// Error Trap: Error reading list file. Return a status code.
	if (is_int($r)) return STATUS_ErrorReadingListFile;

	// Error Trap: Invalid Index. Return a status code.
	if ($Index < 0 || $Index >= count($r)) return STATUS_InvalidIndex;

	// Return the specified line.
	return $r[$Index];

}


function List_Line_GetNext($ListName, $Index) {
	/*
	Purpose
		Used to get the next line in a list file that contains a value.
	Parameters
		ListName (string) = The name of the list we're working with.
		Index (integer) = A zero-based index pointing to the current line in the list file.
			We want to fetch the first line following this one that has a value.
	Return (associative array, integer)
		Returns an associative array with the following elements:
			"Line" (string, null) = The next line in the file that contains a value.
				This returns null and Status return STATUS_OK if:
					- No lines after the specified index (Index) contain a value, or
					- Index is >= the last line in the file.
					- A status condition occurs.
			"Index" (integer) = The zero-based index of the line we're returning.
				This value has meaning only if Line is not null.
				If a status condition occurs, this value is the same as the specified index (Index).
		If an error occurs, returns a status code (integer).
			Possible return values:
				STATUS_ErrorReadingListFile
	Entry Conditions
		The specified list name (ListName) must be a valid list name or this function may crash or return an invalid value.
			This is an optimization.
	Version History
		1.00 2009.6.16 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Return buffer.
	$r = array(
		'Line'		=> null,
		'Index'		=> $Index,
	);

	// Load the list as an array.
	$File = List_Load_AsArray($ListName);
	// Error Trap: Error reading list file. Return a status code.
	if (is_int($File)) return $File;

	// Iterate the lines in the file searching for the next one with a value.
	$LineCount = count($File);
	for ($i = $Index + 1; $i < $LineCount; $i++) {

		// Determine what type of line this is (value, blank, etc.)
		$Line = $File[$i];
		$LineType = List_Line_GetType($Line);

		// We found a value, return it.
		if ($LineType == LINETYPE_Value) {
			$r['Line'] = $Line;
			$r['Index'] = $i;
			return $r;
		}

	}

	// No remaining lines have a value. Line returns null.
	$r['Line'] = null;

	return $r;

}


function List_Line_GetType($Line) {
	/*
	Purpose
		Used to determine what type of data is contained on a specified list's line.
	Parameters
		Line (string) = The line we're working with.
	Return (integer)
		Returns a LINETYPE constant.
			Possible return values:
				LINETYPE_Value
				LINETYPE_Blank
	Version History
		1.00 2009.6.17 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// The line is empty or contains only whitespace. Line type is blank.
	if (strlen(trim($Line)) == 0) return LINETYPE_Blank;

	// This function could be improved (here) in the future to handle comments.
	// For now, we don't want to support comments in list files.

	// The line is not empty. It must be a value.
	return LINETYPE_Value;

}


function List_Load($ListName) {
	/*
	Purpose
		Used to load a list of values from a list file.
	Parameters
		ListName (string) = The list name representing the file we want to load to.
			This is not a file name. It is a proper list name which gets mapped to a file name.
	Return (string, integer)
		Returns the contents of the specified list file as a string.
		Returns an integer status code if an error occurs.
			Possible status codes:
				STATUS_OK
				STATUS_ErrorReadingListFile
				STATUS_ErrorWritingListFile
	Entry Conditions
		The specified list name (ListName) must be a valid list name or this function may crash.
			This is an optimization.
	Version History
		1.00 2009.6.11 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Make sure the list file exists. If it doesn't, create it.
	$e = List_Create($ListName);
	// Error Trap: Error writing list file. Return a status code.
	if ($e != STATUS_OK) return $e;

	// Load the file.
	$File = file_get_contents(cfg::$map_ListToFile[$ListName]);
	// Error Trap: Error reading file. Return status code.
	if ($File === false) return STATUS_ErrorReadingListFile;

	// Success. Return the file contents.
	return $File;

}


function List_Load_AsArray($ListName) {
	/*
	Purpose
		Used to load a list file and return it as an array.
	Parameters
		ListName (string) = The name of the list file we're working with.
			This is not a file name. It is a proper list name which gets mapped to a file name.
	Return (array, integer)
		Returns an array. Each line of the array contains a line from the specified list file (ListName).
		If an error occurs, returns a status code (integer).
			Possible return values:	
				STATUS_ErrorReadingListFile
	Entry Conditions
		The specified list name (ListName) must be a valid list name or this function may crash.
			This is an optimization.
	Version History
		1.00 2009.6.16 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Load the list file as an array.
	$r = File_AsArray(cfg::$map_ListToFile[$ListName]);
	// Error Trap: Error reading list file. Return a status code.
	if (is_int($r)) return STATUS_ErrorReadingListFile;

	// No errors occurred. Return the array.
	return $r;

}


function List_RemoveDuplicates($ListName) {
	/*
	Purpose
		Used to purge duplicates from the specified list.
	Parameters
		ListName (string) = The list name representing the file we want to purge duplicates from.
			This is not a file name. It is a proper list name which gets mapped to a file name.
	Return (integer)
		Returns a status code.
		Valid values:
			STATUS_OK
			STATUS_ErrorReadingListFile
			STATUS_ErrorWritingListFile
	Entry Conditions
		The specified list name (ListName) must be a valid list name or this function may crash.
			This is an optimization.
	Version History
		1.00 2009.6.16 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Load the list file.
	$ListArray = List_Load_AsArray($ListName);
	// Error Trap: An error occured. Return status code.
	if (is_int($ListArray)) return $ListArray;

	// Remove duplicates.
	$ListArray = array_unique($ListArray);
	$ListArray = array_values($ListArray);		// Compact the array.

	// Save the list to to correct file.
	$e = List_Save_FromArray($ListName, $ListArray);
	// Error Trap: Error writing file. Return status code.
	if ($e != STATUS_OK) return $e;

	// Success. Return good status code.
	return STATUS_OK;

}


function List_Save($ListName, $Values) {
	/*
	Purpose
		Used to save a list of values to a list file.
	Parameters
		ListName (string) = The list name representing the file we want to save to.
			This is not a file name. It is a proper list name which gets mapped to a file name.
		Values (string) = The raw data we want to save to the file.
	Return (integer)
		Returns a status code.
		Valid values:
			STATUS_OK
			STATUS_ErrorWritingListFile
	Entry Conditions
		The specified list name (ListName) must be valid or this function will crash.
			This is an optimization.
	Version History
		1.00 2009.6.18 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Get the name of the file we're saving to
	$FileName = cfg::$map_ListToFile[$ListName];

	// Save the file. Error Trap: Error writing file. Return status code.
	if (file_put_contents($FileName, $Values) === false) return STATUS_ErrorWritingListFile;

	// Success. Return OK status code.
	return STATUS_OK;

}


function List_Save_FromArray($ListName, $Arr) {
	/*
	Purpose
		Used to save an array to the specified list file (ListName).
	Parameters
		ListName (string) = The list name representing the file we want to save to.
			This is not a file name. It is a proper list name which gets mapped to a file name.
		Arr (array) = The array containing each line we want to save to the file.
	Return (integer)
		Returns a status code.
		Valid values:
			STATUS_OK
			STATUS_ErrorWritingListFile
	Entry Conditions
		The specified list name (ListName) must be valid or this function will crash.
			This is an optimization.
	Version History
		1.00 2009.6.18 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Save the file. Error Trap: Error writing file. Return status code.
	if (file_put_contents(cfg::$map_ListToFile[$ListName], List_ToString($Arr)) === false) return STATUS_ErrorWritingListFile;

	// Success. Return good status code.
	return STATUS_OK;

}


function List_Sort($ListName) {
	/*
	Purpose
		Used to sort a list of values to a list file.
	Parameters
		ListName (string) = The list name representing the file we want to sort.
			This is not a file name. It is a proper list name which gets mapped to a file name.
	Return (integer)
		Returns a status code.
		Valid values:
			STATUS_OK
			STATUS_ErrorReadingListFile
			STATUS_ErrorSortingList
			STATUS_ErrorWritingListFile
	Entry Conditions
		The specified list name (ListName) must be a valid list name or this function may crash.
			This is an optimization.
	Version History
		1.00 2009.6.16 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Load the list file.
	$e = List_Load_AsArray($ListName);
	// Error Trap: An error occurred. Return a status code.
	if (is_int($e)) return $e;

	// Remove blank lines. This must be done or the result of a sort may be strange (i.e.: leading blank lines).
	$e = Arr_Delete_Whitespace($e);

	// Sort the list.
	// Error Trap: Error on sort. Return a status code.
	if (! natcasesort($e)) return STATUS_ErrorSortingList;

	// Save the list to the correct file.
	$e = List_Save_FromArray($ListName, $e);
	// Error Trap: Error writing file. Return a status code.
	if ($e != STATUS_OK) return $e;

	// Success. Return good status code.
	return STATUS_OK;

}


function List_ToString($Arr) {
	/*
	Purpose
		Used to convert an array into a raw list, ready for saving.
	Parameters
		Arr (array) = The array we want to convert.
	Return (string)
		Returns the specified array (Arr) converted into a string, ready for saving.
		Newlines are automatically selected depending on the platform in which we are running (using PHP_EOL).
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	return implode(PHP_EOL, $Arr);

}


function Log_Write($Msg) {
	/*
	Purpose
		Used to append an entry to the end of the log file.
	Parameters
		Msg (string) = The message we want to log.
	Version History
		1.00 2009.6.19 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Get the filename of the log file.
	$FileName = cfg::$Log_FileName;

	// Write to the file.
	$e = file_put_contents($FileName, $Msg, FILE_APPEND);

	return null;

}


function Msg_Build($Name, $Tokens = null, $Prepend = '', $Postend = '') {
	/*
	Purpose
		Used to build a standard message (from cfg::Msgs) and expand any specified tokens.
	Parameters
		Name (string) = The name of the message we're working with.
		Tokens (optional array) = The list of tokens we're going to expand in the message.
			Default: null
		Prepend (optional string) = A string that can be prepended to the message.
			This is prepended before token expansion, so it can contain token-placeholders.
			Default: "" (zero-length)
		Postend (optional string) = A string that can be appended to the message.
			This is prepended before token expansion, so it can contain token-placeholders.
			Default: "" (zero-length) 
	Return (string)
		Returns the specified message (Name) expanded with all specified tokens (Tokens).
	Entry Conditions
		The specified message name (Name) must exist in cfg::Msgs or this function will crash.
	Version History
		1.00 2009.06.18 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Fetch the specified message
	$Template = $Prepend . cfg::$Msgs[$Name] . $Postend;

	// Tokens have been specified, expand them.
	if (! is_null($Tokens)) $Template = ExpandTokens($Template, $Tokens);

	return $Template;

}


function Parm_Exists($VarName, $CheckSession = false, $VarName_Session = null) {
	/*
	Purpose
		Used to determine if a specified parameter was POSTed or GETed.
		Can optionally check the session as well.
	Parameters
		VarName (string) = The name of the variable we're testing.
		CheckSession (optional boolean) = If true, we also see if the value is available in the session array.
			Default: false
		VarName_Session (optional string) = This is the name of the session variable. This option exists in case
			the session variable name we want to check is different than the GET and POST names.
			Default: This defaults to the same value as VarName.
	Return (boolean)
		If the specified variable was POSTed or GETed, this function returns true.
		If not, this function returns false.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Default parameters
	if (is_null($VarName_Session)) $VarName_Session = $VarName;

	// Check the session too
	if ($CheckSession) return (array_key_exists($VarName, $_POST) || array_key_exists($VarName, $_GET) || array_key_exists($VarName_Session, $_SESSION)) ? true : false; 

	// Don't check the session
	return (array_key_exists($VarName, $_POST) || array_key_exists($VarName, $_GET)) ? true : false;

}


function Parm_Get_DB($CredentialsOnly = false) {
	/*
	Purpose
		Used to return all parameters related to the database as an associative array.
	Parameters
		CredentialsOnly (boolean) = If true, only the actual database credentials are required. In this case,
			no error will be returned in TablePrefix or ColumnPrefix is missing.
			If false, all 6 database-related parameters are requred. If any are missing, an error code is returned.
			Default: false
	Return (associative array, integer)
		Returns all database-related parameters as an associative array.
		This includes the following elements (keys):
			DBServer, DBUsername, DBPassword, DBName, TablePrefix, ColumnPrefix
		Returns a status code if:
			- One or more parameters was not specified (GETed or POSTed).
			- One or more credentials (DBServer, DBUsername, DBPassword, DBName) was zero-length.
		Possible Status Codes:
			STATUS_DatabaseParameterMissing
	Version History
		1.00 2009.06.09 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Preload the return buffer.
	$r = array(
		'DBServer'		=> '',
		'DBUsername'	=> '',
		'DBPassword'	=> '',
		'DBName'		=> ''
	);

	// CredentialsOnly is false, we require all database-related parameters.
	if (! $CredentialsOnly) {
		$r = array_merge($r, array(
			'TablePrefix'	=> '',
			'ColumnPrefix'	=> ''
		));
	}

	// Iterate all elements in the return buffer.
	foreach($r as $k => $v) {

		// Retrieve the parameter.
		$x = GetInput($k);
		// Error Trap: The parameter was not specified. Return a status code.
		if (is_null($x)) return STATUS_DatabaseParameterMissing;
		// Error Trap: A database credential was not specified (zero-length). Return a status code.
		if ($k == 'DBServer' || $k == 'DBUsername' || $k == 'DBPassword' || $k == 'DBName') {
			if (strlen($x) == 0) return STATUS_DatabaseParameterMissing;
		}

		// Save the parameter's value to the return buffer.
		$r[$k] = $x;

	}

	return $r;

}


function Parm_FillArray($ParmList, $DefaultValue = null, $Overwrite = null, $FetchOrder = null) {
	/*
	Purpose
		Used to return all parameters that are listed in the specified array (ParmList).
	Parameters
		ParmList (associative array) = The keys of this array are the parameter names (GET/POST names) that we're
			going to use to fill in the values of the array.
		DefaultValue (optional mixed) = The value assigned to the value of returned array elements that
			have not been passed as parameters.
			Default: null
		Overwrite (optional boolean) = This feature allows elements in the parameter list (ParmList) that already
			have values to be preserved. If this is false, elements in the parameter list (ParmList) that already 
			have values (are not null) are not changed. If this is true, all elements in the parameter list are 
			populated so long as they have been passed (POSTed or GETed).
			Default if null: false
			Default: null
		FetchOrder (optional string) = This determines the order of precedence between GET, POST, and SESSION.
			This contains a string of any of the following characters:
				P - POST
				G - GET
				S - SESSION
			Each single character is examined in order from left to right.
			The specified parameter (ParmName) is searched for in each array (GET, POST, or SESSION).
			As soon as it is found, it is returned.
			Default if null: "PGS"
			Default: null
	Return (associative array)
		Returns all of the elements specified in ParmList with their values updated with the parameter that has
			the same name as the key.
	Entry Conditions
		FetchOrder must be valid or a subfunction (Input_Get_Ordered()) will display a bug trap.
	Version History
		1.00 2009.06.09 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Default parameters.
	if (is_null($Overwrite)) $OverWrite = false;

	// Return buffer.
	$r = array();

	// Iterate all elements in the return buffer.
	foreach($ParmList as $k => $v) {
		
		// The value is null, or Overwrite is true. Go ahead and fetch a value.
		$r[$k] = (is_null($v) || $Overwrite == true) ? Input_Get_Ordered($k, $FetchOrder, $DefaultValue) : $ParmList[$k];

	}

	return $r;

}


function POST_Unset_FormFields($List) {
	/*
	Purpose
		Used to unset specified form field elements in the _POST array.
	Parameters
		List (array) = This contains a list of elements we want to unset in _POST.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	foreach($List as $k) unset($_POST[$k]);

}


function SayPage__About() {
	/*
	Purpose
		Used to display the "About" page.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Build and print the "Bones" template
	print Template_Build__Bones(Template_Get('About'));

}


function SayPage__BugTrap($Msg) {
	/*
	Purpose
		Used to display the "BugTrap" page.
	Parameters
		Msg (string) = The main message to display.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Append the call stack to the specified message.
//	$CallStackArray = apd_callstack();
//	$CallStackString = implode("<br />\n", $CallStackArray);
//	$Msg .= "<br />\n<br />\n$CallStackString";

	// Build the template body: "Error"
	$Template = Template_Get('Error');
	$Tokens = array(
		'Title'			=> cfg::$def_Title_BugTrap,
		'Message'		=> $Msg,
		'URLContinue'	=> cfg::$GlobalTokens['PageURL_Main']
	);

	$tpl_Error = ExpandTokens($Template, $Tokens);

	// Build and print the "Bones" template
	print Template_Build__Bones($tpl_Error);

}


function SayPage__Confirm($Title, $Msg, $Label1, $URL1, $Label2, $URL2) {
	/*
	Purpose
		Used to display the "Confirm" page.
	Parameters
		Title (string) = The title to display.
		Msg (string) = The main message to display.
		Label1 (string) = The text that is displayed for the first link option (usually "Yes").
		URL1 (string) = The URL that will be assigned to the first link option.
		Label2 (string) = The text that is displayed for the second link option (usually "No").
		URL2 (string) = The URL that will be assigned to the second link option.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Build the template body: "Confirm"
	$Template = Template_Get('Confirm');
	$Tokens = array(
		'Title'			=> $Title,
		'Message'		=> $Msg,
		'Label1'		=> $Label1,
		'URL1'			=> $URL1,
		'Label2'		=> $Label2,
		'URL2'			=> $URL2
	);
	$tpl_Confirm = ExpandTokens($Template, $Tokens);
	
	// Build and print the "Bones" template
	print Template_Build__Bones($tpl_Confirm);

}


function SayPage__Confirmed($Msg, $Title, $URLContinue = null) {
	/*
	Purpose
		Used to display the "Confirmed" page.
	Parameters
		Msg (string) = The main message to display.
		Title (string) = The title to display.
		URLContinue (optional string) = The URL of the "continue" link.
			Default: GlobalToken['PageURL_Main']
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Default parameters
	if (is_null($URLContinue)) $URLContinue = cfg::$GlobalTokens['PageURL_Main'];

	// Build the template body: "Error"
	$Template = Template_Get('Confirmed');
	$Tokens = array(
		'Title'			=> $Title,
		'Message'		=> $Msg,
		'URLContinue'	=> $URLContinue
	);
	$tpl_Confirmed = ExpandTokens($Template, $Tokens);

	// Build and print the "Bones" template
	print Template_Build__Bones($tpl_Confirmed);

}


function SayPage__Error($Msg, $Title = null, $URLContinue = null, $Opt = null) {
	/*
	Purpose
		Used to display the "Error" page.
	Parameters
		Msg (string, integer) = Used to determine what message is displayed.
			If this is an integer:
				This represents a status code. In this case, this code is converted
				into the standard message associated with this code and then displayed.
			If this is a string:
				This is the message that will be displayed.
		Title (optional string) = The title to display.
			Default or null: cfg::def_Title_Error
		URLContinue (optional string) = The URL of the "continue" link.
			Default or null: GlobalToken['PageURL_Main']
		Opt (optional string, associative array, null) = Used to determine what message is displayed.
			If this is a string:
				This message is appended to the original message (Msg) after an HTML break (<br />).
			If this is an associative array:
				This maps status codes to messages.  Having this parameter provides a very powerful way to customize 
				displayed messages when you don't know what message is going to be displayed. This way, this function
				can be called as a generic error handler after a function call but messages can be customized.
				Usually, when this is an associative array, this will be a modification of cfg::map_StatusToMsg.
			Default or NULL:
				No message is appended to the original specified message (Msg).
	Entry Conditions
		If Msg is an integer, it must be a valid error code as defined in cfg::map_StatusToMsg or a bug trap will display.
	Version History
		1.00 2009.6.9
	*/

	// Default parameters
	if (is_null($Title)) $Title = cfg::$def_Title_Error;
	if (is_null($URLContinue)) $URLContinue = cfg::$GlobalTokens['PageURL_Main'];

	// Msg is an integer. Convert it to a standard (or customized) error message.
	if (is_int($Msg)) {
		// Opt is an array. Use it to map to a message instead of the standard Status-To-Message map.
		if (is_array($Opt)) {
			// Bug Trap: The specified message list does not contain the specified key.
			if (! array_key_exists($Msg, $Opt)) {
				SayPage__BugTrap('In SayPage__Error(), parameter Msg does not exist in parameter Opt.');
				exit(1);
			}
			$Msg = $Opt[$Msg];
		}
		// Opt is not an array. Use the standard map to get the main message.
		else {
			// Bug Trap: The specified message list does not contain the specified key.
			if (! array_key_exists($Msg, cfg::$map_StatusToMsg)) {
				SayPage__BugTrap('In SayPage__Error(), parameter Msg does not exist in cfg::map_StatusToMsg.');
				exit(1);
			}
			$Msg = cfg::$map_StatusToMsg[$Msg];
		}
	}
	
	// Opt is a string. Append it to the main message.
	if (is_string($Opt)) $Msg .= "<br />\n$Opt";

	// Build the template body: "Error"
	$Template = Template_Get('Error');
	$Tokens = array(
		'Title'			=> $Title,
		'Message'		=> $Msg,
		'URLContinue'	=> $URLContinue
	);
	$tpl_Error = ExpandTokens($Template, $Tokens);

	// Build and print the "Bones" template
	print Template_Build__Bones($tpl_Error);

}


function SayPage__Error_ByStatus($StatusCode, $Title = null, $URLContinue = null) {
	/*
	Purpose
		Used to display a standard error message based on the specified status code.
	Parameters
		StatusCode (integer) = The status code representing the error message we went to display.
		Title (optional string, null) = The title we want to display.
			If NULL, defaults to cfg::def_Title_Error
			Default: NULL
		URLContinue (optional string, null) = The URL of the "continue" link.
			If NULL, defaults to cfg::GlobalToken['PageURL_Main']
			Default: NULL
	Return (integer)
		Returns the specified status code (StatusCode).
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Let a function do the hard work.
	SayPage__Error($StatusCode, $Title, $URLContinue, Err_GetFriendlyMap());

	// Return the specified status code.
	return $StatusCode;

}


function SayPage__License() {
	/*
	Purpose
		Used to display the "License" page.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Build and print the "License" template
	print Template_Build__Bones(Template_Get('License'));

}


function SayPage__Main() {
	/*
	Purpose
		Used to display the "Main" page.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Build the "Bones" template
	print Template_Build__Bones(Template_Build__Main());

}


function SayPage__Msg($Msg, $Title, $URLContinue = null) {
	/*
	Purpose
		Used to display the "Message" page.
	Parameters
		Msg (string) = The main message to display.
		Title (string) = The title to display.
		URLContinue (optional string) = The URL of the "continue" link.
			Default: GlobalToken['PageURL_Main']
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Default parameters
	if (is_null($URLContinue)) $URLContinue = cfg::$GlobalTokens['PageURL_Main'];

	// Build the template body: "Error"
	$Template = Template_Get('Msg');
	$Tokens = array(
		'Title'			=> $Title,
		'Message'		=> $Msg,
		'URLContinue'	=> $URLContinue
	);
	$tpl_Msg = ExpandTokens($Template, $Tokens);

	// Build and print the "Bones" template
	print Template_Build__Bones($tpl_Msg);

}


function SayPage__SysReq() {
	/*
	Purpose
		Used to display the "System Requirements" page.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Build the "Bones" template
	print Template_Build__Bones(Template_Get('SysReq'));

}


function SayPage__Usage() {
	/*
	Purpose
		Used to display the "Usage" page.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Build the template body: "Usage"
	$Template = Template_Get('Usage');
	$Tokens = array(
		'PageURL_Usage'	=> cfg::$GlobalTokens['PageURL_Usage']
	);
	$Body = ExpandTokens($Template, $Tokens);

	// Build and print the "Bones" template
	print Template_Build__Bones($Body);

}


function Session_Unset_FormFields($List) {
	/*
	Purpose
		Used to unset specified form field elements in the _SESSION array.
	Parameters
		List (array) = This contains a list of elements we want to unset in _SESSION.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	foreach($List as $k) unset($_SESSION[cfg::$SessionPrefix_FormFields . $k]);

}


function Status_Set($Status = null, $ReturnValue = null, $ReturnNull = false) {
	/*
	Purpose
		Used to set a global status condition.
		As a convenience, this also returns a specified value.
	Parameters
		Status (optional integer) = The value we will assign to the global status code.
			Default if null: STATUS_OK
			Default: null
		ReturnValue (optional mixed) = The value to return.
			Default if null: Assigned the value of the parameter Status
			Default: null
		ReturnNull (optional boolean) = If true, this function returns null regardless of the value of ReturnValue.
			Setting this parameter true is the only way to make this function will return null.
			Default: false
	Return (mixed)
		Returns the value of ReturnValue.
		If ReturnNull is true, function returns null.
	Version History
		1.00 2009.6.17 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Default parameters
	if (is_null($ReturnValue)) $ReturnValue = $Status;

	// Set the global status code.
	$StatusCode = is_null($Status) ? STATUS_OK : $Status;

	// Return the correct value.
	return $ReturnNull ? null : $ReturnValue;

}


function Tag_Attributes_Build($Attributes) {
	/*
	Purpose
		This is a generalized low-level tag-building function.
	Parameters
		Attributes (AA) = This is a list of attributes that are added to the tag.
			Keys are the attribute name. Values are the attribute value.
	Version History
		1.00 2008.6.22 by Russ Tanner
	*/

	$r = ''; // return buffer

	// Split Attributes array into two normal arrays so there are no problems when html-encoding the names array
	$Names = array_keys($Attributes);
	$Values = array_values($Attributes);

	// Escape the names and values according to the specified escape levels
	$Names = Arr_EscHTML($Names);
	$Values = Arr_EscHTML($Values);

	// Build the string from the names/values lists
	$imax = count($Names);
	for ($i=0; $i<$imax; $i++) $r .= ' ' . $Names[$i] . '="' . $Values[$i] . '"';

	return $r;

}


function Tag_Build($Attributes, $TagName, $TagClose = false) {
	/*
	Purpose
		This is a generalized low-level tag-building function.
	Parameters
		Attributes (AA) = This is a list of attributes that are added to the tag.
			Keys are the attribute name. Values are the attribute value.
		TagName (Optional String) = The name of the tag we're creating.
			This is normally something like "hidden" or "text" or "img".
		TagClose (Optional Boolean) = If true, the returned tag is closed with "/>" instead of just ">".
			Default: false
	Version History
		1.00 2008.6.23 by Russ Tanner
	*/

	// Build the attributes part of the tag
	$AttribString = Tag_Attributes_Build($Attributes);

	// Build the tag prefix and suffix
	$TagPrefix = '<' . $TagName;
	$TagSuffix = ($TagClose ? ' /' : '') . '>';

	// Add the prefix and suffix and return the result
	return $TagPrefix . $AttribString . $TagSuffix;
	
}


function Tag_Build_INPUT_Hidden($TagName, $TagValue) {
	/*
	Purpose
		Used to build an HTML input tag of type "hidden".
	Parameters
		TagName (String) = This value is assigned to the "name" attribute of the tag.
		TagValue (String) = This value is assigned to the "value" attribute of the tag.
	Return (String)
		Returns an HTML input tag of type "hidden".
	Version History
		1.0 6/22/2008 by Russ Tanner
	*/

	// Build an array for the call to Tag_Build()
	$Attributes = array('type'=>'hidden', 'name'=>$TagName, 'value'=>$TagValue);

	// Build the array and return the result
	return Tag_Build($Attributes, 'input', true);
	
}


function Tags_Build_INPUT_Hidden($TagValues, $PreStr = '', $AppStr = "\n") {
	/*
	Purpose
		Used to build a series of HTML input tags of type "hidden".
	Parameters
		TagValues (AA) = Keys are assigned to attribute names and values are assigned to attribute values.
		PreStr (Optional String) = This string is prepended to each INPUT tag.
			Default: ""
		AppStr (Optional String) = This string is appended to each INPUT tag.
			Default: "\n"
	Return (String)
		Returns a series of HTML input tag of type "hidden".
		The name attribute of each tag is assigned from the key of each element of the specified array.
		The value attribute of each tag is assigned from the value of each element of the specified array.
		Each tag is prepended with the value specified in PreStr.
		Each tag is appended with the value specified in AppStr.
	Version History
		1.00 2008.6.22 by Russ Tanner
	*/

	$r = ''; // Return buffer
	
	// Loop through each specified tag value
	foreach ($TagValues as $k => $v) $r .= $PreStr . Tag_Build_INPUT_Hidden($k, $v) . $AppStr;
	
	return $r;

}


function Template_Build__Bones($Body, $Tokens = null) {
	/*
	Purpose
		Used to build the "Bones" template.
		All tokens in the Bones templates are automatically built except "Body".
	Parameters
		Body (string) = This is the string that will replace the "Body" token in the "Bones" template.
		Tokens (optional associative array) = This contains a list of token-values that will replace tokens 
			in the "Bones" template.
			Valid Values:
				"HEAD" (optional string) = This contains any additional information we want placed in 
					the HTML head section of the page.
					Default: "" (zero-length string)
				"TITLE" (optional string) = This is inserted in the <title> tag in the HTML head section of the page.
					Default: cfg::def_TITLE
				"CSS" (optional string) = This contains the style sheet code that will be inserted into 
					the HTML head of the page.
					Default: The "CSS" page-template.
				"JS" (optional string) = This contains the Javascript code that will be inserted into 
					the HTML head of the page.
					Default: The "JS" page-template.
				"Header" (optional string) = The "Header" section of the body of the page.
					Default: The "Header" page-template.
				"Menu" (optional string) = This is the main menu that is displayed on the page.
					Default: The "Menu" page-template.
			If not specified, defaults to NULL.
			Default: See values in list above.
	Return (string)
		Returns the "Bones" template completely built.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Default parameters
	if (is_null($Tokens)) $Tokens = array();

	// Build the Token array we will use to build the "Bones" template.
	$TokensFinal = array(
		'HEAD'			=> array_key_exists('HEAD', $Tokens) ? $Tokens['HEAD'] : '',
		'TITLE'			=> array_key_exists('TITLE', $Tokens) ? $Tokens['TITLE'] : cfg::$def_TITLE,
		'CSS'			=> array_key_exists('CSS', $Tokens) ? $Tokens['CSS'] : Template_Get('CSS', true),
		'JS'			=> array_key_exists('JS', $Tokens) ? $Tokens['JS'] : Template_Get('JS', true),
		'Header'		=> array_key_exists('Header', $Tokens) ? $Tokens['Header'] : Template_Get('Header'),
		'Menu'			=> array_key_exists('Menu', $Tokens) ? $Tokens['Menu'] : Template_Build__Menu(),
		'Body'			=> $Body
	);

	// Process the page bones
	$Template = Template_Get('Bones', true);				// Read the template. This is a base template.
	$Page = ExpandTokens($Template, $TokensFinal);			// Expand the final version of the token array.

	return $Page;

}


function Template_Build__Main() {
	/*
	Purpose
		Used to build the template named "Main".
	Return (string)
		Returns the template named "Main" with all tokens already expanded.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	$Template = Template_Get('Main');
	$Tokens = array_merge(cfg::$GlobalTokens, Fields_LoadPersistent());
	$Tokens = array_merge($Tokens, array(
		'SubjectList'		=> Field_GetDisplay_List('SubjectList'),
		'BodyList'			=> Field_GetDisplay_List('BodyList'),
		'AuthorList'		=> Field_GetDisplay_List('AuthorList'),
		'IPList'			=> Field_GetDisplay_List('IPList')
	));
	$Tokens = Tokens_Encode_HTML($Tokens);
	$r = ExpandTokens($Template, $Tokens);

	return $r;

}


function Template_Build__Menu() {
	/*
	Purpose
		Used to build the template named "Menu".
	Return (string)
		Returns the template named "Menu" with all tokens already expanded.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	$Template = Template_Get('Menu');
	$Tokens = array(
	);
	$Tokens = array_merge(cfg::$GlobalTokens, $Tokens);
	$Tokens = Tokens_Encode_HTML($Tokens);
	$r = ExpandTokens($Template, $Tokens);

	return $r;

}


function Template_Get($Name, $IsBase = null) {
	/*
	Purpose
		Used to retrive a template.
		This is essentially a wrapper for Template_Read(). If we're in development mode,
		templates are fetched from disk. If we're deployed, they are fetched from
		the global configuration object.
	Parameters
		Name (string) = The common name of the template we want to return.
			If we are in development mode, this function uses the template-to-file map to
			get the name of the file to use.
		IsBase (optional boolean) = Used to determine if this is a base template.
			If true, this is a base template. If false, this is not a base template.
			Default: false
	Return (string)
		This function returns the specified template.
		If we are not in development mode, the IsBase parameter is not utilized because when templates
		are stored in the global configuration object ("cfg"), only the necessary part of the templates
		are stored. No parsing is necessary.
		The following statements only apply if we are in development mode.
			If IsBase is true, this is a base template. Therefore, the body is not stripped out.
				Instead, the entire template is returned as is.
			If IsBase is false, this is not a base template. Therefore, the body is stripped out.
				Only the body of the template is returned.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Default parameters
	if (is_null($IsBase)) $IsBase = false;								// IsBase defaults to false

	// We are in development mode. Get the template from disk.
	if (cfg::$DevMode) {

		// Get the filename of the specified template from the map.
		$FileName = cfg::$map_TemplateToFile[$Name];

		// Get the template from the file
		$r = Template_Read($FileName, $IsBase);

	}

	// We are not in development mode. Get the template from the global configuration object ("cfg").
	else $r = cfg::$Templates[$Name];

	return $r;

}


function Template_Read($FileSpec, $IsBase = null) {
	/*
	Purpose
		Used to read and parse a template file.
	Parameters
		FileSpec (string) = The filespec of the template file we want to read.
		IsBase (optional boolean) = Used to determine if this is a base template.
			If true, this is a base template.
			If false, this is not a base template.
			Default: false
	Return (string)
		If IsBase is true, this is a base template. Therefore, the body is not stripped out.
			Instead, the entire file is returned as is.
		If IsBase is false, this is not a base template. Therefore, the body is stripped out.
			Only the body of the file is returned.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Config
	$DELIMITER_BEGIN = '<body>';
	$DELIMITER_END = '</body>';

	// Default parameters
	if (is_null($IsBase)) $IsBase = false;								// IsBase defaults to false

	// Read the template file
	$File = file_get_contents($FileSpec);

	// This is a base template. No need to parse the body out. Just return the entire file as is.
	if ($IsBase) return $File;

	// This is not a base template. Parse out the template portion of the file (the body).
	$x = strpos($File, $DELIMITER_BEGIN) + strlen($DELIMITER_BEGIN);	// Get the start-cut position
	$File = substr($File, $x);											// Cut the start delimiter off
	$x = strpos($File, $DELIMITER_END) - 1;								// Get the end-cut position
	$File = substr($File, 0, $x);										// Cut the end delimiter off

	return $File;

}


function Template_Read_Ex($FileSpec, $IsBase = null) {
	/*
	Purpose
		Used to read and parse a template file.
	Parameters
		FileSpec (string) = The filespec of the template file we want to read.
		IsBase (optional boolean) = Used to determine if this is a base template.
			If true, this is a base template.
			If false, this is not a base template.
			Default: false
	Return (string)
		If IsBase is true, this is a base template. Therefore, the body is not stripped out.
			Instead, the entire file is returned as is.
		If IsBase is false, this is not a base template. Therefore, the body is stripped out.
			Only the body of the file is returned.
	Notes
		If the opening <body> tag contains attributes, this function still parses properly.
	Version History
		1.00 2009.6.16 by Russ Tanner
	*/

	// Config
	$DELIMITER_BEGIN = '/<body.*>/i';						// This is a regexp.
	$DELIMITER_END = '</body>';								// This is just a string.

	// Default parameters
	if (is_null($IsBase)) $IsBase = false;					// IsBase defaults to false

	// Read the template file
	$File = file_get_contents($FileSpec);

	// This is a base template. No need to parse the body out. Just return the entire file as is.
	if ($IsBase) return $File;

	// This is not a base template. Parse out the template portion of the file (the body).
	$Matches = '';											// This will be loaded with an array by preg_match();
	$e = preg_match($DELIMITER_BEGIN, $File, $Matches);		// We use a regexp in case <body> has attributes ("onload=")
	// Error Trap: An error occured in preg_match(). Display a bug trap.
	if ($e === false) {
		SayPage__BugTrap('In Template_Read(), preg_match() returned false');
		exit(1);
	}
	// Error Trap: No <body> tag was located in the template. Display a but trap.
	if ($e == 0) {
		SayPage__BugTrap('In Template_Read(), no BODY tag was located within a template.<br />' . htmlspecialchars($FileSpec));
		exit(1);
	}
	
	$x = strpos($File, $Matches[0]) + strlen($Matches[0]);				// Get a pointer to the end of the <body> tag, plus 1.
	$File = substr($File, $x);											// Cut the start delimiter off
	$x = strpos($File, $DELIMITER_END) - 1;								// Get the end-cut position
	$File = substr($File, 0, $x);										// Cut the end delimiter off

	return $File;

}


function Template_Process($FileSpec, $Tokens, $IsBase = null) {
	/*
	Purpose
		Used to process all of the tokens in a template.
	Parameters
		FileSpec (string) = The filespec of the template we're using.
		Tokens (associative array) = A list of tokens that we're expanding.
		IsBase (optional boolean) = Used to determine if this is a base template.
			If true, this is a base template.
			If false, this is not a base template.
			Default: false
	Return (string, null)
		Returns the specified template with all specified tokens expanded.
		If IsBase is true, then the entire template file is processed, not just the body.
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	// Default parameters
	if (is_null($IsBase)) $IsBase = false;								// IsBase defaults to false

	// Read and parse the body of the template
	$Template = Template_Read($FileSpec, $IsBase);

	// Expand all of the specified tokens
	$r = ExpandTokens($Template, $Tokens);

	return $r;

}


function Tokens_Encode_HTML($Tokens) {
	/*
	Purpose
		Used to HTML-encode a list of tokens.
		This is done to enhance security related to user-submitted data.
	Parameters
		Tokens (array) = The list of string we want to encode.
	Return (array)
		Returns the specified array (Tokens) with each element encoded with htmlspecialchars().
	Version History
		1.00 2009.7.7 Tested by Russ Tanner on PHP 5.1.6.
	*/

	$r = array();		// Return buffer

	foreach($Tokens as $k => $v) $r[$k] = htmlspecialchars($v, cfg::$QuoteStyle, cfg::$CharSet);

	return $r;

}



// Main //


// Begin a session.
session_start();

// Initialize global objects.
cfg::Init();		// The global configuration object.

// Save select form fields to the session.
Fields_SavePersistent();


// Map form submit buttons to standard commands...

// "SaveList"
if (Parm_Exists('Submit_SaveList_SubjectList')) {
	$do = 'SaveList';
	$_POST['q'] = 'SubjectList';
}
elseif (Parm_Exists('Submit_SaveList_BodyList')) {
	$do = 'SaveList';
	$_POST['q'] = 'BodyList';
}
elseif (Parm_Exists('Submit_SaveList_AuthorList')) {
	$do = 'SaveList';
	$_POST['q'] = 'AuthorList';
}
elseif (Parm_Exists('Submit_SaveList_IPList')) {
	$do = 'SaveList';
	$_POST['q'] = 'IPList';
}

// "DeleteComments"
elseif (Parm_Exists('Submit_DeleteComments_SubjectList')) {
	$do = 'DeleteComments';
	$_POST['q'] = 'SubjectList';
}
elseif (Parm_Exists('Submit_DeleteComments_BodyList')) {
	$do = 'DeleteComments';
	$_POST['q'] = 'BodyList';
}
elseif (Parm_Exists('Submit_DeleteComments_AuthorList')) {
	$do = 'DeleteComments';
	$_POST['q'] = 'AuthorList';
}
elseif (Parm_Exists('Submit_DeleteComments_IPList')) {
	$do = 'DeleteComments';
	$_POST['q'] = 'IPList';
}

// "DeleteBatch"
elseif (Parm_Exists('Submit_DeleteBatch_SubjectList')) {
	$do = 'DeleteBatch';
	$_POST['q'] = 'SubjectList';
}
elseif (Parm_Exists('Submit_DeleteBatch_BodyList')) {
	$do = 'DeleteBatch';
	$_POST['q'] = 'BodyList';
}
elseif (Parm_Exists('Submit_DeleteBatch_AuthorList')) {
	$do = 'DeleteBatch';
	$_POST['q'] = 'AuthorList';
}
elseif (Parm_Exists('Submit_DeleteBatch_IPList')) {
	$do = 'DeleteBatch';
	$_POST['q'] = 'IPList';
}

// "DeleteCommentsAll"
elseif (Parm_Exists('Submit_DeleteComments_All')) {
	$do = 'DeleteCommentsAll';
}

// "FindComments"
elseif (Parm_Exists('Submit_FindComments')) {
	$do = 'FindComments';
	$_POST['column'] = GetInput('FindComments_Column');
	$_POST['value'] = GetInput('FindComments_Value');
	$_POST['max'] = GetInput('FindComments_Max');
}

// "FindDuplicates"
elseif (Parm_Exists('Submit_FindDuplicates')) {
	$do = 'FindDuplicates';
	$_POST['column'] = GetInput('FindDuplicates_Column');
	$_POST['value'] = GetInput('FindDuplicates_Value');
	$_POST['count'] = GetInput('FindDuplicates_Count');
	$_POST['max'] = GetInput('FindDuplicates_Max');
}


// A standard form submit button was not clicked.
else {
	
	// Transfer parameters into local convenience variables.
	$do = GetInput('do');

	// "do" is not defined. We default to displaying a page.
	if (is_null($do)) $do = 'SayPage';

}


// Command Processor //

if ($do == 'SayPage') h_SayPage();
elseif ($do == 'ClearList') h_ClearList();
elseif ($do == 'SaveList') h_SaveList();
elseif ($do == 'SortList') h_SortList();
elseif ($do == 'RemoveDuplicates') h_RemoveDuplicates();
elseif ($do == 'AutoDetect') h_AutoDetect();
elseif ($do == 'RestoreDefaults') h_RestoreDefaults();
elseif ($do == 'DeleteComments') h_DeleteComments();
elseif ($do == 'DeleteBatch') h_DeleteBatch_Begin();
elseif ($do == 'DeleteCommentsAll') h_DeleteComments_All();
elseif ($do == '_DeleteBatch') h_DeleteBatch();
elseif ($do == 'FindComments') h_FindComments();
elseif ($do == 'FindCommentsDelete') h_FindComments_Delete();
elseif ($do == 'FindDuplicates') h_FindDuplicates();
elseif ($do == 'FindDuplicatesDelete') h_FindDuplicates_Delete();

// Error: Invalid Parameters
else SayPage__Error('Invalid Parameter: "do"');


?>
